@@ -758,11 +758,7 @@ static GtkWidget * button_build(MainInfo *min, Button *btn, gboolean partial, GC
 		gboolean	set = FALSE;
 		BtnFace		i;
 
-#if GTK_CHECK_VERSION(2, 2, 0)
 		gtk_widget_set_can_focus(but, FALSE);
-#else
-		GTK_WIDGET_UNSET_FLAGS(but, GTK_CAN_FOCUS);
-#endif
 		g_object_set_data(G_OBJECT(but), "min", min);
 		g_object_set_data(G_OBJECT(but), "button", btn);
 
@@ -795,13 +791,13 @@ static GtkWidget * buttonrow_build(MainInfo *min, ButtonRow *brw, gboolean parti
 	GtkWidget	*hbox, *hbox2 = NULL, *btn;
 	guint		i;
 
-	hbox = gtk_hbox_new(FALSE, 0);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 
 	for(i = 0; i < brw->width;)
 	{
 		if(btn_button_get_flags_boolean(&brw->button[i], BTF_NARROW))
 		{
-			hbox2 = gtk_hbox_new(FALSE, 0);
+			hbox2 = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 			for(; i < brw->width && btn_button_get_flags_boolean(&brw->button[i], BTF_NARROW); i++)
 			{
 				if((btn = button_build(min, brw->button + i, partial, func, user)) != NULL)
@@ -816,7 +812,8 @@ static GtkWidget * buttonrow_build(MainInfo *min, ButtonRow *brw, gboolean parti
 		}
 		else
 		{
-			hbox2 = gtk_hbox_new(TRUE, 0);
+			hbox2 = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
+			gtk_box_set_homogeneous(GTK_BOX(hbox2), TRUE);
 			for(; i < brw->width && !btn_button_get_flags_boolean(&brw->button[i], BTF_NARROW); i++)
 			{
 				if((btn = button_build(min, brw->button + i, partial, func, user)) != NULL)
@@ -1137,7 +1134,7 @@ static GtkWidget * buttonsheet_build(MainInfo *min, ButtonSheet *bsh, gboolean p
 		return NULL;
 
 	/* First, stack all rows in a vbox. */
-	vbox = gtk_vbox_new(FALSE, 0);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	for(iter = bsh->rows; iter != NULL; iter = g_list_next(iter))
 	{
 		if((row = buttonrow_build(min, iter->data, partial, func, user)) != NULL)
@@ -1148,7 +1145,7 @@ static GtkWidget * buttonsheet_build(MainInfo *min, ButtonSheet *bsh, gboolean p
 	{
 		scbox = gtk_scrolled_window_new(NULL, NULL);
 		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scbox), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-		gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scbox), vbox);
+		gtk_container_add(GTK_CONTAINER(scbox), vbox);
 		return scbox;
 	}
 	return vbox;
@@ -1204,14 +1201,14 @@ static void btn_callback(GtkWidget *wid, gpointer user)
 				kbd_context_entry_add(min->gui->kbd_ctx, key, KET_CMDSEQ, cmd);
 		}
 	}
-	else if(GTK_IS_HBOX(wid))
+	else if(GTK_IS_BOX(wid))
 		gtk_container_foreach(GTK_CONTAINER(wid), btn_callback, user);
 }
 
 /* 1999-03-11 -	This gets called once for every row in a sheet. */
 static void row_callback(GtkWidget *wid, gpointer user)
 {
-	if(GTK_IS_HBOX(wid))
+	if(GTK_IS_BOX(wid))
 		gtk_container_foreach(GTK_CONTAINER(wid), btn_callback, user);
 	else
 		fprintf(stderr, "**BUTTONS: Unexpected non-hbox widget found!\n");
