@@ -69,7 +69,7 @@ static GtkWidget * init(MainInfo *min, gchar **name)
 	guint		i;
 	GSList		*group = NULL;
 
-	page->vbox = gtk_vbox_new(FALSE, 0);
+	page->vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	page->edit = NULL;
 
 	label = gtk_label_new(_("This page lets you control how the Shortcuts button sheet is positioned relative to\n"
@@ -86,7 +86,7 @@ static GtkWidget * init(MainInfo *min, gchar **name)
 	gtk_box_pack_start(GTK_BOX(page->vbox), label, FALSE, FALSE, 5);
 
 	frame = gtk_frame_new(_("Shortcut Sheet Position"));
-	vbox  = gtk_vbox_new(FALSE, 0);
+	vbox  = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	for(i = 0; i < sizeof plab / sizeof *plab; i++)
 	{
 		page->pos[i] = gtk_radio_button_new_with_label(group, _(plab[i]));
@@ -100,7 +100,7 @@ static GtkWidget * init(MainInfo *min, gchar **name)
 	gtk_box_pack_start(GTK_BOX(page->vbox), frame, FALSE, FALSE, 0);
 
 	frame = gtk_frame_new(_("Separation Style"));
-	vbox  = gtk_vbox_new(FALSE, 0);
+	vbox  = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	for(i = 0, group = NULL; i < sizeof slab / sizeof *slab; i++)
 	{
 		page->sep[i] = gtk_radio_button_new_with_label(group, _(slab[i]));
