@@ -112,7 +112,7 @@ static void set_blocked_entry(GtkWidget *entry, const gchar *str, guint sig)
 /* 1998-09-15 -	Reset all widgets to their most idle states. */
 static void reset_widgets(P_Buttons *page)
 {
-	BtnFace	i;
+	gint	i;
 
 	page->button = NULL;
 	for(i = 0; i < BTN_FACES; i++)
@@ -171,13 +171,13 @@ static void set_widgets(P_Buttons *page)
 /* 1998-09-15 -	Populate the scrolled window with a button sheet. */
 static void populate_scwin(P_Buttons *page)
 {
-	if(page->shvbox != NULL)
-		gtk_widget_destroy(page->shvbox);
+	if(page->shvbox != NULL)	/* The shvbox is made scrollable by gtk_container_add(), which inserts an extra parent. Hm. */
+		gtk_container_remove(GTK_CONTAINER(page->scwin), gtk_widget_get_parent(page->shvbox));
 	page->shvbox = NULL;
 
 	if((page->shvbox = btn_buttonsheet_build(page->min, &page->buttons, page->cur_sheet, TRUE, G_CALLBACK(evt_button_clicked), page)) != NULL)
 	{
-		gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(page->scwin), page->shvbox);
+		gtk_container_add(GTK_CONTAINER(page->scwin), page->shvbox);
 		gtk_widget_show_all(page->shvbox);
 	}
 	reset_widgets(page);
@@ -290,17 +290,17 @@ static void width_dialog(const gchar *text, gint width, void (*func)(gpointer us
 	GtkWidget	*vbox, *label, *spin;
 	GtkAdjustment	*adj;
 
-	vbox = gtk_vbox_new(FALSE, 0);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	label = gtk_label_new(text);
 	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
 
 	spin = gtk_spin_button_new(NULL, 1, 0);
 	if((adj = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(spin))) != NULL)
 	{
-		adj->lower = 1.0f;
-		adj->upper = G_MAXINT;		/* When it comes to arbitrary limits, this is a good one. */
-		adj->step_increment = 1;
-		adj->page_increment = 16;
+		gtk_adjustment_set_lower(adj, 1.0f);
+		gtk_adjustment_set_upper(adj, G_MAXINT);		/* When it comes to arbitrary limits, this is a good one. */
+		gtk_adjustment_set_step_increment(adj, 1);
+		gtk_adjustment_set_page_increment(adj, 16);
 		gtk_spin_button_set_adjustment(GTK_SPIN_BUTTON(spin), adj);
 	}
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin), width);
@@ -394,7 +394,6 @@ static gint evt_dtooltip_changed(GtkWidget *wid, gpointer user)
 			page->modified = TRUE;
 		}
 	}
-
 	return TRUE;
 }
 
@@ -413,7 +412,6 @@ static gint evt_dnarrow_clicked(GtkWidget *wid, gpointer user)
 		set_mode_normal(page);
 		reset_widgets(page);
 		populate_scwin(page);
-/*		set_mode_selected(page, */
 	}
 	return TRUE;
 }
@@ -577,13 +575,15 @@ static void evt_key_clear_clicked(GtkWidget *wid, gpointer user)
 }
 
 /* 1999-05-02 -	This gets called by cdl_dialog() (the color_dialog module) when the bg color changes. */
-static void col_bg_changed(GdkColor *color, gpointer user)
+static void col_bg_changed(const GdkRGBA *color, gpointer user)
 {
 	P_Buttons	*page = user;
 	BtnFace		face;
+	GdkColor	color_old;
 
 	face = page->col_change_face;
-	btn_button_set_color_bg_widget(page->button, face, color);
+	gui_color_from_rgba(&color_old, color);
+	btn_button_set_color_bg_widget(page->button, face, &color_old);
 }
 
 /* 1999-05-02 -	User clicked the background color multi-faced button. Either bring up a dialog,
@@ -600,17 +600,21 @@ static void evt_col_bg_clicked(GtkWidget *wid, gpointer user)
 	{
 		if(function == BTN_PRIMARY)	/* Dialog-time? */
 		{
-			GdkColor	old, *op = NULL;
+			GdkColor	old_old;
+			GdkRGBA		old, *op = NULL;
 			guint		res;
 
-			if(btn_button_get_color_bg(btn, tface, &old))
+			if(btn_button_get_color_bg(btn, tface, &old_old))
+			{
+				gui_rgba_from_color(&old, &old_old);
 				op = &old;
+			}
 			page->col_change_face = tface;
 			res = cdl_dialog_sync_new_wait(_("Edit Background Color"), col_bg_changed, op, page);
 			if(res == DLG_POSITIVE)
 				page->modified = TRUE;
-			else
-				btn_button_set_color_bg_widget(page->button, tface, op);
+			else if(op != NULL)
+				btn_button_set_color_bg_widget(page->button, tface, &old_old);
 		}
 		else		/* Time to reset color to system default. */
 		{
@@ -621,13 +625,15 @@ static void evt_col_bg_clicked(GtkWidget *wid, gpointer user)
 }
 
 /* 1999-05-02 -	This gets called by cdl_dialog() (the color_dialog module) when the fg color changes. */
-static void col_fg_changed(GdkColor *color, gpointer user)
+static void col_fg_changed(const GdkRGBA *color, gpointer user)
 {
 	P_Buttons	*page = user;
 	BtnFace		face;
+	GdkColor	color_old;
 
 	face = page->col_change_face;
-	btn_button_set_color_fg_widget(page->button, face, color);
+	gui_color_from_rgba(&color_old, color);
+	btn_button_set_color_fg_widget(page->button, face, &color_old);
 }
 
 /* 2008-11-28 -	User clicked the foreground color multi-faced button. Either bring up a dialog,
@@ -644,17 +650,21 @@ static void evt_col_fg_clicked(GtkWidget *wid, gpointer user)
 	{
 		if(function == BTN_PRIMARY)	/* Dialog-time? */
 		{
-			GdkColor	old, *op = NULL;
+			GdkColor	old_old;
+			GdkRGBA		old, *op = NULL;
 			guint		res;
 
-			if(btn_button_get_color_fg(btn, tface, &old))
+			if(btn_button_get_color_fg(btn, tface, &old_old))
+			{
+				gui_rgba_from_color(&old, &old_old);
 				op = &old;
+			}
 			page->col_change_face = tface;
 			res = cdl_dialog_sync_new_wait(_("Edit Foreground Color"), col_fg_changed, op, page);
 			if(res == DLG_POSITIVE)
 				page->modified = TRUE;
-			else
-				btn_button_set_color_fg_widget(page->button, tface, op);
+			else if(op != NULL)
+				btn_button_set_color_fg_widget(page->button, tface, &old_old);
 		}
 		else		/* Time to reset color to system default. */
 		{
@@ -670,48 +680,48 @@ static void evt_col_fg_clicked(GtkWidget *wid, gpointer user)
 */
 static void init_bdef(MainInfo *min, P_Buttons *page, BDef *def, const gchar *lab, BtnFace face)
 {
-	GtkWidget	*table, *label, *btn, *img, *hbox;
+	GtkWidget	*grid, *label, *btn, *hbox;
 
 	def->frame = gtk_frame_new(lab);
-	table = gtk_table_new(4, 3, FALSE);
+	grid = gtk_grid_new();
 
 	label = gtk_label_new(_("Label"));
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,  0,0,0,0);
+	gtk_grid_attach(GTK_GRID(grid), label, 0, 0, 1, 1);
 	def->label = gtk_entry_new();
+	gtk_widget_set_hexpand(def->label, TRUE);
+	gtk_widget_set_halign(def->label, GTK_ALIGN_FILL);
 	gtk_entry_set_max_length(GTK_ENTRY(def->label), BTN_LABEL_SIZE - 1);
 	g_object_set_data(G_OBJECT(def->label), "face", GUINT_TO_POINTER(face));
 	def->label_sig = g_signal_connect(G_OBJECT(def->label), "changed", G_CALLBACK(evt_label_changed), page);
-	gtk_table_attach(GTK_TABLE(table), def->label, 1, 3, 0, 1,  GTK_EXPAND|GTK_FILL,0,0,0);
+	gtk_grid_attach(GTK_GRID(grid), def->label, 1, 0, 2, 1);
 
 	label = gtk_label_new(_("Command"));
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,  0,0,0,0);
+	gtk_grid_attach(GTK_GRID(grid), label, 0, 1, 1, 1);
 	def->cmdseq = gtk_entry_new();
 	g_object_set_data(G_OBJECT(def->cmdseq), "face", GUINT_TO_POINTER(face));
 	def->cmdseq_sig = g_signal_connect(G_OBJECT(def->cmdseq), "changed", G_CALLBACK(evt_cmdseq_changed), page);
-	gtk_table_attach(GTK_TABLE(table), def->cmdseq, 1, 2, 1, 2,  GTK_EXPAND|GTK_FILL,0,0,0);
+	gtk_grid_attach(GTK_GRID(grid), def->cmdseq, 1, 1, 1, 1);
 	btn = gui_details_button_new();
 	g_object_set_data(G_OBJECT(btn), "face", GUINT_TO_POINTER(face));
 	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(evt_cmdseq_pick_clicked), page);
-	gtk_table_attach(GTK_TABLE(table), btn, 2, 3, 1, 2,  GTK_FILL,0,0,0);
+	gtk_grid_attach(GTK_GRID(grid), btn, 2, 1, 1, 1);
 
 	label = gtk_label_new(_("Key"));
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,  0,0,0,0);
+	gtk_grid_attach(GTK_GRID(grid), label, 0, 2, 1, 1);
 	def->key = gtk_entry_new();
 	gtk_editable_set_editable(GTK_EDITABLE(def->key), FALSE);
 	g_object_set_data(G_OBJECT(def->key), "face", GUINT_TO_POINTER(face));
 	g_signal_connect(G_OBJECT(def->key), "key_press_event", G_CALLBACK(evt_key_press), page);
-	gtk_table_attach(GTK_TABLE(table), def->key, 1, 2, 2, 3,  GTK_EXPAND|GTK_FILL,0,0,0);
+	gtk_grid_attach(GTK_GRID(grid), def->key, 1, 2, 1, 1);
 
-	btn = gtk_button_new();
-	img = gtk_image_new_from_stock(GTK_STOCK_CLEAR, GTK_ICON_SIZE_MENU);
-	gtk_button_set_image(GTK_BUTTON(btn), img);
+	btn = gtk_button_new_from_icon_name("edit-clear", GTK_ICON_SIZE_MENU);
 	g_object_set_data(G_OBJECT(btn), "face", GUINT_TO_POINTER(face));
 	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(evt_key_clear_clicked), page);
-	gtk_table_attach(GTK_TABLE(table), btn, 2, 3, 2, 3,  GTK_FILL,0,0,0);
+	gtk_grid_attach(GTK_GRID(grid), btn, 2, 2, 1, 1);
 
 	label = gtk_label_new(_("Colors"));
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 3, 4,  0,0,0,0);
-	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_grid_attach(GTK_GRID(grid), label, 0, 3, 1, 1);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	def->col_bg = od_multibutton_new();
 	od_multibutton_set_text(OD_MULTIBUTTON(def->col_bg), 0, _("Background..."),    NULL, NULL, GUINT_TO_POINTER(face));
 	od_multibutton_set_text(OD_MULTIBUTTON(def->col_bg), 1, _("Reset to Default"), NULL, NULL, GUINT_TO_POINTER(face));
@@ -724,10 +734,9 @@ static void init_bdef(MainInfo *min, P_Buttons *page, BDef *def, const gchar *la
 	g_object_set_data(G_OBJECT(def->col_fg), "face", GINT_TO_POINTER(0));
 	g_signal_connect(G_OBJECT(def->col_fg), "clicked", G_CALLBACK(evt_col_fg_clicked), page);
 	gtk_box_pack_start(GTK_BOX(hbox), def->col_fg, TRUE, TRUE, 0);
+	gtk_grid_attach(GTK_GRID(grid), hbox, 1, 3, 3, 1);
 
-	gtk_table_attach(GTK_TABLE(table), hbox, 1, 3, 3, 4,  GTK_EXPAND|GTK_FILL,0,0,0);
-
-	gtk_container_add(GTK_CONTAINER(def->frame), table);
+	gtk_container_add(GTK_CONTAINER(def->frame), grid);
 }
 
 static void evt_sheet_activate(GtkWidget *wid, guint index, gpointer user)
@@ -752,12 +761,12 @@ static GtkWidget * cbt_init(MainInfo *min, gchar **name)
 	guint		i;
 	gint		expand;
 
-	page->vbox = gtk_vbox_new(FALSE, 0);
+	page->vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	page->min  = min;
 	page->button = NULL;
 	page->cur_sheet = NULL;
 
-	hbox = gtk_hbox_new(FALSE, 0);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	label = gtk_label_new(_("Sheet"));
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 	optm = gui_build_combo_box(mlab, G_CALLBACK(evt_sheet_activate), page);
@@ -769,16 +778,16 @@ static GtkWidget * cbt_init(MainInfo *min, gchar **name)
 	gtk_box_pack_start(GTK_BOX(page->vbox), page->scwin, TRUE, TRUE, 0);
 	page->shvbox = NULL;
 
-	page->dvbox = gtk_vbox_new(FALSE, 0);
+	page->dvbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 
-	page->dhbox = gtk_hbox_new(FALSE, 0);
+	page->dhbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	init_bdef(min, page, &page->def[0], _("Primary"), BTN_PRIMARY);
 	gtk_box_pack_start(GTK_BOX(page->dhbox), page->def[0].frame, TRUE, TRUE, 0);
 	init_bdef(min, page, &page->def[1], _("Secondary"), BTN_SECONDARY);
 	gtk_box_pack_start(GTK_BOX(page->dhbox), page->def[1].frame, TRUE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(page->dvbox), page->dhbox, TRUE, TRUE, 0);
 
-	hbox = gtk_hbox_new(FALSE, 0);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	label = gtk_label_new(_("Tooltip"));
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 	page->dtooltip = gtk_entry_new();
@@ -788,7 +797,7 @@ static GtkWidget * cbt_init(MainInfo *min, gchar **name)
 	gtk_box_pack_start(GTK_BOX(page->dvbox), hbox, FALSE, FALSE, 0);
 
 	frame = gtk_frame_new(_("Flags"));
-	hbox = gtk_hbox_new(FALSE, 0);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	page->dnarrow = gtk_check_button_new_with_label(_("Narrow?"));
 	page->dnarsig = g_signal_connect(G_OBJECT(page->dnarrow), "clicked", G_CALLBACK(evt_dnarrow_clicked), page);
 	gtk_box_pack_start(GTK_BOX(hbox), page->dnarrow, TRUE, TRUE, 0);
@@ -801,7 +810,7 @@ static GtkWidget * cbt_init(MainInfo *min, gchar **name)
 	gtk_box_pack_start(GTK_BOX(page->dvbox), frame, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(page->vbox), page->dvbox, FALSE, FALSE, 0);
 
-	hbox = gtk_hbox_new(FALSE, 0);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	for(i = 0; i < sizeof page->bcmd / sizeof page->bcmd[0]; i++)
 	{
 		page->bcmd[i] = gtk_button_new_with_label(_(blab[i]));
@@ -811,14 +820,14 @@ static GtkWidget * cbt_init(MainInfo *min, gchar **name)
 	}
 	gtk_box_pack_start(GTK_BOX(page->vbox), hbox, FALSE, FALSE, 5);
 
-	sep = gtk_hseparator_new();
+	sep = gtk_separator_new(GTK_ORIENTATION_HORIZONTAL);
 	gtk_box_pack_start(GTK_BOX(page->vbox), sep, FALSE, FALSE, 0);
 
-	hbox = gtk_hbox_new(FALSE, 0);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	for(i = 0; i < sizeof page->rcmd / sizeof page->rcmd[0]; i++)
 	{
 		if(i == 2 || i == 3)
-			page->rcmd[i] = gui_arrow_button_new(i == 2 ? GTK_ARROW_UP : GTK_ARROW_DOWN);
+			page->rcmd[i] = gtk_button_new_from_icon_name(i == 2 ? "go-up" : "go-down", GTK_ICON_SIZE_MENU);
 		else
 			page->rcmd[i] = gtk_button_new_with_label(_(rlab[i]));
 		g_object_set_data(G_OBJECT(page->rcmd[i]), "index", GINT_TO_POINTER(i));
