@@ -34,21 +34,25 @@ typedef struct {		/* External command general flags. */
 } PXE_General;
 
 typedef struct {
+	GtkWidget	*vbox;
 	GtkWidget	*hbox;
-	GtkWidget	*bframe;
 	GtkWidget	*reqsel_src;
 	GtkWidget	*reqsel_dst;
 	GtkWidget	*cdsrc;
 	GtkWidget	*cddst;
-	GtkWidget	*aframe;
+} PXE_BF;
+
+typedef struct {
+	GtkWidget	*vbox;
 	GtkWidget	*rssrc;
 	GtkWidget	*rsdst;
-} PXE_BAF;
+} PXE_AF;
 
 typedef struct {		/* Extra widgetry for external commands. Plenty of flags. */
 	GtkWidget	*nbook;		/* A notebook. */
 	PXE_General	gflags;		/* General flags page. */
-	PXE_BAF		baf;
+	PXE_BF		bf;
+	PXE_AF		af;
 } PX_External;
 
 typedef struct {
@@ -334,25 +338,25 @@ static void set_cx_g_flags(PXE_General *pxeg, guint32 flags)
 /* 1999-03-09 -	Set visual state of before/after checkbuttons, according to <flags>. Sets for
 **		the before group if <after> is FALSE, otherwise for the after group. Geddit?
 */
-static void set_cx_ba_flags(PXE_BAF *pxbaf, guint after, guint32 flags)
+static void set_cx_ba_flags(PXE_BF *pxbf, PXE_AF *pxaf, guint after, guint32 flags)
 {
 	if(!after)		/* Set "before" type flags, i.e. CD source XOR dest. */
 	{
 		guint	st_src = FALSE, st_dst = FALSE;		/* Guess a few states. */
 
-		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pxbaf->reqsel_src), (flags & CBAF_REQSEL_SOURCE) != 0);
-		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pxbaf->reqsel_dst), (flags & CBAF_REQSEL_DEST) != 0);
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pxbf->reqsel_src), (flags & CBAF_REQSEL_SOURCE) != 0);
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pxbf->reqsel_dst), (flags & CBAF_REQSEL_DEST) != 0);
 		if(flags & CBAF_CD_SOURCE)
 			st_src = TRUE;
 		else if(flags & CBAF_CD_DEST)
 			st_dst = TRUE;
-		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pxbaf->cdsrc), st_src);
-		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pxbaf->cddst), st_dst);
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pxbf->cdsrc), st_src);
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pxbf->cddst), st_dst);
 	}
 	else			/* Set "after" flags, i.e. rescan source IOR dest. */
 	{
-		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pxbaf->rssrc), (flags & CBAF_RESCAN_SOURCE) != 0);
-		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pxbaf->rsdst), (flags & CBAF_RESCAN_DEST) != 0);
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pxaf->rssrc), (flags & CBAF_RESCAN_SOURCE) != 0);
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pxaf->rsdst), (flags & CBAF_RESCAN_DEST) != 0);
 	}
 }
 
@@ -369,8 +373,8 @@ static void set_cx_external_widgets(P_CmdSeq *page)
 
 	set_cx_g_flags(&page->px_external.gflags, ext->gflags);
 
-	set_cx_ba_flags(&page->px_external.baf, FALSE, ext->baflags[0]);
-	set_cx_ba_flags(&page->px_external.baf, TRUE,  ext->baflags[1]);
+	set_cx_ba_flags(&page->px_external.bf, &page->px_external.af, FALSE, ext->baflags[0]);
+	set_cx_ba_flags(&page->px_external.bf, &page->px_external.af, TRUE,  ext->baflags[1]);
 }
 
 /* 1998-09-27 -	Set up widgets for row, assuming there is a row selected. */
@@ -452,7 +456,6 @@ static void evt_addrow_clicked(GtkWidget *wid, gpointer user)
 	CmdSeq		*cs;
 	CmdRow		*nr;
 	CRType		type;
-	gint		pos;
 
 	if((cs = cmdseq_get_selected(page, NULL)) == NULL)
 		return;
@@ -466,7 +469,7 @@ static void evt_addrow_clicked(GtkWidget *wid, gpointer user)
 		GtkTreeIter	iter;
 		GtkTreePath	*path;
 
-		pos = csq_cmdseq_row_append(cs, nr);
+		csq_cmdseq_row_append(cs, nr);
 		gtk_list_store_append(page->dstore, &iter);
 		gtk_list_store_set(page->dstore, &iter,
 					DEF_COLUMN_TYPE, csq_cmdrow_type_to_string(nr->type),
@@ -632,12 +635,14 @@ static void pick_external(P_CmdSeq *page)
 					"fpu",	N_("First selected, with path, unselect"),
 					"fd",	N_("First selected (destination pane)"),
 					"fQ",	N_("First selected, no quotes"),
+					"fE",	N_("First selected, no extension"),
 					"F",	N_("All selected"),
 					"Fu",	N_("All selected, unselect"),
 					"Fp",	N_("All selected, with paths"),
 					"Fpu",	N_("All selected, with paths, unselect"),
 					"Fd",	N_("All selected (destination pane)"),
 					"FQ",	N_("All selected, no quotes"),
+					"FE",	N_("All selected, no extensions"),
 					"Ps",	N_("Path to source pane's directory"),
 					"Pd",	N_("Path to destination pane's directory"),
 					"Ph",	N_("Path to home directory"),
@@ -774,7 +779,7 @@ static void evt_del_clicked(GtkWidget *wid, gpointer user)
 /* 1998-09-26 -	Build the (currently very pointless) extra controls for builtin rows. */
 static void build_px_builtin(P_CmdSeq *page, PX_Builtin *pxb)
 {
-	pxb->vbox  = gtk_vbox_new(FALSE, 0);
+	pxb->vbox  = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	pxb->label = gtk_label_new(_("(No Options Available)"));
 	gtk_box_pack_start(GTK_BOX(pxb->vbox), pxb->label, TRUE, TRUE, 0);
 }
@@ -792,7 +797,7 @@ static void evt_gf_clicked(GtkWidget *wid, gpointer user)
 
 	if(row->type == CRTP_EXTERNAL)
 	{
-		if(GTK_TOGGLE_BUTTON(wid)->active)
+		if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wid)))
 			row->extra.external.gflags |= flags;
 		else
 			row->extra.external.gflags &= ~flags;
@@ -808,9 +813,9 @@ static void build_pxe_general(P_CmdSeq *page, PXE_General *pxg)
 {
 	GtkWidget	*hbox;
 
-	pxg->vbox = gtk_vbox_new(FALSE, 0);
+	pxg->vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 
-	hbox = gtk_hbox_new(FALSE, 0);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	pxg->runbg = gtk_check_button_new_with_label(_("Run in Background?"));
 	g_object_set_data(G_OBJECT(pxg->runbg), "user", GINT_TO_POINTER(CGF_RUNINBG));
 	g_signal_connect(G_OBJECT(pxg->runbg), "clicked", G_CALLBACK(evt_gf_clicked), page);
@@ -867,12 +872,12 @@ static void evt_baf_cd_clicked(GtkWidget *wid, gpointer user)
 	/* Figure out if we're in source or dest (same handler). */
 	if(g_object_get_data(G_OBJECT(wid), "destination"))
 	{
-		other = page->px_external.baf.cdsrc;
+		other = page->px_external.bf.cdsrc;
 		mask  = CBAF_CD_DEST;
 	}
 	else
 	{
-		other = page->px_external.baf.cddst;
+		other = page->px_external.bf.cddst;
 		mask  = CBAF_CD_SOURCE;
 	}
 	if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wid)))
@@ -902,50 +907,52 @@ static void evt_baf_rescan_clicked(GtkWidget *wid, gpointer user)
 
 /* 1999-03-09 -	Build new simplified widgetry for editing before and after flags. Note that the actual
 **		flags have not changed, they're still muy redundant-capable. It's just the GUI that's
-**		become a bit simpler to look at. At less vertically tall.
+**		become a bit simpler to look at. At least less vertically tall.
 */
-static void build_px_baf(P_CmdSeq *page, PXE_BAF *pxbaf)
+static void build_px_bf(P_CmdSeq *page, PXE_BF *pxbf)
 {
-	GtkWidget	*vbox, *hbox;
+	GtkWidget	*grid;
 
-	pxbaf->hbox = gtk_hbox_new(FALSE, 0);
+	pxbf->vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 
-	pxbaf->bframe = gtk_frame_new(_("Before Execution"));
-	vbox = gtk_vbox_new(FALSE, 0);
+	grid = gtk_grid_new();
+	gtk_grid_set_row_homogeneous(GTK_GRID(grid), TRUE);
+	gtk_grid_set_column_homogeneous(GTK_GRID(grid), TRUE);
+	pxbf->reqsel_src = gtk_check_button_new_with_label(_("Require Source Selection?"));
+	g_object_set_data(G_OBJECT(pxbf->reqsel_src), "user", GINT_TO_POINTER(CBAF_REQSEL_SOURCE));
+	g_signal_connect(G_OBJECT(pxbf->reqsel_src), "clicked", G_CALLBACK(evt_baf_reqsel_clicked), page);
+	gtk_grid_attach(GTK_GRID(grid), pxbf->reqsel_src, 0, 0, 1, 1);
+	pxbf->reqsel_dst = gtk_check_button_new_with_label(_("Require Destination Selection?"));
+	g_object_set_data(G_OBJECT(pxbf->reqsel_dst), "user", GINT_TO_POINTER(CBAF_REQSEL_DEST));
+	g_signal_connect(G_OBJECT(pxbf->reqsel_dst), "clicked", G_CALLBACK(evt_baf_reqsel_clicked), page);
+	gtk_grid_attach(GTK_GRID(grid), pxbf->reqsel_dst, 1, 0, 1, 1);
 
-	pxbaf->reqsel_src = gtk_check_button_new_with_label(_("Require Source Selection?"));
-	g_object_set_data(G_OBJECT(pxbaf->reqsel_src), "user", GINT_TO_POINTER(CBAF_REQSEL_SOURCE));
-	g_signal_connect(G_OBJECT(pxbaf->reqsel_src), "clicked", G_CALLBACK(evt_baf_reqsel_clicked), page);
-	gtk_box_pack_start(GTK_BOX(vbox), pxbaf->reqsel_src, TRUE, TRUE, 0);
-	pxbaf->reqsel_dst = gtk_check_button_new_with_label(_("Require Destination Selection?"));
-	g_object_set_data(G_OBJECT(pxbaf->reqsel_dst), "user", GINT_TO_POINTER(CBAF_REQSEL_DEST));
-	g_signal_connect(G_OBJECT(pxbaf->reqsel_dst), "clicked", G_CALLBACK(evt_baf_reqsel_clicked), page);
-	gtk_box_pack_start(GTK_BOX(vbox), pxbaf->reqsel_dst, TRUE, TRUE, 0);
+	pxbf->cdsrc = gtk_check_button_new_with_label(_("CD Source?"));
+	g_signal_connect(G_OBJECT(pxbf->cdsrc), "clicked", G_CALLBACK(evt_baf_cd_clicked), page);
+	gtk_grid_attach(GTK_GRID(grid), pxbf->cdsrc, 0, 1, 1, 1);
+	pxbf->cddst = gtk_check_button_new_with_label(_("CD Destination?"));
+	g_object_set_data(G_OBJECT(pxbf->cddst), "destination", GINT_TO_POINTER(1));	/* Just a silly flag. */
+	g_signal_connect(G_OBJECT(pxbf->cddst), "clicked", G_CALLBACK(evt_baf_cd_clicked), page);
+	gtk_grid_attach(GTK_GRID(grid), pxbf->cddst, 1, 1, 1, 1);
+	gtk_box_pack_start(GTK_BOX(pxbf->vbox), grid, TRUE, TRUE, 0);
+}
 
-	hbox = gtk_hbox_new(TRUE, 0);
-	pxbaf->cdsrc = gtk_check_button_new_with_label(_("CD Source?"));
-	g_signal_connect(G_OBJECT(pxbaf->cdsrc), "clicked", G_CALLBACK(evt_baf_cd_clicked), page);
-	gtk_box_pack_start(GTK_BOX(hbox), pxbaf->cdsrc, TRUE, TRUE, 0);
-	pxbaf->cddst = gtk_check_button_new_with_label(_("CD Destination?"));
-	g_object_set_data(G_OBJECT(pxbaf->cddst), "destination", GINT_TO_POINTER(1));	/* Just a silly flag. */
-	g_signal_connect(G_OBJECT(pxbaf->cddst), "clicked", G_CALLBACK(evt_baf_cd_clicked), page);
-	gtk_box_pack_start(GTK_BOX(hbox), pxbaf->cddst, TRUE, TRUE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
-	gtk_container_add(GTK_CONTAINER(pxbaf->bframe), vbox);
-	gtk_box_pack_start(GTK_BOX(pxbaf->hbox), pxbaf->bframe, TRUE, TRUE, 0);
+static void build_px_af(P_CmdSeq *page, PXE_AF *pxaf)
+{
+	GtkWidget	*hbox;
 
-	pxbaf->aframe = gtk_frame_new(_("After Execution"));
-	vbox = gtk_vbox_new(FALSE, 0);
-	pxbaf->rssrc = gtk_check_button_new_with_label(_("Rescan Source?"));
-	g_object_set_data(G_OBJECT(pxbaf->rssrc), "user", GINT_TO_POINTER(CBAF_RESCAN_SOURCE));
-	g_signal_connect(G_OBJECT(pxbaf->rssrc), "clicked", G_CALLBACK(evt_baf_rescan_clicked), page);
-	gtk_box_pack_start(GTK_BOX(vbox), pxbaf->rssrc, TRUE, TRUE, 0);
-	pxbaf->rsdst = gtk_check_button_new_with_label(_("Rescan Destination?"));
-	g_object_set_data(G_OBJECT(pxbaf->rsdst), "user", GINT_TO_POINTER(CBAF_RESCAN_DEST));
-	g_signal_connect(G_OBJECT(pxbaf->rsdst), "clicked", G_CALLBACK(evt_baf_rescan_clicked), page);
-	gtk_box_pack_start(GTK_BOX(vbox), pxbaf->rsdst, TRUE, TRUE, 0);
-	gtk_container_add(GTK_CONTAINER(pxbaf->aframe), vbox);
-	gtk_box_pack_start(GTK_BOX(pxbaf->hbox), pxbaf->aframe, TRUE, TRUE, 0);
+	pxaf->vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
+
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
+	pxaf->rssrc = gtk_check_button_new_with_label(_("Rescan Source?"));
+	g_object_set_data(G_OBJECT(pxaf->rssrc), "user", GINT_TO_POINTER(CBAF_RESCAN_SOURCE));
+	g_signal_connect(G_OBJECT(pxaf->rssrc), "clicked", G_CALLBACK(evt_baf_rescan_clicked), page);
+	gtk_box_pack_start(GTK_BOX(hbox), pxaf->rssrc, TRUE, TRUE, 0);
+	pxaf->rsdst = gtk_check_button_new_with_label(_("Rescan Destination?"));
+	g_object_set_data(G_OBJECT(pxaf->rsdst), "user", GINT_TO_POINTER(CBAF_RESCAN_DEST));
+	g_signal_connect(G_OBJECT(pxaf->rsdst), "clicked", G_CALLBACK(evt_baf_rescan_clicked), page);
+	gtk_box_pack_start(GTK_BOX(hbox), pxaf->rsdst, TRUE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(pxaf->vbox), hbox, FALSE, FALSE, 0);
 }
 
 /* 1998-09-26 -	Build the extra configuration widgetry for external commands. Plenty of stuff. */
@@ -956,8 +963,10 @@ static void build_px_external(P_CmdSeq *page, PX_External *pxe)
 	build_pxe_general(page, &pxe->gflags);
 	gtk_notebook_append_page(GTK_NOTEBOOK(pxe->nbook), pxe->gflags.vbox, gtk_label_new(_("General")));
 
-	build_px_baf(page, &pxe->baf);
-	gtk_notebook_append_page(GTK_NOTEBOOK(pxe->nbook), pxe->baf.hbox, gtk_label_new(_("Before&After")));
+	build_px_bf(page, &pxe->bf);
+	gtk_notebook_append_page(GTK_NOTEBOOK(pxe->nbook), pxe->bf.vbox, gtk_label_new(_("Before")));
+	build_px_af(page, &pxe->af);
+	gtk_notebook_append_page(GTK_NOTEBOOK(pxe->nbook), pxe->af.vbox, gtk_label_new(_("After")));
 }
 
 static GtkWidget * ccs_init(MainInfo *min, gchar **name)
@@ -966,15 +975,15 @@ static GtkWidget * ccs_init(MainInfo *min, gchar **name)
 			*dblab[] = { N_("Add Row"), N_("Duplicate"), N_("Delete Row") };
 	GCallback	dbfunc[] = { G_CALLBACK(evt_addrow_clicked), G_CALLBACK(evt_duprow_clicked), G_CALLBACK(evt_delrow_clicked) };
 	guint			i;
-	GtkWidget		*scwin, *label, *vbox, *hbox, *vbox2, *sep, *paned, *evbo;
+	GtkWidget		*scwin, *label, *vbox, *hbox, *vbox2, *sep, *paned;
 	GtkCellRenderer		*cr;
 	GtkTreeViewColumn	*vc;
 	P_CmdSeq		*page = &the_page;
 
-	page->vbox = gtk_vbox_new(FALSE, 0);
-	paned = gtk_vpaned_new();
+	page->vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
+	paned = gtk_paned_new(GTK_ORIENTATION_VERTICAL);
 
-	vbox = gtk_vbox_new(FALSE, 0);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	scwin = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	page->store = gtk_list_store_new(CMDSEQ_COLUMN_COUNT, G_TYPE_STRING, G_TYPE_POINTER);
@@ -987,9 +996,10 @@ static GtkWidget * ccs_init(MainInfo *min, gchar **name)
 	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(page->view), FALSE);
 	g_signal_connect(G_OBJECT(gtk_tree_view_get_selection(GTK_TREE_VIEW(page->view))), "changed", G_CALLBACK(evt_cmdseq_selection_changed), page);
 	gtk_container_add(GTK_CONTAINER(scwin), page->view);
+	gtk_widget_set_size_request(scwin, -1, 100);
 	gtk_box_pack_start(GTK_BOX(vbox), scwin, TRUE, TRUE, 0);
 
-	page->nhbox = gtk_hbox_new(FALSE, 0);
+	page->nhbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	label = gtk_label_new(_("Name"));
 	gtk_box_pack_start(GTK_BOX(page->nhbox), label, FALSE, FALSE, 0);
 	page->name = gtk_entry_new();
@@ -1000,11 +1010,11 @@ static GtkWidget * ccs_init(MainInfo *min, gchar **name)
 	gtk_paned_pack1(GTK_PANED(paned), vbox, TRUE, TRUE);
 
 
-	vbox = gtk_vbox_new(FALSE, 0);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	page->dframe = gtk_frame_new(_("Definition"));
 
-	hbox = gtk_hbox_new(FALSE, 0);
-	vbox2 = gtk_vbox_new(FALSE, 0);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
+	vbox2 = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	for(i = 0; i < sizeof page->dbtn / sizeof page->dbtn[0]; i++)
 	{
 		page->dbtn[i] = gtk_button_new_with_label(_(dblab[i]));
@@ -1013,7 +1023,7 @@ static GtkWidget * ccs_init(MainInfo *min, gchar **name)
 	}
 	gtk_box_pack_start(GTK_BOX(hbox), vbox2, FALSE, FALSE, 0);
 
-	vbox2 = gtk_vbox_new(FALSE, 0);
+	vbox2 = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	page->dstore = gtk_list_store_new(DEF_COLUMN_COUNT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER);
 	page->sig_delete = g_signal_connect(G_OBJECT(page->dstore), "row_deleted", G_CALLBACK(evt_row_deleted), page);
 	page->dview = gtk_tree_view_new_with_model(GTK_TREE_MODEL(page->dstore));
@@ -1027,11 +1037,12 @@ static GtkWidget * ccs_init(MainInfo *min, gchar **name)
 	scwin = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	gtk_container_add(GTK_CONTAINER(scwin), page->dview);
+	gtk_widget_set_size_request(scwin, -1, 100);
 	gtk_box_pack_start(GTK_BOX(vbox2), scwin, TRUE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(hbox), vbox2, TRUE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 5);
 
-	page->dhbox = gtk_hbox_new(FALSE, 0);
+	page->dhbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	page->dtype = gui_build_combo_box(tlab, G_CALLBACK(evt_type_selected), page);
 	gtk_box_pack_start(GTK_BOX(page->dhbox), page->dtype, FALSE, FALSE, 0);
 	page->ddef = gtk_entry_new();
@@ -1053,20 +1064,17 @@ static GtkWidget * ccs_init(MainInfo *min, gchar **name)
 	gtk_notebook_append_page(GTK_NOTEBOOK(page->dextra), page->px_external.nbook, NULL);
 	gtk_box_pack_start(GTK_BOX(vbox), page->dextra, FALSE, FALSE, 0);
 
-	sep = gtk_hseparator_new();
+	sep = gtk_separator_new(GTK_ORIENTATION_HORIZONTAL);
 	gtk_box_pack_start(GTK_BOX(vbox), sep, FALSE, FALSE, 5);
 	page->drepeat = gtk_check_button_new_with_label(_("Repeat Sequence Until No Source Selection?"));
 	g_signal_connect(G_OBJECT(page->drepeat), "clicked", G_CALLBACK(evt_repeat_clicked), page);
 	gtk_box_pack_start(GTK_BOX(vbox), page->drepeat, FALSE, FALSE, 0);
 
 	gtk_container_add(GTK_CONTAINER(page->dframe), vbox);
-	/* Don't add the definition frame directly to the Paned; wrap in EventBox for clipping. */
-	evbo = gtk_event_box_new();
-	gtk_container_add(GTK_CONTAINER(evbo), page->dframe);
-	gtk_paned_pack2(GTK_PANED(paned), evbo, TRUE, TRUE);
+	gtk_paned_pack2(GTK_PANED(paned), page->dframe, TRUE, TRUE);
 	gtk_box_pack_start(GTK_BOX(page->vbox), paned, TRUE, TRUE, 0);
 
-	page->bhbox = gtk_hbox_new(FALSE, 0);
+	page->bhbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	page->badd = gtk_button_new_with_label(_("Add"));
 	g_signal_connect(G_OBJECT(page->badd), "clicked", G_CALLBACK(evt_add_clicked), page);
 	gtk_box_pack_start(GTK_BOX(page->bhbox), page->badd, TRUE, TRUE, 5);
@@ -1268,3 +1276,28 @@ GHashTable * ccs_get_current(void)
 {
 	return the_page.cmdseq;
 }
+
+/* 2014-12-26 -	Goto the named command sequence in the main list. Used by Style configuration editor. */
+gboolean ccs_goto_cmdseq(const char *name)
+{
+	GtkTreeIter	iter;
+
+	if(!gtk_tree_model_get_iter_first(GTK_TREE_MODEL(the_page.store), &iter))
+		return FALSE;
+	while(TRUE)
+	{
+		gchar	*name_here;
+		gtk_tree_model_get(GTK_TREE_MODEL(the_page.store), &iter, CMDSEQ_COLUMN_NAME, &name_here, -1);
+		if(strcmp(name_here, name) == 0)
+		{
+			GtkTreePath	*path = gtk_tree_model_get_path(GTK_TREE_MODEL(the_page.store), &iter);
+			gtk_tree_view_set_cursor(GTK_TREE_VIEW(the_page.view), path, NULL, FALSE);
+			gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(the_page.view), path, NULL, FALSE, 0, 0);
+			gtk_tree_path_free(path);
+			return TRUE;
+		}
+		if(!gtk_tree_model_iter_next(GTK_TREE_MODEL(the_page.store), &iter))
+			break;
+	}
+	return FALSE;
+}
