@@ -55,8 +55,6 @@ typedef struct {
 	GtkWidget	*mhbox;
 	GtkWidget	*madd, *mdel;
 
-	GdkPixmap	*mpix[5];		/* Mouse icon pixmaps and masks. */
-	GdkBitmap	*mmsk[5];
 	GdkPixbuf	*mpbuf[5];		/* Mouse icon pixbufs. */
 
 	GtkWidget	*cmccmd;		/* Entry widget for Click-M-Click command. */
@@ -367,28 +365,28 @@ static void evt_mstate_clicked(GtkWidget *wid, gpointer user)
 	guint		i, x, y,
 			mask[] = { GDK_SHIFT_MASK, GDK_CONTROL_MASK, GDK_MOD1_MASK, GDK_MOD2_MASK,
 				   GDK_MOD3_MASK, GDK_MOD4_MASK, GDK_MOD5_MASK };
-	GtkWidget	*table, *label, *check[sizeof mask / sizeof mask[0]];
+	GtkWidget	*grid, *label, *check[sizeof mask / sizeof mask[0]];
 
 	ts = gtk_tree_view_get_selection(GTK_TREE_VIEW(page->mview));
 	if(ts == NULL || !gtk_tree_selection_get_selected(ts, NULL, &ti))
 		return;
 	gtk_tree_model_get(GTK_TREE_MODEL(page->mstore), &ti, MOUSE_COLUMN_MOUSE, &mouse, -1);
 
-	table = gtk_table_new(1 + (sizeof mask / sizeof *mask) / 2 + 1, 2, FALSE);
+	grid = gtk_grid_new();
 	label = gtk_label_new(_("The following modifier key(s) must\n"
 				"be held down when the mouse button\n"
 				"is clicked to trigger the command"));
-	gtk_table_attach(GTK_TABLE(table), label, 0, 2, 0, 1,  GTK_EXPAND|GTK_FILL,0,0,0);
+	gtk_grid_attach(GTK_GRID(grid), label, 0, 0, 2, 1);
 	for(i = 0, x = 0, y = 1; i < sizeof mask / sizeof mask[0]; i++, y++)
 	{
 		check[i] = gtk_check_button_new_with_label(gtk_accelerator_name(0, mask[i]));
 		if(ctrl_mouse_get_state(mouse) & mask[i])
 			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check[i]), TRUE);
-		gtk_table_attach(GTK_TABLE(table), check[i], x, x + 1, y, y + 1,  GTK_EXPAND|GTK_FILL,0,0,0);
+		gtk_grid_attach(GTK_GRID(grid), check[i], x, y, 1, 1);
 		if(i == (sizeof mask / sizeof *mask) / 2)
 			x++, y = 0;
 	}
-	dlg = dlg_dialog_sync_new(table, _("Edit Modifiers"), NULL);
+	dlg = dlg_dialog_sync_new(grid, _("Edit Modifiers"), NULL);
 	if(dlg_dialog_sync_wait(dlg) == DLG_POSITIVE)
 	{
 		guint	ns = 0U;
@@ -507,7 +505,7 @@ static void evt_clickmclick_delay_changed(GtkAdjustment *adj, gpointer user)
 	P_Controls	*page = user;
 
 	if(page)
-		ctrl_clickmclick_set_delay(page->ctrlinfo, adj->value);
+		ctrl_clickmclick_set_delay(page->ctrlinfo, gtk_adjustment_get_value(adj));
 }
 
 /* ----------------------------------------------------------------------------------------- */
@@ -563,14 +561,14 @@ static void build_mouse_menu(P_Controls *page)
 	                                NULL);
 
 	g_signal_connect(G_OBJECT(page->mdcombo), "changed", G_CALLBACK(evt_mbutton_changed), page);
-	gtk_table_attach(GTK_TABLE(page->mdtable), page->mdcombo, 1, 2, 0, 1,  GTK_FILL|GTK_EXPAND,0,0,0);
+	gtk_grid_attach(GTK_GRID(page->mdtable), page->mdcombo, 1, 0, 1, 1);
 }
 
 static GtkWidget * cct_init(MainInfo *min, gchar **name)
 {
 	P_Controls	*page = &the_page;
 	GtkWidget	*vbox, *frame, *label, *wid, *hbox, *scwin;
-	GtkObject	*adj;
+	GtkAdjustment	*adj;
 	GtkCellRenderer	*cr;
 	GtkTreeViewColumn *vc;
 	GtkTreeSelection *ts;
@@ -582,10 +580,10 @@ static GtkWidget * cct_init(MainInfo *min, gchar **name)
 	page->min = min;
 	page->modified = FALSE;
 
-	page->vbox = gtk_vbox_new(FALSE, 0);
+	page->vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 
 	page->kframe = gtk_frame_new(_("Global Keyboard Shortcuts"));
-	vbox = gtk_vbox_new(FALSE, 0);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	page->kstore = gtk_list_store_new(KEY_COLUMN_COUNT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER);
 	page->kview = gtk_tree_view_new_with_model(GTK_TREE_MODEL(page->kstore));
 	cr = gtk_cell_renderer_text_new();
@@ -602,23 +600,25 @@ static GtkWidget * cct_init(MainInfo *min, gchar **name)
 	gtk_container_add(GTK_CONTAINER(scwin), page->kview);
 	gtk_box_pack_start(GTK_BOX(vbox), scwin, TRUE, TRUE, 0);
 
-	page->kdtable = gtk_table_new(2, 3, FALSE);
+	page->kdtable = gtk_grid_new();
 	label = gtk_label_new(_("Key"));
-	gtk_table_attach(GTK_TABLE(page->kdtable), label, 0, 1, 0, 1,  0,0,0,0);
+	gtk_grid_attach(GTK_GRID(page->kdtable), label, 0, 0, 1, 1);
 	page->kdkey = gtk_entry_new();
 	g_signal_connect(G_OBJECT(page->kdkey), "key_press_event", G_CALLBACK(evt_kkey_press), page);
 	gtk_editable_set_editable(GTK_EDITABLE(page->kdkey), FALSE);
-	gtk_table_attach(GTK_TABLE(page->kdtable), page->kdkey, 1, 3, 0, 1,  GTK_EXPAND|GTK_FILL,0,0,0);
+	gtk_grid_attach(GTK_GRID(page->kdtable), page->kdkey, 1, 0, 3, 1);
 	label = gtk_label_new(_("Command"));
-	gtk_table_attach(GTK_TABLE(page->kdtable), label, 0, 1, 1, 2,  0,0,0,0);
+	gtk_grid_attach(GTK_GRID(page->kdtable), label, 0, 1, 1, 1);
 	page->kdcmd = gtk_entry_new();
+	gtk_widget_set_hexpand(page->kdcmd, TRUE);
+	gtk_widget_set_halign(page->kdcmd, GTK_ALIGN_FILL);
 	g_signal_connect(G_OBJECT(page->kdcmd), "changed", G_CALLBACK(evt_kcmd_changed), page);
-	gtk_table_attach(GTK_TABLE(page->kdtable), page->kdcmd, 1, 2, 1, 2,  GTK_EXPAND|GTK_FILL,0,0,0);
+	gtk_grid_attach(GTK_GRID(page->kdtable), page->kdcmd, 1, 1, 1, 1);
 	page->kdcmdpick = gui_details_button_new();
 	g_signal_connect(G_OBJECT(page->kdcmdpick), "clicked", G_CALLBACK(evt_kcmdpick_clicked), page);
-	gtk_table_attach(GTK_TABLE(page->kdtable), page->kdcmdpick, 2, 3, 1, 2,  0,0,0,0);
+	gtk_grid_attach(GTK_GRID(page->kdtable), page->kdcmdpick, 2, 1, 1, 1);
 	gtk_box_pack_start(GTK_BOX(vbox), page->kdtable, FALSE, FALSE, 0);
-	hbox = gtk_hbox_new(FALSE, 0);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	page->kadd = gtk_button_new_with_label(_("Add"));
 	g_signal_connect(G_OBJECT(page->kadd), "clicked", G_CALLBACK(evt_kadd_clicked), page);
 	gtk_box_pack_start(GTK_BOX(hbox), page->kadd, TRUE, TRUE, 5);
@@ -630,7 +630,7 @@ static GtkWidget * cct_init(MainInfo *min, gchar **name)
 	gtk_box_pack_start(GTK_BOX(page->vbox), page->kframe, TRUE, TRUE, 0);
 
 	page->mframe = gtk_frame_new(_("Dirpane Mouse Buttons"));
-	vbox = gtk_vbox_new(FALSE, 0);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 
 	page->mstore = gtk_list_store_new(MOUSE_COLUMN_COUNT, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER);
 	page->mview = gtk_tree_view_new_with_model(GTK_TREE_MODEL(page->mstore));
@@ -653,23 +653,25 @@ static GtkWidget * cct_init(MainInfo *min, gchar **name)
 	gtk_container_add(GTK_CONTAINER(scwin), page->mview);
 	gtk_box_pack_start(GTK_BOX(vbox), scwin, TRUE, TRUE, 0);
 
-	page->mdtable = gtk_table_new(2, 4, FALSE);
+	page->mdtable = gtk_grid_new();
 	label = gtk_label_new(_("Button"));
-	gtk_table_attach(GTK_TABLE(page->mdtable), label, 0, 1, 0, 1,  0,0,0,0);
+	gtk_grid_attach(GTK_GRID(page->mdtable), label, 0, 0, 1, 1);
 	build_mouse_menu(page);
 	wid = gtk_button_new_with_label(_("Edit Modifiers..."));
 	g_signal_connect(G_OBJECT(wid), "clicked", G_CALLBACK(evt_mstate_clicked), page);
-	gtk_table_attach(GTK_TABLE(page->mdtable), wid, 2, 3, 0, 1, GTK_FILL|GTK_EXPAND,0,5,0);
+	gtk_grid_attach(GTK_GRID(page->mdtable), wid, 2, 0, 2, 1);
 	label = gtk_label_new(_("Command"));
-	gtk_table_attach(GTK_TABLE(page->mdtable), label, 0, 1, 1, 2,  0,0,0,0);
+	gtk_grid_attach(GTK_GRID(page->mdtable), label, 0, 1, 1, 1);
 	page->mdcmd = gtk_entry_new();
+	gtk_widget_set_hexpand(page->mdcmd, TRUE);
+	gtk_widget_set_halign(page->mdcmd, GTK_ALIGN_FILL);
 	g_signal_connect(G_OBJECT(page->mdcmd), "changed", G_CALLBACK(evt_mcmd_changed), page);
-	gtk_table_attach(GTK_TABLE(page->mdtable), page->mdcmd, 1, 3, 1, 2,  GTK_EXPAND|GTK_FILL,0,0,0);
+	gtk_grid_attach(GTK_GRID(page->mdtable), page->mdcmd, 1, 1, 2, 1);
 	page->mdcmdpick = gui_details_button_new();
 	g_signal_connect(G_OBJECT(page->mdcmdpick), "clicked", G_CALLBACK(evt_mcmdpick_clicked), page);
-	gtk_table_attach(GTK_TABLE(page->mdtable), page->mdcmdpick, 3, 4, 1, 2,  0,0,0,0);
+	gtk_grid_attach(GTK_GRID(page->mdtable), page->mdcmdpick, 3, 1, 1, 1);
 	gtk_box_pack_start(GTK_BOX(vbox), page->mdtable, FALSE, FALSE, 0);
-	page->mhbox = gtk_hbox_new(FALSE, 0);
+	page->mhbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	page->madd = gtk_button_new_with_label(_("Add"));
 	g_signal_connect(G_OBJECT(page->madd), "clicked", G_CALLBACK(evt_madd_clicked), page);
 	gtk_box_pack_start(GTK_BOX(page->mhbox), page->madd, TRUE, TRUE, 5);
@@ -681,7 +683,7 @@ static GtkWidget * cct_init(MainInfo *min, gchar **name)
 	gtk_box_pack_start(GTK_BOX(page->vbox), page->mframe, TRUE, TRUE, 0);
 
 	frame = gtk_frame_new(_("Click-M-Click Gesture"));
-	hbox = gtk_hbox_new(FALSE, 0);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	label = gtk_label_new(_("Command"));
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 	page->cmccmd = gtk_entry_new();
@@ -694,7 +696,7 @@ static GtkWidget * cct_init(MainInfo *min, gchar **name)
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
 	adj = gtk_adjustment_new(0.400f, 0.100f, 1.5f, 0.1f, 0.1f, 0.0f);
 	g_signal_connect(G_OBJECT(adj), "value_changed", G_CALLBACK(evt_clickmclick_delay_changed), page);
-	page->cmcdelay = gtk_hscale_new(GTK_ADJUSTMENT(adj));
+	page->cmcdelay = gtk_scale_new(GTK_ORIENTATION_HORIZONTAL, GTK_ADJUSTMENT(adj));
 	gtk_scale_set_value_pos(GTK_SCALE(page->cmcdelay), GTK_POS_RIGHT);
 	gtk_scale_set_digits(GTK_SCALE(page->cmcdelay), 3);
 	gtk_box_pack_start(GTK_BOX(hbox), page->cmcdelay, TRUE, TRUE, 0);
