@@ -68,6 +68,7 @@ typedef struct {			/* Pane definition widgets. */
 	GtkWidget	*ffont;		/* Check button for font overriding. */
 	GtkWidget	*fontbutton;	/* Font button for picking font. */
 	GtkWidget	*frband;	/* Check button for rubber banding. */
+	GtkWidget	*fruled;	/* Check button for GTK+ ruling of tree view. */
 
 	GSList		*splist;
 	GtkWidget	*spos[3];	/* Radio buttons for scrollbar position. */
@@ -135,7 +136,7 @@ typedef struct {		/* Holds data used when editing a column format. */
 	GtkWidget	*title;		/* Entry widget holding the wanted title. */
 	GtkWidget	*just;		/* Justification option menu. */
 	guint		curr_just;	/* Current justification (0=left, 1=right, 2=center). */
-	GtkObject	*wadj;		/* An adjustment for the width. */
+	GtkAdjustment	*wadj;		/* An adjustment for the width. */
 	GtkWidget	*width;		/* A spin button for the width. */
 	union {
 	EE_Name		name;
@@ -284,16 +285,16 @@ static void evt_aview_row_activated(GtkWidget *wid, GtkTreePath *path, GtkTreeVi
 */
 static void build_content_format(GtkWidget *vbox, GtkWidget **wid, const gchar *ltext, const gchar *dtext)
 {
-	GtkWidget	*table, *label;
+	GtkWidget	*grid, *label;
 
-	table = gtk_table_new(1, 2, FALSE);
+	grid = gtk_grid_new();
 	label = gtk_label_new(ltext);
 	*wid = gtk_entry_new();
 	gtk_entry_set_max_length(GTK_ENTRY(*wid), DP_DATEFMT_SIZE - 1);
 	gtk_entry_set_text(GTK_ENTRY(*wid), dtext);
-	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
-	gtk_table_attach_defaults(GTK_TABLE(table), *wid,  1, 2, 0, 1);
-	gtk_box_pack_start(GTK_BOX(vbox), table, TRUE, TRUE, 0);
+	gtk_grid_attach(GTK_GRID(grid), label, 0, 0, 1, 1);
+	gtk_grid_attach(GTK_GRID(grid), *wid,  1, 0, 1, 1);
+	gtk_box_pack_start(GTK_BOX(vbox), grid, TRUE, TRUE, 0);
 }
 
 static void evt_size_type_toggled(GtkWidget *wid, gpointer user)
@@ -321,13 +322,13 @@ static GtkWidget * add_content_widgets(P_Edit *edit)
 {
 	gchar		*tmp = "", tickstr[2] = "?";
 	DpCFmt		*fmt = &edit->def->edit.format[edit->def->srow];
-	GtkObject	*adj;
-	GtkWidget	*vbox, *frame, *table, *label, *rb, *trb[SZE_NUM_UNITS], *sep;
+	GtkAdjustment	*adj;
+	GtkWidget	*vbox, *frame, *grid, *label, *rb, *trb[SZE_NUM_UNITS], *sep;
 	ExtraType	type = ET_NONE;
 	GSList		*group;
 	guint		i, ti = 0;
 
-	vbox = gtk_vbox_new(FALSE, 0);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 
 	switch(edit->content)
 	{
@@ -341,7 +342,7 @@ static GtkWidget * add_content_widgets(P_Edit *edit)
 			type = ET_NAME;
 			break;
 		case DPC_SIZE:
-			table = gtk_table_new(SZE_NUM_UNITS, 2, FALSE);
+			grid = gtk_grid_new();
 			for(i = 0, group = NULL; i < SZE_NUM_UNITS; i++)
 			{
 				const SzUnit	unit = (SzUnit) i;
@@ -349,37 +350,37 @@ static GtkWidget * add_content_widgets(P_Edit *edit)
 				trb[i] = rb = gtk_radio_button_new_with_label(group, _(sze_get_unit_label(unit)));
 				g_object_set_data(G_OBJECT(rb), "type", GINT_TO_POINTER(unit));
 				g_signal_connect(G_OBJECT(rb), "toggled", G_CALLBACK(evt_size_type_toggled), edit);
-				gtk_table_attach(GTK_TABLE(table), rb, 0, 2, i, i + 1,  GTK_EXPAND|GTK_FILL,0,0,0);
+				gtk_grid_attach(GTK_GRID(grid), rb, 0, i, 2, 1);
 				if(fmt->extra.size.unit == unit)
 					ti = i;
 				group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(rb));
 			}
-			gtk_box_pack_start(GTK_BOX(vbox), table, TRUE, TRUE, 0);
+			gtk_box_pack_start(GTK_BOX(vbox), grid, TRUE, TRUE, 0);
 
-			edit->extra.size.options = gtk_table_new(2, 2, FALSE);
+			edit->extra.size.options = gtk_grid_new();
 			edit->extra.size.ticks = gtk_check_button_new_with_label(_("Place Tick Every 3 Digits?"));
-			gtk_table_attach(GTK_TABLE(edit->extra.size.options), edit->extra.size.ticks, 0, 1, 0, 1,  0,0,0,0);
+			gtk_grid_attach(GTK_GRID(edit->extra.size.options), edit->extra.size.ticks, 0, 0, 1, 1);
 			edit->extra.size.tick = gtk_entry_new();
 			gtk_entry_set_max_length(GTK_ENTRY(edit->extra.size.tick), 1);
 			gtk_entry_set_width_chars(GTK_ENTRY(edit->extra.size.tick), 1);
 			tickstr[0] = fmt->extra.size.tick;
 			gtk_entry_set_text(GTK_ENTRY(edit->extra.size.tick), tickstr);
-			gtk_table_attach(GTK_TABLE(edit->extra.size.options), edit->extra.size.tick, 1, 2, 0, 1,  GTK_SHRINK,0,0,0);
+			gtk_grid_attach(GTK_GRID(edit->extra.size.options), edit->extra.size.tick, 1, 0, 1, 1);
 			g_signal_connect(G_OBJECT(edit->extra.size.ticks), "toggled", G_CALLBACK(evt_size_ticks_toggled), edit);
 			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(edit->extra.size.ticks), fmt->extra.size.ticks);
 
 			label = gtk_label_new(_("Digits of Precision"));
-			gtk_table_attach(GTK_TABLE(edit->extra.size.options), label, 0, 1, 1, 2,  GTK_EXPAND|GTK_FILL,0,0,0);
+			gtk_grid_attach(GTK_GRID(edit->extra.size.options), label, 0, 1, 1, 1);
 			adj = gtk_adjustment_new(0.0, 0.0, 6.0, 1.0, 1.0, 0.0);
 			edit->extra.size.digits = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1.0, 0);
 			gtk_spin_button_set_value(GTK_SPIN_BUTTON(edit->extra.size.digits), fmt->extra.size.digits);
-			gtk_table_attach(GTK_TABLE(edit->extra.size.options), edit->extra.size.digits, 1, 2, 1, 2,  GTK_EXPAND|GTK_FILL,0,0,0);
+			gtk_grid_attach(GTK_GRID(edit->extra.size.options), edit->extra.size.digits, 1, 1, 1, 1);
 			gtk_box_pack_start(GTK_BOX(vbox), edit->extra.size.options, TRUE, TRUE, 0);
 
 			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(trb[ti]), TRUE);
 			gtk_widget_set_sensitive(edit->extra.size.options, ti != 0);
 
-			sep = gtk_hseparator_new();
+			sep = gtk_separator_new(GTK_ORIENTATION_HORIZONTAL);
 			gtk_box_pack_start(GTK_BOX(vbox), sep, FALSE, FALSE, 0);
 			edit->extra.size.dir_show_fs_size = gtk_check_button_new_with_label(_("Show Dir's File System Size?"));
 			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(edit->extra.size.dir_show_fs_size), fmt->extra.size.dir_show_fs_size);
@@ -426,9 +427,11 @@ static GtkWidget * add_content_widgets(P_Edit *edit)
 		case DPC_MTIME:
 		case DPC_CHTIME:
 		case DPC_CRTIME:
-			table = gtk_table_new(1, 2, FALSE);
+			grid = gtk_grid_new();
 			label = gtk_label_new(_("Format"));
 			edit->extra.date.format = gtk_entry_new();
+			gtk_widget_set_hexpand(edit->extra.date.format, TRUE);
+			gtk_widget_set_halign(edit->extra.date.format, GTK_ALIGN_FILL);
 			gtk_entry_set_max_length(GTK_ENTRY(edit->extra.date.format), DP_DATEFMT_SIZE - 1);
 			if(edit->content == DPC_ATIME)
 				tmp = fmt->extra.a_time.format;
@@ -439,9 +442,9 @@ static GtkWidget * add_content_widgets(P_Edit *edit)
 			else if(edit->content == DPC_CRTIME)
 				tmp = fmt->extra.cr_time.format;
 			gtk_entry_set_text(GTK_ENTRY(edit->extra.date.format), tmp);
-			gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
-			gtk_table_attach_defaults(GTK_TABLE(table), edit->extra.date.format, 1, 2, 0, 1);
-			gtk_box_pack_start(GTK_BOX(vbox), table, TRUE, TRUE, 0);
+			gtk_grid_attach(GTK_GRID(grid), label, 0, 0, 1, 1);
+			gtk_grid_attach(GTK_GRID(grid), edit->extra.date.format, 1, 0, 1, 1);
+			gtk_box_pack_start(GTK_BOX(vbox), grid, TRUE, TRUE, 0);
 			type = ET_DATE;
 		case DPC_TYPE:
 			break;
@@ -485,7 +488,7 @@ static void accept_format(P_Edit *edit)
 
 	strcpy(fmt->title, gtk_entry_get_text(GTK_ENTRY(edit->title)));
 	fmt->just = edit->curr_just;
-	fmt->width = (gint) GTK_ADJUSTMENT(edit->wadj)->value;
+	fmt->width = (gint) gtk_adjustment_get_value(edit->wadj);
 	if(gtk_tree_selection_get_selected(gtk_tree_view_get_selection(GTK_TREE_VIEW(edit->def->sview)), NULL, &iter))
 		gtk_list_store_set(edit->def->sstore, &iter, CONTENT_COLUMN_TITLE, fmt->title, -1);
 
@@ -569,7 +572,7 @@ static gint evt_sview_edit_clicked(GtkWidget *wid, gpointer user)
 {
 	PDef		*def = user;
 	DpCFmt		*fmt = &def->edit.format[def->srow];
-	GtkWidget	*vbox, *frame1, *frame2, *table, *algn, *label;
+	GtkWidget	*vbox, *frame1, *frame2, *grid, *label;
 	const gchar	*lab[]   = { N_("Content"), N_("Title"), N_("Justification"), N_("Width") },
 			*jtext[] = { N_("Left"), N_("Right"), N_("Center"), NULL };
 	static P_Edit	edit;
@@ -579,24 +582,24 @@ static gint evt_sview_edit_clicked(GtkWidget *wid, gpointer user)
 	edit.def     = def;
 	edit.content = fmt->content;
 
-	vbox   = gtk_vbox_new(FALSE, 0);
+	vbox   = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	frame1 = gtk_frame_new(_("Basic Settings"));
 
-	table = gtk_table_new(4, 2, FALSE);
+	grid = gtk_grid_new();
 	for(i = 0; i < sizeof lab / sizeof lab[0]; i++)
 	{
-		algn  = gtk_alignment_new(0.9f, 0.0f, 0.0f, 0.0f);
 		label = gtk_label_new(_(lab[i]));
-		gtk_container_add(GTK_CONTAINER(algn), label);
-		gtk_table_attach(GTK_TABLE(table), algn, 0, 1, i, i+1, GTK_FILL,0,0,0);
+		gtk_grid_attach(GTK_GRID(grid), label, 0, i, 1, 1);
 	}
 	label = gtk_label_new(dpf_get_content_name(edit.content));
-	gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, 0, 1);
+	gtk_widget_set_hexpand(label, TRUE);
+	gtk_widget_set_halign(label, GTK_ALIGN_FILL);
+	gtk_grid_attach(GTK_GRID(grid), label, 1, 0, 1, 1);
 
 	edit.title = gtk_entry_new();
 	gtk_entry_set_max_length(GTK_ENTRY(edit.title), DP_TITLE_SIZE - 1);
 	gtk_entry_set_text(GTK_ENTRY(edit.title), fmt->title);
-	gtk_table_attach_defaults(GTK_TABLE(table), edit.title, 1, 2, 1, 2);
+	gtk_grid_attach(GTK_GRID(grid), edit.title, 1, 1, 1, 1);
 
 	edit.just = gui_build_combo_box(jtext, G_CALLBACK(evt_edit_justification), &edit);
 	switch(fmt->just)
@@ -613,15 +616,15 @@ static gint evt_sview_edit_clicked(GtkWidget *wid, gpointer user)
 			break;
 	}
 	gtk_combo_box_set_active(GTK_COMBO_BOX(edit.just), edit.curr_just);
-	gtk_table_attach_defaults(GTK_TABLE(table), edit.just, 1, 2, 2, 3);
+	gtk_grid_attach(GTK_GRID(grid), edit.just, 1, 2, 1, 1);
 
 	edit.wadj  = gtk_adjustment_new(fmt->width, 10.0, 800.0, 1.0f, 50.0, 0.0);
 	edit.width = gtk_spin_button_new(GTK_ADJUSTMENT(edit.wadj), 0, 0);
 	gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(edit.width), FALSE);
-	gtk_table_attach_defaults(GTK_TABLE(table), edit.width, 1, 2, 3, 4);
+	gtk_grid_attach(GTK_GRID(grid), edit.width, 1, 3, 1, 1);
 
 	gtk_container_set_border_width(GTK_CONTAINER(frame1), 5);
-	gtk_container_add(GTK_CONTAINER(frame1), table);
+	gtk_container_add(GTK_CONTAINER(frame1), grid);
 
 	gtk_box_pack_start(GTK_BOX(vbox), frame1, TRUE, TRUE, 0);
 
@@ -825,6 +828,14 @@ static void evt_default_path_changed(GtkWidget *wid, gpointer user)
 	g_strlcpy(def->edit.def_path, gtk_entry_get_text(GTK_ENTRY(wid)), sizeof def->edit.def_path);
 }
 
+/* 2012-05-27 - Set default path to "current directory". */
+static void evt_default_path_current(GtkWidget *wid, gpointer user)
+{
+	PDef    *def = user;
+
+	gtk_entry_set_text(GTK_ENTRY(def->dentry), "@cwd");
+}
+
 /* 1999-05-15 -	User just hit the "Grab Current" button for the default path. Do it. */
 static void evt_default_path_grab(GtkWidget *wid, gpointer user)
 {
@@ -902,6 +913,13 @@ static void evt_flag_rubberbanding_clicked(GtkWidget *wid, gpointer user)
 	def->edit.rubber_banding = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wid));
 }
 
+static void evt_flag_rules_clicked(GtkWidget *wid, gpointer user)
+{
+	PDef	*def = user;
+
+	def->edit.rules = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wid));
+}
+
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1999-05-15 -	User clicked on one of the "scrollbar position" widgets. Update. */
@@ -1001,7 +1019,7 @@ static GtkWidget * init_content_list_view(GtkListStore *store, gboolean availabl
 static void init_pane_page(P_DirPane *page, guint index, const gchar *other)
 {
 	PDef		*def;
-	GtkWidget	*label, *scwin, *vbox, *frame, *hbox, *table, *button, *sep, *fhbox;
+	GtkWidget	*label, *scwin, *vbox, *frame, *hbox, *grid, *button, *sep, *fhbox;
 	const gchar	*smitem[] = { N_("Directories First"), N_("Directories Last"), N_("Directories Mixed"), NULL },
 				*spos[] = { N_("System Default"), N_("Left of List"), N_("Right of List") },
 				*tfmt[] = { N_("Copy To %s"), N_("Copy From %s"), N_("Swap With %s") };
@@ -1018,14 +1036,14 @@ static void init_pane_page(P_DirPane *page, guint index, const gchar *other)
 	def->page  = page;
 	def->index = index;
 
-	def->vbox = gtk_vbox_new(FALSE, 0);
+	def->vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 
 	/* Columns part. */
 	frame = gtk_frame_new(_("Columns"));
-	hbox  = gtk_hbox_new(FALSE, 0);
+	hbox  = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 
 	/*  Available content types. */
-	vbox = gtk_vbox_new(FALSE, 0);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	label = gtk_label_new(_("Available Content Types"));
 	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
 	def->astore = init_content_list_store(TRUE);
@@ -1038,11 +1056,11 @@ static void init_pane_page(P_DirPane *page, guint index, const gchar *other)
 	gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
 
 	/*  Separation between available & selected content types. */
-	sep = gtk_vseparator_new();
+	sep = gtk_separator_new(GTK_ORIENTATION_VERTICAL);
 	gtk_box_pack_start(GTK_BOX(hbox), sep, FALSE, FALSE, 5);
 
 	/*  Selected content types. */
-	vbox = gtk_vbox_new(FALSE, 0);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	label = gtk_label_new(_("Selected Content Types"));
 	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
 	def->sstore = init_content_list_store(FALSE);
@@ -1054,25 +1072,25 @@ static void init_pane_page(P_DirPane *page, guint index, const gchar *other)
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	gtk_container_add(GTK_CONTAINER(scwin), def->sview);
 	gtk_box_pack_start(GTK_BOX(vbox), scwin, TRUE, TRUE, 0);
-	table = gtk_table_new(1, sizeof def->sbutton / sizeof def->sbutton[0], FALSE);
+	grid = gtk_grid_new();
 	def->sbutton[0] = gtk_button_new_with_label(_("Edit..."));
 	def->sbutton[1] = gtk_button_new_with_label(_("Remove"));
-	def->sbutton[2] = gui_arrow_button_new(GTK_ARROW_UP);
-	def->sbutton[3] = gui_arrow_button_new(GTK_ARROW_DOWN);
+	def->sbutton[2] = gtk_button_new_from_icon_name("go-up", GTK_ICON_SIZE_MENU);
+	def->sbutton[3] = gtk_button_new_from_icon_name("go-down", GTK_ICON_SIZE_MENU);
 	for(i = 0; i < sizeof def->sbutton / sizeof def->sbutton[0]; i++)
 	{
 		g_signal_connect(G_OBJECT(def->sbutton[i]), "clicked", G_CALLBACK(scfunc[i]), def);
-		gtk_table_attach_defaults(GTK_TABLE(table), def->sbutton[i], i, i + 1, 0, 1);
+		gtk_grid_attach(GTK_GRID(grid), def->sbutton[i], i, 0, 1, 1);
 	}
-	gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), grid, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
 	gtk_container_add(GTK_CONTAINER(frame), hbox);
 	gtk_box_pack_start(GTK_BOX(def->vbox), frame, TRUE, TRUE, 0);
 
 	/* Sorting. */
 	frame = gtk_frame_new(_("Sorting"));
-	vbox = gtk_vbox_new(FALSE, 0);
-	hbox = gtk_hbox_new(FALSE, 0);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	label = gtk_label_new(_("Sort On"));
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 	def->scmenu = dpf_get_content_combo_box(G_CALLBACK(evt_sort_set_content), def);
@@ -1082,7 +1100,7 @@ static void init_pane_page(P_DirPane *page, guint index, const gchar *other)
 	def->smmenu = gui_build_combo_box(smitem, G_CALLBACK(evt_sort_set_mode), def);
 	gtk_box_pack_start(GTK_BOX(hbox), def->smmenu, TRUE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
-	hbox = gtk_hbox_new(FALSE, 0);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	def->sinvert = gtk_check_button_new_with_label(_("Inverse Sorting?"));
 	g_signal_connect(G_OBJECT(def->sinvert), "clicked", G_CALLBACK(evt_sort_invert_clicked), def);
 	gtk_box_pack_start(GTK_BOX(hbox), def->sinvert, TRUE, FALSE, 0);
@@ -1095,11 +1113,14 @@ static void init_pane_page(P_DirPane *page, guint index, const gchar *other)
 
 	/* Default directory. */
 	frame = gtk_frame_new(_("Default Directory"));
-	hbox = gtk_hbox_new(FALSE, 0);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	def->dentry = gtk_entry_new();
 	gtk_entry_set_max_length(GTK_ENTRY(def->dentry), PATH_MAX - 1);
 	g_signal_connect(G_OBJECT(def->dentry), "changed", G_CALLBACK(evt_default_path_changed), def);
 	gtk_box_pack_start(GTK_BOX(hbox), def->dentry, TRUE, TRUE, 0);
+	button = gtk_button_new_with_label(_("Starting Directory"));
+	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(evt_default_path_current), def);
+	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
 	button = gtk_button_new_with_label(_("Grab Current"));
 	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(evt_default_path_grab), def);
 	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
@@ -1110,29 +1131,29 @@ static void init_pane_page(P_DirPane *page, guint index, const gchar *other)
 	gtk_box_pack_start(GTK_BOX(def->vbox), frame, FALSE, FALSE, 0);
 
 	/* General section below default directory (flags, scrollbar, and 'ctrl' modification. */
-	hbox = gtk_hbox_new(FALSE, 0);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 
 	/* Flags. */
 	frame = gtk_frame_new(_("Flags"));
-	table = gtk_table_new(4, 2, TRUE);
+	grid = gtk_grid_new();
 
 	def->fabove = gtk_check_button_new_with_label(_("Path Above?"));
 	g_signal_connect(G_OBJECT(def->fabove), "clicked", G_CALLBACK(evt_flag_above_clicked), def);
-	gtk_table_attach_defaults(GTK_TABLE(table), def->fabove, 0, 1, 0, 1);
+	gtk_grid_attach(GTK_GRID(grid), def->fabove, 0, 0, 1, 1);
 
 	def->fhide = gtk_check_button_new_with_label(_("Hide Allowed?"));
 	g_signal_connect(G_OBJECT(def->fhide), "clicked", G_CALLBACK(evt_flag_hide_clicked), def);
-	gtk_table_attach_defaults(GTK_TABLE(table), def->fhide, 0, 1, 1, 2);
+	gtk_grid_attach(GTK_GRID(grid), def->fhide, 0, 1, 1, 1);
 
 	def->fscroll = gtk_check_button_new_with_label(_("Scrollbar Always?"));
 	g_signal_connect(G_OBJECT(def->fscroll), "clicked", G_CALLBACK(evt_flag_scrollbar_clicked), def);
-	gtk_table_attach_defaults(GTK_TABLE(table), def->fscroll, 1, 2, 0, 1);
+	gtk_grid_attach(GTK_GRID(grid), def->fscroll, 1, 0, 1, 1);
 
 	def->fhparent = gtk_check_button_new_with_label(_("Huge Parent Button?"));
 	g_signal_connect(G_OBJECT(def->fhparent), "clicked", G_CALLBACK(evt_flag_hugeparent_clicked), def);
-	gtk_table_attach_defaults(GTK_TABLE(table), def->fhparent, 1, 2, 1, 2);
+	gtk_grid_attach(GTK_GRID(grid), def->fhparent, 1, 1, 1, 1);
 
-	fhbox = gtk_hbox_new(FALSE, 0);
+	fhbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	def->ffont = gtk_check_button_new_with_label(_("Set Custom Font?"));
 	g_signal_connect(G_OBJECT(def->ffont), "clicked", G_CALLBACK(evt_flag_setfont_clicked), def);
 	gtk_box_pack_start(GTK_BOX(fhbox), def->ffont, FALSE, FALSE, 0);
@@ -1140,18 +1161,22 @@ static void init_pane_page(P_DirPane *page, guint index, const gchar *other)
 	g_signal_connect(G_OBJECT(def->fontbutton), "font_set", G_CALLBACK(evt_font_set), def);
 	gtk_widget_set_sensitive(def->fontbutton, FALSE);
 	gtk_box_pack_start(GTK_BOX(fhbox), def->fontbutton, TRUE, TRUE, 0);
-	gtk_table_attach_defaults(GTK_TABLE(table), fhbox, 0, 2, 2, 3);
+	gtk_grid_attach(GTK_GRID(grid), fhbox, 0, 2, 1, 1);
 
 	def->frband = gtk_check_button_new_with_label(_("Rubber banding Selection?"));
 	g_signal_connect(G_OBJECT(def->frband), "clicked", G_CALLBACK(evt_flag_rubberbanding_clicked), def);
-	gtk_table_attach_defaults(GTK_TABLE(table), def->frband, 0, 2, 3, 4);
+	gtk_grid_attach(GTK_GRID(grid), def->frband, 0, 3, 1, 1);
 
-	gtk_container_add(GTK_CONTAINER(frame), table);
+	def->fruled = gtk_check_button_new_with_label(_("Ruled Rows?"));
+	g_signal_connect(G_OBJECT(def->fruled), "clicked", G_CALLBACK(evt_flag_rules_clicked), def);
+	gtk_grid_attach(GTK_GRID(grid), def->fruled, 1, 2, 1, 1);
+
+	gtk_container_add(GTK_CONTAINER(frame), grid);
 	gtk_box_pack_start(GTK_BOX(hbox), frame, TRUE, TRUE, 0);
 
 	/* Scrollbar position (new). */
 	frame = gtk_frame_new(_("Scrollbar Position"));
-	vbox = gtk_vbox_new(FALSE, 0);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	def->splist = gui_radio_group_new(sizeof def->spos / sizeof def->spos[0], spos, def->spos);
 	for(i = 0; i < sizeof def->spos / sizeof def->spos[0]; i++)
 	{
@@ -1164,7 +1189,7 @@ static void init_pane_page(P_DirPane *page, guint index, const gchar *other)
 	gtk_box_pack_start(GTK_BOX(def->vbox), hbox, FALSE, FALSE, 0);
 
 	/* Pane "tools" (i.e., copy/swap) buttons (formerly on top). */
-	hbox = gtk_hbox_new(FALSE, 0);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	for(i = 0; i < sizeof tfmt / sizeof tfmt[0]; i++)
 	{
 		g_snprintf(buf, sizeof buf, _(tfmt[i]), other);
@@ -1191,8 +1216,8 @@ static void evt_pane_mode_toggled(GtkWidget *wid, gpointer user)
 	GtkWidget	*row;
 
 	if((row = g_object_get_data(G_OBJECT(wid), "row")) != NULL)
-		gtk_widget_set_sensitive(row, GTK_TOGGLE_BUTTON(wid)->active);	/* Sometimes, GTK+ is just plain nice. */
-	if(GTK_TOGGLE_BUTTON(wid)->active)
+		gtk_widget_set_sensitive(row, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wid)));
+	if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wid)))
 	{
 		page->paning.edit.mode = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(wid), "index"));
 		page->paning.value = g_object_get_data(G_OBJECT(wid), "value");
@@ -1202,14 +1227,14 @@ static void evt_pane_mode_toggled(GtkWidget *wid, gpointer user)
 static void init_paning_page(P_DirPane *page)
 {
 	PPaning		*p = &page->paning;
-	GtkWidget	*table, *rb, *lrb, *row, *value = NULL, *frame, *vbox;
+	GtkWidget	*grid, *rb, *lrb, *row, *value = NULL, *frame, *vbox;
 	const gchar	*orientname[] = { N_("Horizontal"), N_("Vertical") },
 			*modename[] = { N_("Don't Track"), N_("Ratio"), N_("Size, Left Pane"), N_("Size, Right Pane") };
 	guint		i;
 
-	p->vbox = gtk_vbox_new(FALSE, 0);
+	p->vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	frame   = gtk_frame_new(_("Pane Orientation"));
-	vbox    = gtk_vbox_new(FALSE, 0);
+	vbox    = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	for(i = 0; i < 2; i++)
 	{
 		p->orientation[i] = gtk_radio_button_new_with_label(i == 1 ?
@@ -1223,17 +1248,20 @@ static void init_paning_page(P_DirPane *page)
 	gtk_container_add(GTK_CONTAINER(frame), vbox);
 	gtk_box_pack_start(GTK_BOX(p->vbox), frame, FALSE, FALSE, 0);
 	frame = gtk_frame_new(_("Split Tracking"));
-	table = gtk_table_new(sizeof modename / sizeof modename[0], 2, FALSE);
+	grid = gtk_grid_new();
 	for(i = 0, lrb = NULL; i < sizeof modename / sizeof modename[0]; i++, lrb = rb)
 	{
 		p->mode[i] = rb = gtk_radio_button_new_with_label_from_widget(lrb ? GTK_RADIO_BUTTON(lrb) : NULL, _(modename[i]));
+		g_object_set_data(G_OBJECT(rb), "index", GINT_TO_POINTER(i));
+		g_signal_connect(G_OBJECT(rb), "toggled", G_CALLBACK(evt_pane_mode_toggled), page);
+		gtk_grid_attach(GTK_GRID(grid), rb, 0, i, 1, 1);
 		row = NULL;
 		if(i == 1)
 		{
-			GtkObject	*adj;
+			GtkAdjustment	*adj;
 
 			adj = gtk_adjustment_new(0.5, 0.0, 1.0, 0.0125, 0.125, 0);
-			row = value = gtk_hscale_new(GTK_ADJUSTMENT(adj));
+			row = value = gtk_scale_new(GTK_ORIENTATION_HORIZONTAL, GTK_ADJUSTMENT(adj));
 			gtk_scale_set_digits(GTK_SCALE(value), 4);
 			gtk_scale_set_value_pos(GTK_SCALE(value), GTK_POS_RIGHT);
 		}
@@ -1241,37 +1269,36 @@ static void init_paning_page(P_DirPane *page)
 		{
 			GtkWidget	*label;
 
-			row = gtk_hbox_new(FALSE, 0);
+			row = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 			value = gtk_entry_new();
 			gtk_entry_set_text(GTK_ENTRY(value), "256");	/* Better than being empty. */
 			gtk_box_pack_start(GTK_BOX(row), value, TRUE, TRUE, 0);
 			label = gtk_label_new(_("pixels"));
 			gtk_box_pack_start(GTK_BOX(row), label, FALSE, FALSE, 5);
 		}
-		g_object_set_data(G_OBJECT(rb), "index", GINT_TO_POINTER(i));
-		g_signal_connect(G_OBJECT(rb), "toggled", G_CALLBACK(evt_pane_mode_toggled), page);
-		gtk_table_attach(GTK_TABLE(table), rb, 0, 1, i, i+1, GTK_FILL,0,0,0);
 		if(row != NULL)
 		{
+			gtk_widget_set_hexpand(row, TRUE);
+			gtk_widget_set_halign(row, GTK_ALIGN_FILL);
 			g_object_set_data(G_OBJECT(rb), "row", row);
 			g_object_set_data(G_OBJECT(rb), "value", value);
 			gtk_widget_set_sensitive(row, FALSE);
-			gtk_table_attach(GTK_TABLE(table), row, 1, 2, i, i+1, GTK_EXPAND|GTK_FILL,0,0,0);
+			gtk_grid_attach(GTK_GRID(grid), row, 1, i, 1, 1);
 		}
 	}
-	gtk_container_add(GTK_CONTAINER(frame), table);
+	gtk_container_add(GTK_CONTAINER(frame), grid);
 	gtk_box_pack_start(GTK_BOX(p->vbox), frame, FALSE, FALSE, 0);
 }
 
 /* 2004-04-25 -	Store boolean when toggle button toggles. Re-usable. */
 static void evt_history_toggled(GtkWidget *wid, gpointer user)
 {
-	*(gboolean *) user = GTK_TOGGLE_BUTTON(wid)->active;
+	*(gboolean *) user = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wid));
 }
 
 static void init_history_page(P_DirPane *page)
 {
-	page->history.vbox = gtk_vbox_new(FALSE, 0);
+	page->history.vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 
 	page->history.select = gtk_check_button_new_with_label(_("Remember Selected Rows?"));
 	g_signal_connect(G_OBJECT(page->history.select), "toggled", G_CALLBACK(evt_history_toggled), &page->history.edit.select);
@@ -1354,7 +1381,7 @@ static void cdp_accept(MainInfo *min)
 		case DPSPLIT_FREE:
 			break;
 		case DPSPLIT_RATIO:
-			the_page.paning.edit.value = gtk_range_get_adjustment(GTK_RANGE(the_page.paning.value))->value;
+			the_page.paning.edit.value = gtk_range_get_value(GTK_RANGE(the_page.paning.value));
 			break;
 		case DPSPLIT_ABS_LEFT:
 		case DPSPLIT_ABS_RIGHT:
@@ -1462,6 +1489,7 @@ static void save_pane(FILE *out, DPFormat *fmt, const gchar *node_name)
 	xml_put_boolean(out, "set_font", fmt->set_font);
 	xml_put_text(out, "font_name", fmt->font_name);
 	xml_put_boolean(out, "rubber_banding", fmt->rubber_banding);
+	xml_put_boolean(out, "rules", fmt->rules);
 	xml_put_text(out, "sbar_pos", sbarpos_name[fmt->sbar_pos]);
 	xml_put_node_close(out, node_name);
 }
@@ -1682,6 +1710,8 @@ static void load_pane(const XmlNode *node, DPFormat *fmt)
 			g_snprintf(fmt->font_name, sizeof fmt->font_name, "%s", fontname);
 		if(xml_get_boolean(node, "rubber_banding", &tmp))
 			fmt->rubber_banding = tmp;
+		if(xml_get_boolean(node, "rules", &tmp))
+			fmt->rules = tmp;
 		if(xml_get_text(node, "sbar_pos", &sbarpos))
 		{
 			gboolean	found = FALSE;
