@@ -17,6 +17,8 @@
 ** 2000-07-02 -	Took a step closer to the rest of the world, by marking strings for translation.
 */
 
+#include <ctype.h>
+
 #include "gentoo.h"
 
 #include "styles.h"
@@ -68,6 +70,7 @@ typedef struct {			/* Action properties notebook page. */
 	GtkWidget	*vbox;
 	GtkListStore	*store;
 	GtkWidget	*view;
+	GtkWidget	*editcmd;	/* Shortcut to go to the Command editing configuration page. */
 	GtkWidget	*adel;		/* Delete (or override) command button. */
 } PAction;
 
@@ -129,8 +132,8 @@ static void set_widgets_preview(P_Styles *page, Style *stl)
 				gtk_list_store_set(store, &iter, COLUMN_ICON, pbuf, -1);
 		}
 	}
-	dpf_cell_set_style_colors(page->dpvisual.pre_icon_r, stl, FALSE);
-	dpf_cell_set_style_colors(page->dpvisual.pre_name_r, stl, TRUE);
+	dpf_cell_set_style_colors(page->dpvisual.pre_icon_r, stl, FALSE, FALSE);
+	dpf_cell_set_style_colors(page->dpvisual.pre_name_r, stl, TRUE,  TRUE);
 }
 
 /* 1999-05-24 -	Set action list. */
@@ -196,7 +199,8 @@ static void set_widgets(P_Styles *page, Style *stl)
 /* 1999-05-25 -	Reset action editing widgets. */
 static void reset_widgets_action(P_Styles *page)
 {
-	gtk_label_set_text(GTK_LABEL(GTK_BIN(page->dpaction.adel)->child), _("Delete Action"));
+	gtk_widget_set_sensitive(page->dpaction.editcmd, FALSE);
+	gtk_label_set_text(GTK_LABEL(gtk_bin_get_child(GTK_BIN(page->dpaction.adel))), _("Delete Action"));
 	gtk_widget_set_sensitive(page->dpaction.adel, FALSE);
 }
 
@@ -214,7 +218,7 @@ static void reset_widgets(P_Styles *page)
 
 /* ----------------------------------------------------------------------------------------- */
 
-static Style * style_get_selected(P_Styles *page, GtkTreeIter *iter)
+static Style * style_get_selected(const P_Styles *page, GtkTreeIter *iter)
 {
 	GtkTreeIter	myiter;
 
@@ -359,10 +363,20 @@ static void evt_vprop_override_toggled(GtkWidget *wid, gpointer user)
 		else
 		{
 			const GdkColor	*def;
+			GdkColor	tmp;
 
 			/* FIXME: This assumes only background can be non-overridden in root. */
 			if((def = stl_style_property_get_color(style, pname)) == NULL)
-				def = &(gtk_widget_get_style(GTK_WIDGET(page->min->gui->pane[0].view))->base[GTK_STATE_NORMAL]);
+			{
+				GtkStyleContext	*sctx = gtk_widget_get_style_context(GTK_WIDGET(page->min->gui->pane[0].view));
+				GdkRGBA		bg;
+
+				gtk_style_context_get_background_color(sctx, GTK_STATE_FLAG_NORMAL, &bg);
+				tmp.red   = 65535.0 * bg.red;
+				tmp.green = 65535.0 * bg.green;
+				tmp.blue  = 65535.0 * bg.blue;
+				def = &tmp;
+			}
 			stl_style_property_set_color(style, pname, def);
 		}
 	}
@@ -373,15 +387,17 @@ static void evt_vprop_override_toggled(GtkWidget *wid, gpointer user)
 }
 
 /* 1999-05-24 -	A color property has changed. Update previews and stuff. */
-static void evt_vprop_color_changed(GdkColor *color, gpointer user)
+static void evt_vprop_color_changed(const GdkRGBA *color, gpointer user)
 {
 	P_Styles	*page = user;
 	Style		*style;
+	GdkColor	color_old;
 
 	if((style = style_get_selected(page, NULL)) == NULL)
 		return;
 
-	stl_style_property_set_color(style, page->curr_prop, color);
+	gui_color_from_rgba(&color_old, color);
+	stl_style_property_set_color(style, page->curr_prop, &color_old);
 	set_widgets(page, style);
 }
 
@@ -412,12 +428,20 @@ static void evt_vprop_edit_clicked(GtkWidget *wid, gpointer user)
 	else
 	{
 		const GdkColor	*col;
-		GdkColor	initial;
+		GdkColor	col_old;
+		GdkRGBA		initial, *ip = NULL;
 
 		if((col = stl_style_property_get_color(style, page->curr_prop)) != NULL)
-			initial = *col;
-		if(cdl_dialog_sync_new_wait(_("Edit Color"), evt_vprop_color_changed, &initial, page) != DLG_POSITIVE)
-			stl_style_property_set_color(style, page->curr_prop, &initial);
+		{
+			col_old = *col;
+			gui_rgba_from_color(&initial, col);
+			ip = &initial;
+		}
+		if(cdl_dialog_sync_new_wait(_("Edit Color"), evt_vprop_color_changed, ip, page) != DLG_POSITIVE)
+		{
+			if(col != NULL)
+				stl_style_property_set_color(style, page->curr_prop, &col_old);
+		}
 		else
 			page->modified = TRUE;
 		set_widgets(page, style);
@@ -426,13 +450,24 @@ static void evt_vprop_edit_clicked(GtkWidget *wid, gpointer user)
 
 /* ----------------------------------------------------------------------------------------- */
 
+/* 2014-12-25 -	Set sensitivity of the shortcut button for command-editing. User-defined commands only. */
+static void set_aprop_editcmd_button(const P_Styles *page, Style *style)
+{
+	const gchar	*action = stl_style_property_get_action(style, page->curr_prop);
+
+	if(action != NULL && *action != '\0' && g_hash_table_contains(ccs_get_current(), action))
+	{
+		gtk_widget_set_sensitive(page->dpaction.editcmd, islower((unsigned char) action[0]));
+	}
+}
+
 /* 2009-04-22 -	Set the proper label for the Delete Action button. Depends on override. */
 static gboolean set_aprop_delete_button(P_Styles *page, Style *style)
 {
-	gboolean		unique, ovr;
+	gboolean	unique, ovr;
 
 	unique = stl_style_property_is_unique(style, page->curr_prop);
-	gtk_label_set_text(GTK_LABEL(GTK_BIN(page->dpaction.adel)->child),
+	gtk_label_set_text(GTK_LABEL(gtk_bin_get_child(GTK_BIN(page->dpaction.adel))),
 				unique ? _("Delete Action") : _("Revert to Inherited Command"));
 	ovr = stl_style_property_get_override(style, page->curr_prop);
 	gtk_widget_set_sensitive(page->dpaction.adel, ovr);
@@ -448,7 +483,10 @@ static void evt_aprop_selection_changed(GtkTreeSelection *sel, gpointer user)
 	GtkTreeIter	myiter;
 
 	if((style = style_get_selected(page, NULL)) == NULL)
+	{
+		gtk_widget_set_sensitive(page->dpaction.editcmd, FALSE);
 		return;
+	}
 	if(gtk_tree_selection_get_selected(gtk_tree_view_get_selection(GTK_TREE_VIEW(page->dpaction.view)), NULL, &myiter))
 	{
 		gboolean		ovr;
@@ -456,7 +494,7 @@ static void evt_aprop_selection_changed(GtkTreeSelection *sel, gpointer user)
 		GList			*cellrenderers;
 
 		gtk_tree_model_get(GTK_TREE_MODEL(page->dpaction.store), &myiter, ACTION_COLUMN_ACTION, &page->curr_prop, -1);
-
+		set_aprop_editcmd_button(page, style);
 		ovr = set_aprop_delete_button(page, style);
 		/* Set the name column's editable mode, depending on override. */
 		col = gtk_tree_view_get_column(GTK_TREE_VIEW(page->dpaction.view), 0);
@@ -534,8 +572,7 @@ static void evt_aprop_cmdseq_populate_popup(GtkEntry *entry, GtkMenu *menu, gpoi
 {
 	GtkWidget	*wid;
 
-	wid = gtk_image_menu_item_new_with_label(_("Select Command ..."));
-	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(wid), gtk_image_new_from_stock(GTK_STOCK_FIND, GTK_ICON_SIZE_MENU));
+	wid = gtk_menu_item_new_with_label(_("Select Command ..."));	/* FIXME: Add back (deprecated) icon?! */
 	g_signal_connect(G_OBJECT(wid), "activate", G_CALLBACK(evt_aprop_cmdseq_pick_activated), user);
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), wid);
 	gtk_widget_show_all(wid);
@@ -592,7 +629,7 @@ static void evt_aprop_add_clicked(GtkWidget *wid, gpointer user)
 	if((style = style_get_selected(page, NULL)) == NULL)
 		return;
 
-	hbox = gtk_hbox_new(FALSE, 0);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	label = gtk_label_new(_("Name"));
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 	entry = gtk_entry_new();
@@ -615,6 +652,22 @@ static void evt_aprop_add_clicked(GtkWidget *wid, gpointer user)
 	dlg_dialog_sync_destroy(dlg);
 }
 
+/* 2014-12-26 -	Go to the command editor for the currently chosen command. Convenient? */
+static void evt_editcmd_clicked(GtkWidget *wid, gpointer user)
+{
+	const P_Styles	*page = user;
+	const Style	*style;
+	const gchar	*cmdseq;
+
+	if((style = style_get_selected(page, NULL)) == NULL)
+		return;
+	cmdseq = stl_style_property_get_action(style, page->curr_prop);
+	if(ccs_goto_cmdseq(cmdseq))
+	{
+		cfg_goto_page("Definitions");	/* This won't win any prices for low coupling. */
+	}
+}
+
 /* 1999-05-25 -	User clicked the "Delete" (or "Revert...") button. Do it. */
 static void evt_aprop_delete_clicked(GtkWidget *wid, gpointer user)
 {
@@ -708,8 +761,8 @@ static void build_pvisual(P_Styles *page)
 	GtkWidget		*hbox, *frame, *vbox, *label;
 	guint			i;
 
-	pv->vbox = gtk_vbox_new(FALSE, 0);
-	hbox = gtk_hbox_new(FALSE, 0);
+	pv->vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	label = gtk_label_new(_("Preview"));
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 	store = gtk_list_store_new(2, GDK_TYPE_PIXBUF, G_TYPE_STRING);
@@ -728,11 +781,11 @@ static void build_pvisual(P_Styles *page)
 	gtk_box_pack_start(GTK_BOX(hbox), pv->preview, TRUE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(pv->vbox), hbox, FALSE, FALSE, 2);
 
-	hbox = gtk_hbox_new(FALSE, 0);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	for(i = 0; i < 3; i++)
 	{
 		frame = gtk_frame_new(_(vplab[i]));
-		vbox = gtk_vbox_new(FALSE, 0);
+		vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 		gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
 		pv->override[i] = gtk_check_button_new_with_label(_("Override Parent's?"));
 		g_object_set_data(G_OBJECT(pv->override[i]), "user", (gpointer) vpname[i]);
@@ -756,7 +809,7 @@ static void build_paction(P_Styles *page)
 	GtkCellRenderer	*cr;
 	GtkTreeViewColumn *vc;
 
-	pa->vbox = gtk_vbox_new(FALSE, 0);
+	pa->vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	pa->store = gtk_list_store_new(ACTION_COLUMN_COUNT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER, G_TYPE_INT);
 	pa->view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(pa->store));
 	cr = gtk_cell_renderer_text_new();
@@ -778,10 +831,14 @@ static void build_paction(P_Styles *page)
 	gtk_container_add(GTK_CONTAINER(scwin), pa->view);
 	gtk_box_pack_start(GTK_BOX(pa->vbox), scwin, TRUE, TRUE, 0);
 
-	hbox = gtk_hbox_new(TRUE, 0);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
+	gtk_box_set_homogeneous(GTK_BOX(hbox), TRUE);
 	btn = gtk_button_new_with_label(_("Add Action..."));
 	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(evt_aprop_add_clicked), page);
 	gtk_box_pack_start(GTK_BOX(hbox), btn, TRUE, TRUE, 5);
+	pa->editcmd = gtk_button_new_with_label(_("Edit Command"));
+	g_signal_connect(G_OBJECT(pa->editcmd), "clicked", G_CALLBACK(evt_editcmd_clicked), page);
+	gtk_box_pack_start(GTK_BOX(hbox), pa->editcmd, TRUE, TRUE, 5);
 	pa->adel = gtk_button_new_with_label(_("Delete Action"));
 	g_signal_connect(G_OBJECT(pa->adel), "clicked", G_CALLBACK(evt_aprop_delete_clicked), page);
 	gtk_box_pack_start(GTK_BOX(hbox), pa->adel, TRUE, TRUE, 5);
@@ -792,7 +849,7 @@ static void build_paction(P_Styles *page)
 static GtkWidget * cst_init(MainInfo *min, gchar **name)
 {
 	P_Styles	*page = &the_page;
-	GtkWidget	*table, *label, *frame, *hbox, *btn;
+	GtkWidget	*grid, *label, *frame, *hbox, *btn;
 	GtkCellRenderer	*cr;
 	GtkTreeViewColumn *vc;
 
@@ -801,7 +858,7 @@ static GtkWidget * cst_init(MainInfo *min, gchar **name)
 	page->modified = FALSE;
 	page->store = NULL;
 
-	page->vbox = gtk_vbox_new(FALSE, 0);
+	page->vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	page->handlers = gui_handler_group_new();
 
 	page->scwin = gtk_scrolled_window_new(FALSE, FALSE);
@@ -817,25 +874,27 @@ static GtkWidget * cst_init(MainInfo *min, gchar **name)
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(page->scwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	gtk_box_pack_start(GTK_BOX(page->vbox), page->scwin, TRUE, TRUE, 0);
 
-	page->dvbox = gtk_vbox_new(FALSE, 0);
+	page->dvbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 
-	table = gtk_table_new(2, 3, FALSE);
+	grid = gtk_grid_new();
 	label = gtk_label_new(_("Name"));
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,  0,0,0,0);
+	gtk_grid_attach(GTK_GRID(grid), label, 0, 0, 1, 1);
 	page->dname = gtk_entry_new();
 	gtk_entry_set_max_length(GTK_ENTRY(page->dname), STL_STYLE_NAME_SIZE - 1);
 	gui_handler_group_connect(page->handlers, G_OBJECT(page->dname), "changed", G_CALLBACK(evt_style_name_changed), page);
-	gtk_table_attach(GTK_TABLE(table), page->dname, 1, 3, 0, 1,  GTK_EXPAND|GTK_FILL,0,0,0);
+	gtk_grid_attach(GTK_GRID(grid), page->dname, 1, 0, 2, 1);
 	label = gtk_label_new(_("Parent"));
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,  0,0,0,0);
+	gtk_grid_attach(GTK_GRID(grid), label, 0, 1, 1, 1);
 	page->dparent = gtk_entry_new();
+	gtk_widget_set_hexpand(page->dparent, TRUE);
+	gtk_widget_set_halign(page->dparent, GTK_ALIGN_FILL);
 	gtk_entry_set_max_length(GTK_ENTRY(page->dparent), STL_STYLE_NAME_SIZE - 1);
 	gtk_editable_set_editable(GTK_EDITABLE(page->dparent), FALSE);
-	gtk_table_attach(GTK_TABLE(table), page->dparent, 1, 2, 1, 2,  GTK_EXPAND|GTK_FILL,0,0,0);
+	gtk_grid_attach(GTK_GRID(grid), page->dparent, 1, 1, 1, 1);
 	page->dreparent = gui_details_button_new();
 	g_signal_connect(G_OBJECT(page->dreparent), "clicked", G_CALLBACK(evt_style_parent_clicked), page);
-	gtk_table_attach(GTK_TABLE(table), page->dreparent, 2, 3, 1, 2,  0,0,0,0);
-	gtk_box_pack_start(GTK_BOX(page->dvbox), table, FALSE, FALSE, 0);
+	gtk_grid_attach(GTK_GRID(grid), page->dreparent, 2, 1, 1, 1);
+	gtk_box_pack_start(GTK_BOX(page->dvbox), grid, FALSE, FALSE, 0);
 
 	frame = gtk_frame_new(_("Inherited Properties"));
 	page->dpnbook = gtk_notebook_new();
@@ -849,7 +908,7 @@ static GtkWidget * cst_init(MainInfo *min, gchar **name)
 	gtk_box_pack_start(GTK_BOX(page->dvbox), frame, FALSE, FALSE, 5);
 	gtk_box_pack_start(GTK_BOX(page->vbox), page->dvbox, FALSE, FALSE, 0);
 
-	hbox = gtk_hbox_new(FALSE, 0);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	btn = gtk_button_new_with_label(_("Add"));
 	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(evt_style_add_clicked), page);
 	gtk_box_pack_start(GTK_BOX(hbox), btn, TRUE, TRUE, 5);
