@@ -39,11 +39,11 @@ typedef struct {
 	GtkListStore	*store;		/* Tree model holding existing types. */
 	GtkWidget	*view;		/* Tree view. */
 
-	GtkWidget	*ntable;	/* Table holding the type name widgetry. */
+	GtkWidget	*ngrid;		/* Grid holding the type name widgetry. */
 	GtkWidget	*name;		/* Entry widget for type name. */
 
 	GtkWidget	*iframe;	/* Identification frame. */
-	GtkWidget	*itable;	/* This holds all the identification widgets. */
+	GtkWidget	*igrid;		/* This holds all the identification widgets. */
 	GtkWidget	*ithbox;	/* Hbox for the 'type' id row. */
 	GtkWidget	*itype[7];	/* Radio buttons for the type selection. Boring. */
 	GSList		*itlist;	/* Radio button grouping list. */
@@ -137,7 +137,7 @@ static void set_widgets(P_Types *page, FType *tpe, GtkTreeIter *iter)
 	itext[2] = tpe->file_re_src;
 
 	gtk_entry_set_text(GTK_ENTRY(page->name), tpe->name);
-	gtk_widget_set_sensitive(page->ntable, TRUE);
+	gtk_widget_set_sensitive(page->ngrid, TRUE);
 
 	for(i = 0; i < 7; i++)
 	{
@@ -206,7 +206,7 @@ static void reset_widgets(P_Types *page)
 	page->curr_type = NULL;
 	gtk_tree_selection_unselect_all(gtk_tree_view_get_selection(GTK_TREE_VIEW(page->view)));
 	gtk_entry_set_text(GTK_ENTRY(page->name), "");
-	gtk_widget_set_sensitive(page->ntable, FALSE);
+	gtk_widget_set_sensitive(page->ngrid, FALSE);
 
 	gtk_widget_set_sensitive(page->iphbox, FALSE);
 	for(i = 0; i < 6; i++)
@@ -657,7 +657,7 @@ static GtkWidget * ctp_init(MainInfo *min, gchar **name)
 	page->curr_type = NULL;
 	page->modified  = FALSE;
 
-	page->vbox = gtk_vbox_new(FALSE, 0);
+	page->vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 
 	page->handlers = gui_handler_group_new();
 
@@ -674,21 +674,23 @@ static GtkWidget * ctp_init(MainInfo *min, gchar **name)
         gtk_container_add(GTK_CONTAINER(scwin), page->view);
         gtk_box_pack_start(GTK_BOX(page->vbox), scwin, TRUE, TRUE, 0);
 
-	page->ntable = gtk_table_new(2, 2, FALSE);
+	page->ngrid = gtk_grid_new();
 	label = gtk_label_new(_("Name"));
-	gtk_table_attach(GTK_TABLE(page->ntable), label, 0, 1, 0, 1,  GTK_FILL,0,5,0);
+	gtk_grid_attach(GTK_GRID(page->ngrid), label, 0, 0, 1, 1);
 	page->name = gtk_entry_new();
+	gtk_widget_set_hexpand(page->name, TRUE);
+	gtk_widget_set_halign(page->name, GTK_ALIGN_FILL);
 	gtk_entry_set_max_length(GTK_ENTRY(page->name), STL_STYLE_NAME_SIZE - 1);
 	gui_handler_group_connect(page->handlers, G_OBJECT(page->name), "changed", G_CALLBACK(evt_name_changed), page);
-	gtk_table_attach_defaults(GTK_TABLE(page->ntable), page->name, 1, 2, 0, 1);
+	gtk_grid_attach(GTK_GRID(page->ngrid), page->name, 1, 0, 1, 1);
 
-	gtk_box_pack_start(GTK_BOX(page->vbox), page->ntable, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(page->vbox), page->ngrid, FALSE, FALSE, 0);
 
 	page->iframe = gtk_frame_new(_("Identification"));
-	page->itable = gtk_table_new(7, 3, FALSE);
+	page->igrid = gtk_grid_new();
 	label = gtk_label_new(_("Require Type"));
-	gtk_table_attach(GTK_TABLE(page->itable), label, 0, 1, 0, 1,  GTK_FILL,0,0,0);
-	page->ithbox = gtk_hbox_new(FALSE, 0);
+	gtk_grid_attach(GTK_GRID(page->igrid), label, 0, 0, 1, 1);
+	page->ithbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 
 	page->itlist = gui_radio_group_new(7, tlab, page->itype);
 	page->itmode = type;
@@ -698,12 +700,12 @@ static GtkWidget * ctp_init(MainInfo *min, gchar **name)
 		g_object_set_data(G_OBJECT(page->itype[i]), "user", GINT_TO_POINTER(i));
 		gtk_box_pack_start(GTK_BOX(page->ithbox), page->itype[i], TRUE, TRUE, 0);
 	}
-	gtk_table_attach(GTK_TABLE(page->itable), page->ithbox, 1, 3, 0, 1, GTK_EXPAND|GTK_FILL,0,0,0);
+	gtk_grid_attach(GTK_GRID(page->igrid), page->ithbox, 1, 0, 2, 1);
 
 	page->irperm = gtk_check_button_new_with_label(_("Require Protection"));
 	gui_handler_group_connect(page->handlers, G_OBJECT(page->irperm), "clicked", G_CALLBACK(evt_id_reqperm_clicked), page);
-	gtk_table_attach(GTK_TABLE(page->itable), page->irperm, 0, 1, 1, 2,  GTK_FILL,0,0,0);
-	page->iphbox = gtk_hbox_new(FALSE, 0);
+	gtk_grid_attach(GTK_GRID(page->igrid), page->irperm, 0, 1, 1, 1);
+	page->iphbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	for(i = 0; i < 6; i++)
 	{
 		page->iperm[i] = gtk_toggle_button_new_with_label(_(iplab[i]));
@@ -711,10 +713,12 @@ static GtkWidget * ctp_init(MainInfo *min, gchar **name)
 		gui_handler_group_connect(page->handlers, G_OBJECT(page->iperm[i]), "clicked", G_CALLBACK(evt_id_perm_clicked), page);
 		gtk_box_pack_start(GTK_BOX(page->iphbox), page->iperm[i], TRUE, TRUE, 0);
 	}
-	gtk_table_attach(GTK_TABLE(page->itable), page->iphbox, 1, 3, 1, 2,  GTK_EXPAND | GTK_FILL,0,0,0);
+	gtk_widget_set_hexpand(page->iphbox, TRUE);
+	gtk_widget_set_halign(page->iphbox, GTK_ALIGN_FILL);
+	gtk_grid_attach(GTK_GRID(page->igrid), page->iphbox, 1, 1, 2, 1);
 
-	sep = gtk_hseparator_new();
-	gtk_table_attach(GTK_TABLE(page->itable), sep, 0, 3, 4, 5, GTK_EXPAND | GTK_FILL,0,0,5);
+	sep = gtk_separator_new(GTK_ORIENTATION_HORIZONTAL);
+	gtk_grid_attach(GTK_GRID(page->igrid), sep, 0, 4, 3, 1);
 
 	for(i = 0; i < 3; i++)
 	{
@@ -723,19 +727,19 @@ static GtkWidget * ctp_init(MainInfo *min, gchar **name)
 		page->ident[i].check = gtk_check_button_new_with_label(_(idlab[i]));
 		g_object_set_data(G_OBJECT(page->ident[i].check), "user", GINT_TO_POINTER(i));
 		gui_handler_group_connect(page->handlers, G_OBJECT(page->ident[i].check), "clicked", G_CALLBACK(evt_id_check_clicked), page);
-		gtk_table_attach(GTK_TABLE(page->itable), page->ident[i].check, 0, 1, y, y+1,  GTK_FILL,0,0,0);
+		gtk_grid_attach(GTK_GRID(page->igrid), page->ident[i].check, 0, y, 1, 1);
 
 		page->ident[i].entry = gtk_entry_new();
 		gtk_entry_set_max_length(GTK_ENTRY(page->ident[i].entry), idmax[i] - 1);
 		g_object_set_data(G_OBJECT(page->ident[i].entry), "user", GINT_TO_POINTER(i));
 		gui_handler_group_connect(page->handlers, G_OBJECT(page->ident[i].entry), "changed", G_CALLBACK(evt_id_entry_changed), page);
-		gtk_table_attach(GTK_TABLE(page->itable), page->ident[i].entry, 1, (i == 0) ? 3 : 2, y, y+1,  GTK_EXPAND | GTK_FILL,0,0,0);
+		gtk_grid_attach(GTK_GRID(page->igrid), page->ident[i].entry, 1, y, (i == 0) ? 2 : 1, 1);
 
 		if(i >= 1)
 		{
 			GtkWidget	*hbox;
 
-			hbox = gtk_hbox_new(FALSE, 0);
+			hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 			page->ident[i].glob = gtk_check_button_new_with_label(_("Glob?"));
 			g_object_set_data(G_OBJECT(page->ident[i].glob), "user", GINT_TO_POINTER(i));
 			gui_handler_group_connect(page->handlers, G_OBJECT(page->ident[i].glob), "clicked", G_CALLBACK(evt_id_glob_changed), page);
@@ -746,43 +750,43 @@ static GtkWidget * ctp_init(MainInfo *min, gchar **name)
 			gui_handler_group_connect(page->handlers, G_OBJECT(page->ident[i].nocase), "clicked", G_CALLBACK(evt_id_nocase_changed), page);
 			gtk_box_pack_start(GTK_BOX(hbox), page->ident[i].nocase, FALSE, FALSE, 0);
 
-			gtk_table_attach(GTK_TABLE(page->itable), hbox, 2, 3, y, y + 1, GTK_SHRINK,0,0,0);
+			gtk_grid_attach(GTK_GRID(page->igrid), hbox, 2, y, 1, 1);
 		}
 	}
-	gtk_container_add(GTK_CONTAINER(page->iframe), page->itable);
+	gtk_container_add(GTK_CONTAINER(page->iframe), page->igrid);
 	gtk_box_pack_start(GTK_BOX(page->vbox), page->iframe, FALSE, FALSE, 5);
 
 	page->sframe = gtk_frame_new(_("Type's Style"));
-	page->shbox = gtk_hbox_new(FALSE, 0);
+	page->shbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	label = gtk_label_new(_("Style"));
 	gtk_box_pack_start(GTK_BOX(page->shbox), label, FALSE, FALSE, 5);
 	page->sbtn = gtk_button_new();
 	gui_handler_group_connect(page->handlers, G_OBJECT(page->sbtn), "clicked", G_CALLBACK(evt_style_clicked), page);
 
-	page->sbhbox = gtk_hbox_new(FALSE, 0);
+	page->sbhbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	page->sicon  = NULL;
 	page->slabel = gtk_label_new("");
 	gtk_box_pack_start(GTK_BOX(page->sbhbox), page->slabel, TRUE, TRUE, 0);
 	gtk_container_add(GTK_CONTAINER(page->sbtn), page->sbhbox);
 
 	gtk_box_pack_start(GTK_BOX(page->shbox), page->sbtn, TRUE, TRUE, 5);
-	vbox = gtk_vbox_new(FALSE, 0);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	gtk_box_pack_start(GTK_BOX(vbox), page->shbox, FALSE, FALSE, 5);
 	gtk_container_add(GTK_CONTAINER(page->sframe), vbox);
 
 	gtk_box_pack_start(GTK_BOX(page->vbox), page->sframe, FALSE, FALSE, 5);
 
-	page->bhbox = gtk_hbox_new(FALSE, 0);
+	page->bhbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	page->add = gtk_button_new_with_label(_("Add"));
 	g_object_set_data(G_OBJECT(page->add), "user", &min->cfg);
 	g_signal_connect(G_OBJECT(page->add), "clicked", G_CALLBACK(evt_add_clicked), page);
 	gtk_box_pack_start(GTK_BOX(page->bhbox), page->add, TRUE, TRUE, 5);
 
-	page->up = gui_arrow_button_new(GTK_ARROW_UP);
+	page->up = gtk_button_new_from_icon_name("go-up", GTK_ICON_SIZE_MENU);
 	g_signal_connect(G_OBJECT(page->up), "clicked", G_CALLBACK(evt_up_clicked), page);
 	gtk_box_pack_start(GTK_BOX(page->bhbox), page->up, FALSE, FALSE, 0);
 
-	page->down = gui_arrow_button_new(GTK_ARROW_DOWN);
+	page->down = gtk_button_new_from_icon_name("go-down", GTK_ICON_SIZE_MENU);
 	g_signal_connect(G_OBJECT(page->down), "clicked", G_CALLBACK(evt_down_clicked), page);
 	gtk_box_pack_start(GTK_BOX(page->bhbox), page->down, FALSE, FALSE, 0);
 
@@ -816,10 +820,11 @@ static void ctp_update(MainInfo *min)
 static void ctp_accept(MainInfo *min)
 {
 	P_Types	*page = &the_page;
-	GList	*iter;
 
 	if(page->modified)
 	{
+		const GList	*iter;
+
 		for(iter = min->cfg.type; iter != NULL; iter = g_list_next(iter))
 			typ_type_destroy(iter->data);
 		g_list_free(min->cfg.type);
@@ -1004,16 +1009,15 @@ void ctp_replace_style(const gchar *from, Style *to)
 **		This gets called by the cfg_styles module, just before it destroys the <from>
 **		styles.
 */
-void ctp_relink_styles(StyleInfo *from, StyleInfo *to)
+void ctp_relink_styles(const StyleInfo *from, const StyleInfo *to)
 {
-	GList	*iter;
-	FType	*type;
-	Style	*ns;
+	const GList	*iter;
 
 	for(iter = the_page.type; iter != NULL; iter = g_list_next(iter))
 	{
-		type = iter->data;
-		ns   = stl_styleinfo_style_find(to, stl_style_get_name(type->style));
+		FType	*type = iter->data;
+		Style	*ns = stl_styleinfo_style_find(to, stl_style_get_name(type->style));
+
 		if(type->style != ns)
 		{
 			type->style = ns;
