@@ -61,7 +61,7 @@ static void isearch_close(void)
 /* 2003-11-04 -	Handle keypresses. Traps special keys for handy functionality. */
 static gint evt_isearch_keypress(GtkWidget *wid, GdkEventKey *evt, gpointer user)
 {
-	if(evt->keyval == GDK_Escape || (evt->keyval == GDK_g && (evt->state & GDK_CONTROL_MASK) != 0))
+	if(evt->keyval == GDK_KEY_Escape || (evt->keyval == GDK_KEY_g && (evt->state & GDK_CONTROL_MASK) != 0))
 	{
 		isearch_close();
 		return TRUE;
@@ -159,7 +159,7 @@ static GtkWidget ** widgetry_builder(MainInfo *min)
 	ISearchWidgetry	*wid;
 
 	wid = g_malloc(sizeof *wid);
-	wid->hbox = gtk_hbox_new(FALSE, 0);
+	wid->hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	wid->label = gtk_label_new(_("ISearch"));
 	gtk_box_pack_start(GTK_BOX(wid->hbox), wid->label, FALSE, FALSE, 5);
 	wid->entry = gtk_entry_new();
