@@ -76,20 +76,19 @@ static void dir_size_update(const FUCount *count, gpointer user)
 	gtk_label_set_text(GTK_LABEL(dsi->contents), buf);
 }
 
-static GtkWidget * label_pack(GtkWidget *table, const gchar *text, gint x, gint y, gboolean left)
+static GtkWidget * label_pack(GtkWidget *grid, const gchar *text, gint x, gint y, gboolean left)
 {
-	GtkWidget	*algn, *lab;
+	GtkWidget	*lab;
 
-	algn = gtk_alignment_new(left ? 0.0f : 0.9f, 0.0f, 0.0f, 0.0f);
 	lab  = gtk_label_new(text);
-	if(left)
-		gtk_label_set_justify(GTK_LABEL(lab), GTK_JUSTIFY_LEFT);
-	gtk_container_add(GTK_CONTAINER(algn), lab);
-	gtk_table_attach(GTK_TABLE(table), algn, x, x + 1, y, y + 1, GTK_FILL,0,4,0);
+	gtk_misc_set_alignment(GTK_MISC(lab), left ? 0.f : 0.9f, 0.f);
+	gtk_widget_set_hexpand(lab, TRUE);
+	gtk_grid_attach(GTK_GRID(grid), lab, x, y, 1, 1);
+
 	return lab;
 }
 
-static gint build_date(const gchar *label, gint y, guint64 date, const gchar *fmt, GtkWidget *tab)
+static gint build_date(const gchar *label, gint y, guint64 date, const gchar *fmt, GtkWidget *grid)
 {
 	gchar		buf[64];
 	time_t		tvalue;
@@ -98,9 +97,9 @@ static gint build_date(const gchar *label, gint y, guint64 date, const gchar *fm
 	tvalue = (time_t) date;
 	if(tvalue != 0 && (tm = localtime(&tvalue)) != NULL)
 	{
-		label_pack(tab, label, 0, y, FALSE);
-		if(strftime(buf, sizeof buf, fmt, tm) >= 0)
-			label_pack(tab, buf, 1, y++, TRUE);
+		label_pack(grid, label, 0, y, FALSE);
+		if(strftime(buf, sizeof buf, fmt, tm) > 0)
+			label_pack(grid, buf, 1, y++, TRUE);
 	}
 	return y;
 }
@@ -168,11 +167,17 @@ static GtkWidget * build_info(MainInfo *min, const DirPane *dp, const DirRow2 *r
 	gchar			buf[128], sbuf[32], *ptr;
 	guint			y;
 	const gchar		*iname;
-	GtkWidget		*nbook, *tab, *szlab, *sep, *entry, *gp;
+	GtkWidget		*nbook, *grid, *szlab, *sep, *entry, *gp;
 
 	nbook = gtk_notebook_new();
 
-	tab = gtk_table_new(17, 2, FALSE);
+	grid = gtk_grid_new();
+	gtk_widget_set_margin_start(grid, 5);
+	gtk_widget_set_margin_end(grid, 5);
+	gtk_widget_set_margin_top(grid, 5);
+	gtk_widget_set_margin_bottom(grid, 5);
+	gtk_grid_set_column_spacing(GTK_GRID(grid), 5);
+	gtk_grid_set_row_spacing(GTK_GRID(grid), 5);
 
 	y = 0;
 	if(dp_row_get_ftype(m, row)->style != NULL)
@@ -186,39 +191,41 @@ static GtkWidget * build_info(MainInfo *min, const DirPane *dp, const DirRow2 *r
 				GtkWidget	*img;
 
 				img = gtk_image_new_from_pixbuf(pbuf);
-				gtk_table_attach(GTK_TABLE(tab), img, 0, 1, y, y + 1, 0,0,5,5);
+				gtk_grid_attach(GTK_GRID(grid), img, 0, y, 1, 1);
 			}
 		}
 	}
-	label_pack(tab, dp_row_get_name_display(m, row), 1, y++, TRUE);
+	label_pack(grid, dp_row_get_name_display(m, row), 1, y++, TRUE);
 
 	/* For symbolic links, display name of the target, at least. */
 	if(dp_row_get_file_type(m, row, FALSE) == G_FILE_TYPE_SYMBOLIC_LINK)
 	{
 		GtkWidget	*hbox, *icon;
 
-		label_pack(tab, _("Link To"), 0, y, FALSE);
-		hbox = gtk_hbox_new(FALSE, 0);
+		label_pack(grid, _("Link To"), 0, y, FALSE);
+		hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 		entry = gtk_entry_new();
 		gtk_widget_set_sensitive(entry, FALSE);	/* Used to be just non-editable, but ... this looks better. */
 		gtk_entry_set_text(GTK_ENTRY(entry), dp_row_get_link_target(m, row));
 		gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
-		icon = gtk_image_new_from_stock(dp_row_get_flags(m, row, DPRF_LINK_EXISTS) ? GTK_STOCK_YES : GTK_STOCK_NO, GTK_ICON_SIZE_BUTTON);
+		icon = gtk_image_new_from_icon_name(dp_row_get_flags(m, row, DPRF_LINK_EXISTS) ? "face-smile" : "face-sad", GTK_ICON_SIZE_BUTTON);
 		gtk_box_pack_start(GTK_BOX(hbox), icon, FALSE, FALSE, 0);
-		gtk_table_attach(GTK_TABLE(tab), hbox, 1, 2, y, y + 1,  GTK_EXPAND|GTK_FILL,0,5,0);
+		gtk_widget_set_hexpand(hbox, TRUE);
+		gtk_widget_set_halign(hbox, GTK_ALIGN_FILL);
+		gtk_grid_attach(GTK_GRID(grid), hbox, 1, y, 1, 1);
 		y++;
 	}
-	sep = gtk_hseparator_new();
-	gtk_table_attach(GTK_TABLE(tab), sep, 0, 2, y, y + 1, GTK_EXPAND | GTK_FILL,0,0,5);
+	sep = gtk_separator_new(GTK_ORIENTATION_HORIZONTAL);
+	gtk_grid_attach(GTK_GRID(grid), sep, 0, y, 2, 1);
 	y++;
 
-	label_pack(tab, _("Type"), 0, y, FALSE);
-	label_pack(tab, dp_row_get_ftype(m, row)->name, 1, y++, TRUE);
+	label_pack(grid, _("Type"), 0, y, FALSE);
+	label_pack(grid, dp_row_get_ftype(m, row)->name, 1, y++, TRUE);
 
-	label_pack(tab, _("Location"), 0, y, FALSE);
-	label_pack(tab, dp->dir.path, 1, y++, TRUE);
+	label_pack(grid, _("Location"), 0, y, FALSE);
+	label_pack(grid, dp->dir.path, 1, y++, TRUE);
 
-	label_pack(tab, _("Size"), 0, y, FALSE);
+	label_pack(grid, _("Size"), 0, y, FALSE);
 	sze_put_offset(sbuf, sizeof sbuf, dp_row_get_size(m, row), SZE_AUTO, 3, ',');
 	if(dp_row_get_size(m, row) < 1024)
 		g_strlcpy(buf, sbuf, sizeof buf);
@@ -230,7 +237,7 @@ static GtkWidget * build_info(MainInfo *min, const DirPane *dp, const DirRow2 *r
 		st = stu_tickify(stbuf + sizeof stbuf - 1, dp_row_get_size(m, row), info_options.tick[0]);
 		g_snprintf(buf, sizeof buf, _("%s (%s bytes)"), sbuf, st);
 	}
-	szlab = label_pack(tab, buf, 1, y++, TRUE);
+	szlab = label_pack(grid, buf, 1, y++, TRUE);
 	/* Building info for a directory, and recursing enabled? */
 	if(dp_row_get_file_type(m, row, TRUE) == G_FILE_TYPE_DIRECTORY && info_options.recurse_dirs)
 	{
@@ -239,8 +246,8 @@ static GtkWidget * build_info(MainInfo *min, const DirPane *dp, const DirRow2 *r
 		/* Start asynchronous directory sizing. Nice. */
 		*dsi = g_malloc(sizeof *dsi);
 		(*dsi)->size     = szlab;
-		label_pack(tab, _("Contains"), 0, y, FALSE);
-		(*dsi)->contents = label_pack(tab, "(content info)", 1, y++, TRUE);
+		label_pack(grid, _("Contains"), 0, y, FALSE);
+		(*dsi)->contents = label_pack(grid, "(content info)", 1, y++, TRUE);
 		(*dsi)->handle   = fut_dir_size_gfile_start(dir, dir_size_update, *dsi);
 	}
 
@@ -253,43 +260,43 @@ static GtkWidget * build_info(MainInfo *min, const DirPane *dp, const DirRow2 *r
 
 		if((path = g_file_get_path(dp_get_file_from_row(dp, row))) != NULL)
 		{
-			sep = gtk_hseparator_new();
-			gtk_table_attach(GTK_TABLE(tab), sep, 0, 2, y, y + 1, GTK_EXPAND | GTK_FILL,0,0,5);
+			sep = gtk_separator_new(GTK_ORIENTATION_HORIZONTAL);
+			gtk_grid_attach(GTK_GRID(grid), sep, 0, y, 2, 1);
 			y++;
-			label_pack(tab, _("'File' Info"), 0, y, FALSE);
-			label_pack(tab, fle_file(min, path), 1, y++, TRUE);
+			label_pack(grid, _("'File' Info"), 0, y, FALSE);
+			label_pack(grid, fle_file(min, path), 1, y++, TRUE);
 			g_free(path);
 		}
 	}
 #endif
-	sep = gtk_hseparator_new();
-	gtk_table_attach(GTK_TABLE(tab), sep, 0, 2, y, y + 1, GTK_EXPAND | GTK_FILL,0,0,5);
+	sep = gtk_separator_new(GTK_ORIENTATION_HORIZONTAL);
+	gtk_grid_attach(GTK_GRID(grid), sep, 0, y, 2, 1);
 	y++;
 
-	label_pack(tab, _("Owner"), 0, y, FALSE);
+	label_pack(grid, _("Owner"), 0, y, FALSE);
 	if((ptr = (gchar *) usr_lookup_uname(dp_row_get_uid(m, row))) != NULL)
 		g_snprintf(buf, sizeof buf, "%s (%d)", ptr, (gint) dp_row_get_uid(m, row));
 	else
 		g_snprintf(buf, sizeof buf, "%d", (gint) dp_row_get_uid(m, row));
-	label_pack(tab, buf, 1, y++, TRUE);
+	label_pack(grid, buf, 1, y++, TRUE);
 
-	label_pack(tab, _("Group"), 0, y, FALSE);
+	label_pack(grid, _("Group"), 0, y, FALSE);
 	if((ptr = (gchar *) usr_lookup_gname(dp_row_get_gid(m, row))) != NULL)
 		g_snprintf(buf, sizeof buf, "%s (%d)", ptr, (gint) dp_row_get_gid(m, row));
 	else
 		g_snprintf(buf, sizeof buf, "%d", (gint) dp_row_get_gid(m, row));
-	label_pack(tab, buf, 1, y++, TRUE);
+	label_pack(grid, buf, 1, y++, TRUE);
 
-	sep = gtk_hseparator_new();
-	gtk_table_attach(GTK_TABLE(tab), sep, 0, 2, y, y + 1, GTK_EXPAND | GTK_FILL,0,0,5);
+	sep = gtk_separator_new(GTK_ORIENTATION_HORIZONTAL);
+	gtk_grid_attach(GTK_GRID(grid), sep, 0, y, 2, 1);
 	y++;
 
-	y = build_date(_("Accessed"), y, dp_row_get_time_accessed(m, row), info_options.df_access, tab);
-	y = build_date(_("Modified"), y, dp_row_get_time_modified(m, row), info_options.df_modify, tab);
-	y = build_date(_("Changed"),  y, dp_row_get_time_changed(m, row), info_options.df_change, tab);
-	y = build_date(_("Created"),  y, dp_row_get_time_created(m, row), info_options.df_change, tab);
+	y = build_date(_("Accessed"), y, dp_row_get_time_accessed(m, row), info_options.df_access, grid);
+	y = build_date(_("Modified"), y, dp_row_get_time_modified(m, row), info_options.df_modify, grid);
+	y = build_date(_("Changed"),  y, dp_row_get_time_changed(m, row), info_options.df_change, grid);
+	y = build_date(_("Created"),  y, dp_row_get_time_created(m, row), info_options.df_change, grid);
 
-	gtk_notebook_append_page(GTK_NOTEBOOK(nbook), tab, gtk_label_new(_("Basic")));
+	gtk_notebook_append_page(GTK_NOTEBOOK(nbook), grid, gtk_label_new(_("Basic")));
 
 	if((gp = build_gio_info(dp, row)) != NULL)
 	{
