@@ -97,7 +97,7 @@ static gboolean do_join(MainInfo *min, DirPane *src, DirPane *dst, GtkListStore
 	if((outs = g_file_create(out, G_FILE_CREATE_NONE, NULL, &err)) != NULL)
 	{
 		GtkTreeIter		iter;
-		gsize			put;
+		gssize			put;
 		gchar			*buf;
 		const gsize		buf_size = cpy_get_buf_size();
 
@@ -118,6 +118,7 @@ static gboolean do_join(MainInfo *min, DirPane *src, DirPane *dst, GtkListStore
 					if((ins = g_file_read(in, NULL, &err)) != NULL)
 					{
 						const gsize	size = dp_row_get_size(dp_get_tree_model(src), dr);
+
 						pgs_progress_item_begin(min, dp_row_get_name_display(dp_get_tree_model(src), dr), size);
 						put = chunked_splice(min, G_INPUT_STREAM(ins), G_OUTPUT_STREAM(outs), size, buf, buf_size, &err);
 						if(put < 0 || !g_input_stream_close(G_INPUT_STREAM(ins), NULL, &err))
@@ -199,7 +200,7 @@ gint cmd_join(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 			const gchar		*first = NULL, *tail;
 			gunichar		ch;
 
-			vbox  = gtk_vbox_new(FALSE, 0);
+			vbox  = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 			label = gtk_label_new(_("Click and Drag Files to Reorder, Then Click Join."));
 			gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
 
