@@ -67,11 +67,12 @@ static gboolean move_gfile_directory(MainInfo *min, DirPane *src, DirPane *dst,
 
 gboolean move_gfile(MainInfo *min, DirPane *src, DirPane *dst, const GFile *sfile, const GFile *dfile, gboolean progress, GError **err)
 {
-	gboolean	ok;
 	GFileInfo	*fi;
+	gboolean	ok;
 
 	if(src == NULL || dst == NULL || sfile == NULL || dfile == NULL)
 		return FALSE;
+
 	switch(ovw_overwrite_unary_file(dst, dfile))
 	{
 	case OVW_SKIP:
@@ -85,25 +86,40 @@ gboolean move_gfile(MainInfo *min, DirPane *src, DirPane *dst, const GFile *sfil
 		if(!del_delete_gfile(min, (GFile *) dfile, FALSE, err))
 			return FALSE;
 	}
-	/* Grab the FileInfo, gives handy access to both name and size. */
 	if((fi = g_file_query_info((GFile *) sfile, "standard::*", G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS, NULL, err)) == NULL)
 		return FALSE;
+	/* First try a native move. This is so fast, that we don't want to spend time recursing
+	** to figure out the true size of directories. Instead, use the directory entry's size.
+	*/
 	pgs_progress_item_begin(min, g_file_info_get_display_name(fi), g_file_info_get_size(fi));
 	g_object_unref(fi);
 
-	ok = g_file_move((GFile *) sfile, (GFile *) dfile, G_FILE_COPY_NOFOLLOW_SYMLINKS, NULL, cb_progress, min, err);
-	pgs_progress_item_end(min);
-	/* If it failed, check if it was because it "would recurse". If so, then we need to
-	** implement a fall-back, since GIO doesn't do that for us. Luckily, this whole
-	** program is supposed to be a _file manager_, so it's right up our alley.
-	*/
-	if(!ok && err != NULL && *err != NULL && (*err)->code == G_IO_ERROR_WOULD_RECURSE)
+	ok = g_file_move((GFile *) sfile, (GFile *) dfile, G_FILE_COPY_NOFOLLOW_SYMLINKS | G_FILE_COPY_NO_FALLBACK_FOR_MOVE, pgs_progress_get_cancellable(), cb_progress, min, err);
+	if(!ok && !(err != NULL && g_error_matches(*err, G_IO_ERROR, G_IO_ERROR_CANCELLED)))
 	{
-		/* Drop the GError, since we've handled it and it was semi-expected. */
-		g_error_free(*err);
-		*err = NULL;
-		ok = move_gfile_directory(min, src, dst, sfile, dfile, progress, err);
+		guint64	size = 0;
+
+		/* Initial native-only attempt failed, try again with the fallback enabled.
+		** At this point we first need to figure out the size, for the progress reporting.
+		*/
+		if(fut_size_gfile(min, sfile, &size, NULL, err))
+		{
+			g_clear_error(err);
+			pgs_progress_item_resize(min, size);
+			ok = g_file_move((GFile *) sfile, (GFile *) dfile, G_FILE_COPY_NOFOLLOW_SYMLINKS, pgs_progress_get_cancellable(), cb_progress, min, err);
+			/* If it failed, check if it was because it "would recurse". If so, then we need to
+			** implement a fall-back, since GIO doesn't do that for us. Luckily, this whole
+			** program is supposed to be a _file manager_, so it's right up our alley.
+			*/
+			if(!ok && err != NULL && g_error_matches(*err, G_IO_ERROR, G_IO_ERROR_WOULD_RECURSE))
+			{
+				/* Drop the GError, since we've handled it and it was semi-expected. */
+				g_clear_error(err);
+				ok = move_gfile_directory(min, src, dst, sfile, dfile, progress, err);
+			}
+		}
 	}
+	pgs_progress_item_end(min);
 
 	return ok;
 }
@@ -126,7 +142,7 @@ gint cmd_move(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 
 	err_clear(min);
 	ovw_overwrite_begin(min, _("\"%s\" Already Exists - Continue With Move?"), 0UL);
-	pgs_progress_begin(min, _("Moving..."), PFLG_COUNT_RECURSIVE | PFLG_ITEM_VISIBLE | PFLG_BYTE_VISIBLE);
+	pgs_progress_begin(min, _("Moving..."), PFLG_ITEM_VISIBLE | PFLG_BYTE_VISIBLE);
 	for(iter = slist; ok && (iter != NULL); iter = g_slist_next(iter))
 	{
 		sfile = dp_get_file_from_row(src, iter->data);
