@@ -11,6 +11,7 @@
 #include "dialog.h"
 #include "cmdarg.h"
 #include "cmdseq.h"
+#include "cmd_configure.h"
 #include "configure.h"
 
 #include "cmd_quit.h"
@@ -32,9 +33,15 @@ gint cmd_quit(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 	{
 		gint	res;
 
-		res = dlg_dialog_sync_new_simple_wait(_("You may have some unsaved configuration changes.\n"
-							"Quitting without saving will lose them. Really quit?"),
-							_("Confirm Quitting"), _("_Quit|_Save, then Quit|_Cancel"));
+		if(cmd_configure_autosave())
+			res = 1;
+		else
+		{
+			res = dlg_dialog_sync_new_simple_wait(_("You may have some unsaved configuration changes.\n"
+								"Quitting without saving will lose them. Really quit?"),
+								_("Confirm Quitting"), _("_Quit|_Save, then Quit|_Cancel"));
+		}
+
 		if(res == -1)
 			return TRUE;
 		if(res == 1)
