@@ -109,7 +109,7 @@ static gboolean do_renameseq(RenSeqInfo *rsi, DirPane *src, GError **err)
 	GSList		*slist, *iter, *tlist = NULL;
 	GString		*tmppfx;
 	gboolean	ok = TRUE;
-	guint		num, index;
+	guint		index;
 	GtkTreeModel	*m = dp_get_tree_model(src);
 
 	/* A Rename in gentoo is never a move. Gotta keep'em separated. */
@@ -118,7 +118,6 @@ static gboolean do_renameseq(RenSeqInfo *rsi, DirPane *src, GError **err)
 		return FALSE;
 
 	slist = dp_get_selection(src);
-	num   = g_slist_length(slist);
 	tmppfx = g_string_sized_new(64);
 	g_string_append_printf(tmppfx, ".gt!%u:%x", (guint) getpid(), (guint) time(NULL) % 1007);
 	/* Give all files temporary names, so we can handle overlaps in sequencing. */
@@ -280,8 +279,8 @@ gint cmd_renameseq(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
 	static RenSeqInfo	rsi = { NULL };
 	const gchar		*bname[] = { N_("8, Octal"), N_("10, Decimal"), N_("16, Hex (a-f)"), N_("16, Hex (A-F)") };
-	GtkObject		*adj;
-	GtkWidget		*vbox, *table, *label;
+	GtkAdjustment		*adj;
+	GtkWidget		*vbox, *grid, *label;
 	RenSeqDialog		dlg;
 	gint			i, ret;
 	GSList			*sel;
@@ -303,66 +302,66 @@ gint cmd_renameseq(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 	}
 	dlg.rsi = &rsi;
 
-	vbox  = gtk_vbox_new(FALSE, 0);
+	vbox  = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	label = gtk_label_new(_("This command renames all selected files\n"
 				"into a numbered sequence. The controls\n"
 				"below let you define how the names are\n"
 				"formed."));
 	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
 
-	table = gtk_table_new(4, 4, FALSE);
+	grid = gtk_grid_new();
 	label = gtk_label_new(_("Start At"));
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,  0,0,0,0);
+	gtk_grid_attach(GTK_GRID(grid), label, 0, 0, 1, 1);
 	adj = gtk_adjustment_new(1.0, 0.0, 4924967295.0, 1.0, 10.0, 0.0);
 	dlg.start = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1.0, 0);
 	init_start(&rsi, dlg.start);
 	g_signal_connect(G_OBJECT(dlg.start), "changed", G_CALLBACK(evt_something_changed), &dlg);
-	gtk_table_attach(GTK_TABLE(table), dlg.start, 1, 3, 0, 1,  GTK_EXPAND|GTK_FILL,0,0,0);
+	gtk_widget_set_hexpand(dlg.start, TRUE);
+	gtk_widget_set_halign(dlg.start, GTK_ALIGN_FILL);
+	gtk_grid_attach(GTK_GRID(grid), dlg.start, 1, 0, 2, 1);
 	label = gtk_label_new("(Base 10)");
-	gtk_table_attach(GTK_TABLE(table), label, 3, 4, 0, 1,  0,0,0,0);
+	gtk_grid_attach(GTK_GRID(grid), label, 3, 0, 1, 1);
 
 	label = gtk_label_new(_("Base"));
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,  0,0,0,0);
+	gtk_grid_attach(GTK_GRID(grid), label, 0, 1, 1, 1);
 	dlg.base = gtk_combo_box_text_new();
 	for(i = 0; i < sizeof bname / sizeof bname[0]; i++)
-	{
 		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(dlg.base), _(bname[i]));
-	}
 	g_signal_connect(G_OBJECT(dlg.base), "changed", G_CALLBACK(evt_something_changed), &dlg);
-	gtk_table_attach(GTK_TABLE(table), dlg.base, 1, 2, 1, 2,  GTK_EXPAND|GTK_FILL,0,0,0);
+	gtk_grid_attach(GTK_GRID(grid), dlg.base, 1, 1, 1, 1);
 
 	label = gtk_label_new(_("Precision"));
-	gtk_table_attach(GTK_TABLE(table), label, 2, 3, 1, 2,  0,0,0,0);
+	gtk_grid_attach(GTK_GRID(grid), label, 2, 1, 1, 1);
 	adj   = gtk_adjustment_new(rsi.precision, 0.0, 10.0, 1.0, 1.0, 0.0);
 	dlg.precision = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0, 0);
 	g_signal_connect(G_OBJECT(adj), "value_changed", G_CALLBACK(evt_something_changed), &dlg);
-	gtk_table_attach(GTK_TABLE(table), dlg.precision, 3, 4, 1, 2,  GTK_EXPAND|GTK_FILL,0,0,0);
+	gtk_grid_attach(GTK_GRID(grid), dlg.precision, 3, 1, 1, 1);
 
 	label = gtk_label_new(_("Head"));
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,  0,0,0,0);
+	gtk_grid_attach(GTK_GRID(grid), label, 0, 2, 1, 1);
 	dlg.head = gui_dialog_entry_new();
 	gtk_entry_set_text(GTK_ENTRY(dlg.head), rsi.head->str);
 	g_signal_connect(G_OBJECT(dlg.head), "changed", G_CALLBACK(evt_something_changed), &dlg);
-	gtk_table_attach(GTK_TABLE(table), dlg.head, 1, 3, 2, 3,  GTK_EXPAND|GTK_FILL,0,0,0);
+	gtk_grid_attach(GTK_GRID(grid), dlg.head, 1, 2, 2, 1);
 
 	label = gtk_label_new(_("Tail"));
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 3, 4,  0,0,0,0);
+	gtk_grid_attach(GTK_GRID(grid), label, 0, 3, 1, 1);
 	dlg.tail = gui_dialog_entry_new();
 	gtk_entry_set_text(GTK_ENTRY(dlg.tail), rsi.tail->str);
 	g_signal_connect(G_OBJECT(dlg.tail), "changed", G_CALLBACK(evt_something_changed), &dlg);
-	gtk_table_attach(GTK_TABLE(table), dlg.tail, 1, 3, 3, 4,  GTK_EXPAND|GTK_FILL,0,0,0);
+	gtk_grid_attach(GTK_GRID(grid), dlg.tail, 1, 3, 2, 1);
 
 	dlg.guess = gtk_button_new_with_label(_("Guess"));
 	g_signal_connect(G_OBJECT(dlg.guess), "clicked", G_CALLBACK(evt_guess_clicked), &dlg);
-	gtk_table_attach(GTK_TABLE(table), dlg.guess, 3, 4, 2, 4,  0,GTK_EXPAND|GTK_FILL,0,0);
+	gtk_grid_attach(GTK_GRID(grid), dlg.guess, 3, 2, 1, 2);
 
 	label = gtk_label_new(_("Preview"));
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 4, 5,  0,0,0,0);
+	gtk_grid_attach(GTK_GRID(grid), label, 0, 4, 1, 1);
 	dlg.preview = gui_dialog_entry_new();
 	gtk_editable_set_editable(GTK_EDITABLE(dlg.preview), FALSE);
-	gtk_table_attach(GTK_TABLE(table), dlg.preview, 1, 4, 4, 5,  GTK_EXPAND|GTK_FILL,0,0,5);
+	gtk_grid_attach(GTK_GRID(grid), dlg.preview, 1, 4, 3, 1);
 
-	gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), grid, FALSE, FALSE, 0);
 
 	gtk_combo_box_set_active(GTK_COMBO_BOX(dlg.base), rsi.base);
 	preview_update(&dlg);
