@@ -50,6 +50,7 @@ typedef struct
 	gchar		v_step[VALUE_SIZE];
 	gchar		v_last[VALUE_SIZE];
 	gchar		v_num[VALUE_SIZE];
+	gchar		v_offset[VALUE_SIZE];
 } NameGenerator;
 
 typedef struct {
@@ -60,11 +61,11 @@ typedef struct {
 	GtkWidget	*nhbox;		/* Hbox for the naming issue. */
 	GtkWidget	*nformat;	/* Entry widget for name formatter. */
 	GtkWidget	*nfill;		/* Check box for zero-fill of number in format. */
-	GtkObject	*npadj;		/* Adjustment for precision. */
+	GtkAdjustment	*npadj;		/* Adjustment for precision. */
 	GtkWidget	*nprec;		/* Scale widget for number precision. */
-	GtkObject	*nbadj;		/* Adjustment for index base. */
+	GtkAdjustment	*nbadj;		/* Adjustment for index base. */
 	GtkWidget	*nbase;		/* Spin widget for index base. */
-	GtkObject	*nsadj;		/* Adjustment for index step. */
+	GtkAdjustment	*nsadj;		/* Adjustment for index step. */
 	GtkWidget	*nstep;		/* Spin widget for index step. */
 
 	GtkWidget	*nbook;		/* Notebook with mode-settings. */
@@ -176,6 +177,13 @@ static void namegenerator_configure(NameGenerator *sng, const SplitInfo *spi)
 	g_hash_table_insert(sng->parameters, "num", sng->v_num);
 	g_snprintf(sng->v_last, sizeof sng->v_last, sng->v_format, sng->last);
 	g_hash_table_insert(sng->parameters, "last", sng->v_last);
+	sng->v_offset[0] = '\0';
+	g_hash_table_insert(sng->parameters, "offset", sng->v_offset);
+}
+
+static void namegenerator_update_offset(NameGenerator *sng, guint64 offset)
+{
+	g_snprintf(sng->v_offset, sizeof sng->v_offset, sng->v_format, offset);
 }
 
 static gboolean namegenerator_generate(NameGenerator *sng, gchar *buf, gsize buf_size, const gchar *format, guint64 index)
@@ -215,6 +223,7 @@ static void update_preview_ss(const gchar *format, gboolean zero_fill, gint prec
 			gtk_list_store_append(spi->preview_store, &iter);
 			sze_put_offset(part, sizeof part, pos, SZE_BYTES_NO_UNIT, 0, ',');
 			gtk_list_store_set(spi->preview_store, &iter, 0, part, -1);
+			namegenerator_update_offset(spi->namegen, pos);
 			namegenerator_generate(spi->namegen, part, sizeof part, format, part_index);
 			gtk_list_store_set(spi->preview_store, &iter, 1, part, -1);
 			sze_put_offset(part, sizeof part, chunk, SZE_BYTES_NO_UNIT, 0, ',');
@@ -420,7 +429,7 @@ static GtkWidget * build_ss(SplitInfo *spi)
 	GtkWidget	*frame, *hbox, *label;
 
 	frame = gtk_frame_new(_("Fixed Size Split"));
-	hbox  = gtk_hbox_new(FALSE, 0);
+	hbox  = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	label = gtk_label_new(_("Segment Size"));
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 	spi->sssize = gtk_combo_box_text_new_with_entry();
@@ -453,7 +462,7 @@ static GtkWidget * build_fc(SplitInfo *spi)
 
 	frame = gtk_frame_new(_("Fixed Count Split"));
 
-	hbox = gtk_hbox_new(FALSE, 0);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	label = gtk_label_new(_("Segment Count"));
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 	spi->fccount = gtk_spin_button_new_with_range(2, 1000, 1);	/* Incredibly arbitrary, yes. */
@@ -504,7 +513,8 @@ static void evt_nformat_details_clicked(GtkWidget *button, gpointer user)
 	{ "base", N_("The value from the Base box") },
 	{ "step", N_("The amount that index will change for each file") },
 	{ "num", N_("The total number of files that will be created") },
-	{ "last", N_("The value of index for the last file to be created") }
+	{ "last", N_("The value of index for the last file to be created") },
+	{ "offset", N_("The part's offset into the original file") },
 	};
 	Dialog			*dlg;
 	GtkListStore		*store;
@@ -530,10 +540,10 @@ static void evt_nformat_details_clicked(GtkWidget *button, gpointer user)
 	vc = gtk_tree_view_column_new_with_attributes("(description)", cr, "text", 1, NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(view), vc);
 	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(view), FALSE);
-	gtk_widget_set_size_request(view, 380, 160);
 	gtk_container_add(GTK_CONTAINER(scwin), view);
 
 	dlg = dlg_dialog_sync_new(scwin, _("Pick Code"), NULL);
+	gtk_widget_set_size_request(GTK_WIDGET(dlg_dialog_get_dialog(dlg)), 380, 240);
 	if(dlg_dialog_sync_wait(dlg) == DLG_POSITIVE)
 	{
 		SplitInfo	*spi = user;
@@ -576,7 +586,7 @@ gint cmd_split(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 	GtkTreeViewColumn	*vc;
 
 	spi.min = min;
-	spi.vbox = gtk_vbox_new(FALSE, 0);
+	spi.vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	spi.label = gtk_label_new(_("Split"));
 
 	/* Build this early, since the signal handler references it. */
@@ -586,16 +596,16 @@ gint cmd_split(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 	gtk_notebook_append_page(GTK_NOTEBOOK(spi.nbook), build_ss(&spi), NULL);
 	gtk_notebook_append_page(GTK_NOTEBOOK(spi.nbook), build_fc(&spi), NULL);
 
-	hbox = gtk_hbox_new(FALSE, 0);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	label = gtk_label_new(_("Mode"));
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(spi.vbox), spi.label, FALSE, FALSE, 0);
-	hsep = gtk_hseparator_new();
+	hsep = gtk_separator_new(GTK_ORIENTATION_HORIZONTAL);
 	gtk_box_pack_start(GTK_BOX(spi.vbox), hsep, FALSE, FALSE, 4);
 	spi.mode = gui_build_combo_box(mode, G_CALLBACK(evt_mode_select), &spi);
 	gtk_box_pack_start(GTK_BOX(hbox), spi.mode, TRUE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(spi.vbox), hbox, FALSE, FALSE, 0);
-	spi.nhbox = gtk_hbox_new(FALSE, 0);
+	spi.nhbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	label = gtk_label_new(_("Name Format"));
 	gtk_box_pack_start(GTK_BOX(spi.nhbox), label, FALSE, FALSE, 2);
 	spi.nformat = gui_dialog_entry_new();
@@ -620,7 +630,7 @@ gint cmd_split(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 	gtk_box_pack_start(GTK_BOX(spi.nhbox), spi.nstep, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(spi.vbox), spi.nhbox, FALSE, FALSE, 2);
 
-	hbox = gtk_hbox_new(FALSE, 0);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	spi.nfill = gtk_check_button_new_with_label(_("Zero-Fill Numbers?"));
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(spi.nfill), TRUE);
 	g_signal_connect(G_OBJECT(spi.nfill), "toggled", G_CALLBACK(evt_check_changed), &spi);
@@ -628,7 +638,7 @@ gint cmd_split(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 	label = gtk_label_new("Precision");
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 	spi.npadj = gtk_adjustment_new(3, 2, 9, 1, 3, 0.);
-	spi.nprec = gtk_hscale_new(GTK_ADJUSTMENT(spi.npadj));
+	spi.nprec = gtk_scale_new(GTK_ORIENTATION_HORIZONTAL, GTK_ADJUSTMENT(spi.npadj));
 	gtk_scale_set_digits(GTK_SCALE(spi.nprec), 0);
 	gtk_scale_set_value_pos(GTK_SCALE(spi.nprec), GTK_POS_RIGHT);
 	g_signal_connect(G_OBJECT(spi.nprec), "value_changed", G_CALLBACK(evt_spin_changed), &spi);
