@@ -201,29 +201,31 @@ static void evt_pick_clicked(GtkWidget *wid, gpointer user)
 gint cmd_symlinkedit(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
 	static SleInfo	sle;
-	GtkWidget	*label, *table, *pick;
+	GtkWidget	*label, *grid, *pick;
 	Dialog		*dlg;
 	gboolean	ok = FALSE;
 
 	sle.min = min;
 	sle.src = src;
 
-	sle.vbox = gtk_vbox_new(FALSE, 0);
-	table = gtk_table_new(2, 3, FALSE);
+	sle.vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
+	grid = gtk_grid_new();
 	label = gtk_label_new(_("Name"));
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,  0,0,0,0);
+	gtk_grid_attach(GTK_GRID(grid), label, 0, 0, 1, 1);
 	sle.name = gui_dialog_entry_new();
 	gtk_entry_set_max_length(GTK_ENTRY(sle.name), FILENAME_MAX - 1);
-	gtk_table_attach(GTK_TABLE(table), sle.name, 1, 3, 0, 1,  GTK_EXPAND|GTK_FILL,0,0,0);
+	gtk_widget_set_hexpand(sle.name, TRUE);
+	gtk_widget_set_halign(sle.name, GTK_ALIGN_FILL);
+	gtk_grid_attach(GTK_GRID(grid), sle.name, 1, 0, 2, 1);
 	label = gtk_label_new(_("Contents"));
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,  0,0,0,0);
+	gtk_grid_attach(GTK_GRID(grid), label, 0, 1, 1, 1);
 	sle.contents = gui_dialog_entry_new();
 	gtk_entry_set_max_length(GTK_ENTRY(sle.contents), PATH_MAX - 1);
-	gtk_table_attach(GTK_TABLE(table), sle.contents, 1, 2, 1, 2,  GTK_EXPAND|GTK_FILL,0,0,0);
+	gtk_grid_attach(GTK_GRID(grid), sle.contents, 1, 1, 1, 1);
 	pick = gui_details_button_new();
 	g_signal_connect(G_OBJECT(pick), "clicked", G_CALLBACK(evt_pick_clicked), &sle);
-	gtk_table_attach(GTK_TABLE(table), pick, 2, 3, 1, 2,  0,0,0,0);
-	gtk_box_pack_start(GTK_BOX(sle.vbox), table, TRUE, TRUE, 0);
+	gtk_grid_attach(GTK_GRID(grid), pick, 2, 1, 1, 1);
+	gtk_box_pack_start(GTK_BOX(sle.vbox), grid, TRUE, TRUE, 0);
 
 	if(dp_has_selection(src))
 	{
