@@ -17,6 +17,7 @@
 #include <unistd.h>
 
 #include "cmdseq_config.h"
+#include "convstream.h"
 #include "dirpane.h"
 #include "errors.h"
 #include "fileutil.h"
@@ -27,8 +28,7 @@
 
 typedef struct {			/* Options for the "ViewText" command. */
 	gboolean	modified;
-	gboolean	use_mmap;			/* Use mmap() for files of non-zero length? */
-	gsize		buf_size;			/* Buffer size, when not using mmap(). */
+	gsize		buf_size;			/* Buffer size. */
 	gsize		check_size;			/* The ViewTextOrHex command inspects this many bytes. */
 	gboolean	exit_on_left;			/* Exit text reader on Left Arrow keypress? */
 } OptViewText;
@@ -38,38 +38,17 @@ static CmdCfg		*viewtext_cmc = NULL;
 
 /* ----------------------------------------------------------------------------------------- */
 
-/* 1999-03-06 -	A general viewer-trampoline. Will view all currently selected files using
-**		the given viewing function.
-*/
-static gint view_selection(MainInfo *min, DirPane *dp, gint (*viewer)(MainInfo *min, const gchar *full_name))
-{
-	GSList	*slist, *iter;
-
-	if((slist = dp_get_selection(dp)) == NULL)
-		return 1;
-	for(iter = slist; iter != NULL; iter = g_slist_next(iter))
-	{
-		viewer(min, dp_full_name(dp, DP_SEL_INDEX(dp, iter)));
-		dp_unselect(dp, DP_SEL_INDEX(dp, iter));
-	}
-	dp_free_selection(slist);
-
-	return 1;
-}
-
-/* ----------------------------------------------------------------------------------------- */
-
 /* 1998-05-19 -	Just open a window containing some text viewing widgetry.
 ** 1998-09-10 -	The creation of txi_open() above allowed this one to shrink from 50 to 5 lines, sort of.
 */
-static gint view_file_text(MainInfo *min, const gchar *full_name)
+static gint view_file_text(MainInfo *min, GFile *file, const gchar *encoding, GError **err)
 {
 	GtkWidget	*txv;
 
 	if((txv = txv_open(min, NULL)) != NULL)
 	{
 		txv_show(txv);
-		txv_text_load(txv, full_name, viewtext_options.use_mmap, viewtext_options.buf_size);
+		txv_text_load(txv, file, viewtext_options.buf_size, encoding, err);
 		txv_enable(txv);
 	}
 	return txv != NULL;
@@ -78,14 +57,14 @@ static gint view_file_text(MainInfo *min, const gchar *full_name)
 /* ----------------------------------------------------------------------------------------- */
 
 /* 1998-12-18 -	View file in hex mode. */
-static gint view_file_hex(MainInfo *min, const gchar *full_name)
+static gint view_file_hex(MainInfo *min, GFile *file, const gchar *encoding, GError **err)
 {
 	GtkWidget	*txv;
 
 	if((txv = txv_open(min, NULL)) != NULL)
 	{
 		txv_show(txv);
-		txv_text_load_hex(txv, full_name);
+		txv_text_load_hex(txv, file, err);
 		txv_enable(txv);
 	}
 	return txv != NULL;
@@ -95,95 +74,115 @@ static gint view_file_hex(MainInfo *min, const gchar *full_name)
 
 /* 1999-02-03 -	Inspect the first bytes (check_size) for printability. If all are OK, return 1.
 **		Otherwise return 0. If an error occurs, -1 is returned.
-** 1999-02-19 -	No longer uses mmap(), tragically. I didn't feel like adding the necessary
-**		fall-back code (in case mmap() isn't supported on the file system).
-** 1999-05-29 -	Changed order of open() and size-check; now produces errno code if file not found.
+** 2010-02-28 -	Ported to GIO, some comments that no longer apply were removed.
+** 2010-07-20 -	Now tries to charset-convert the input, if that fails it's not text. Nice.
 */
-static gint file_is_text(MainInfo *min, const gchar *full_name)
+static gint file_is_text(MainInfo *min, GFile *file, const gchar *encoding, GError **err)
 {
-	gchar	*buf;
-	gint	fd, txt = -1;
-	gsize	size, i;
+	GFileInputStream	*is;
+	gboolean		txt = FALSE;
 
-	if((fd = open(full_name, O_RDONLY)) >= 0)
+	if(encoding == NULL)
+		encoding = convstream_get_default_encoding();
+
+	if((is = g_file_read(file, NULL, err)) != NULL)
 	{
-		if(fut_fsize(fd, &size))
-		{
-			gsize	got;
+		gchar	*buf;
+		gssize	got;
 
-			if(size == 0 || size > viewtext_options.check_size)
-				size = viewtext_options.check_size;
-			buf = g_malloc(size);
-			if((got = read(fd, buf, size)) > 0)
-			{
-				txt = 1;
-				for(i = 0; txt && (i < got); i++)
-				{
-					if((guchar) buf[i] < 7)
-						txt = 0;
-				}
-			}
-			g_free(buf);
+		buf = g_malloc(viewtext_options.check_size);
+		if((got = g_input_stream_read(G_INPUT_STREAM(is), buf, viewtext_options.check_size, NULL, err)) > 0)
+		{
+			gchar	*out = g_convert(buf, got, convstream_get_target_encoding(), encoding, NULL, NULL, NULL);
+
+			txt = (out != NULL);
+			g_free(out);
 		}
-		close(fd);
+		g_free(buf);
+		g_object_unref(is);
 	}
-	return txt;
+	return *err != NULL ? -1 : txt;
 }
 
 /* 1999-02-03 -	This function views a file named <full_name> as either ASCII text or hex data.
 **		Note that it does this by actually inspecting the first few bytes of the file,
 **		rather than using gentoo's type system. This makes it work for all files, even
 **		those of type "Unknown".
+** 2010-02-28 -	Ported to GIO.
 */
-static gint view_file_text_or_hex(MainInfo *min, const gchar *full_name)
+static gint view_file_text_or_hex(MainInfo *min, GFile *file, const gchar *encoding, GError **err)
 {
 	gint	res;
 
-	if((res = file_is_text(min, full_name)) >= 0)
+	if((res = file_is_text(min, file, encoding, err)) >= 0)
 	{
 		if(res == 0)
-			view_file_hex(min, full_name);
+			view_file_hex(min, file, NULL, err);
 		else
-			view_file_text(min, full_name);
-	}
-	else if(errno)
-	{
-		err_set(min, errno, "ViewText", full_name);
-		err_show(min);
+			view_file_text(min, file, encoding, err);
 	}
 	return res >= 0;
 }
 
 /* ----------------------------------------------------------------------------------------- */
 
+/* 1999-03-06 -	A general viewer-trampoline. Will view all currently selected files using
+**		the given viewing function.
+** 2010-02-28 -	Ported to GIO and GTK+ 2.0.
+*/
+static gint view_selection(MainInfo *min, DirPane *dp, gint (*viewer)(MainInfo *min, GFile *file, const gchar *encoding, GError **err), const gchar *encoding)
+{
+	gboolean	ok = TRUE;
+	GSList		*slist, *iter;
+	GError		*err = NULL;
+
+	if((slist = dp_get_selection(dp)) == NULL)
+		return 1;
+	for(iter = slist; ok && iter != NULL; iter = g_slist_next(iter))
+	{
+		GFile	*file;
+
+		if((file = dp_get_file_from_row(dp, iter->data)) != NULL)
+		{
+			ok = viewer(min, file, encoding, &err);
+			if(ok)
+				dp_unselect(dp, iter->data);
+			else
+				err_set_gerror(min, &err, "ViewText", file);
+			g_object_unref(file);
+		}
+	}
+	dp_free_selection(slist);
+
+	return ok;
+}
+
 /* 1999-03-06 -	View selected files in plain text mode.
 ** 1999-05-29 -	Now uses the command argument system to control plain/hex/auto modes, thus
 **		replacing the old ViewTextHex and ViewTextOrHex commands. Neater.
 */
 gint cmd_viewtext(MainInfo *min, DirPane *src, DirPane *dst, const CmdArg *ca)
 {
-	const gchar	*sk;
-	guint		mode, skip = 0;
+	const gchar	*encoding;
+	gint		mode;
 
-	if((sk = car_keyword_get_value(ca, "skip", NULL)) != NULL)
-	{
-		gchar	*endptr;
-
-		skip = strtol(sk, &endptr, 0);
-		if(!*sk || *endptr)
-			skip = 0;
-	}
-	mode = car_keyword_get_enum(ca, "mode", 1, "auto", "text", "hex");
+	encoding = car_keyword_get_value(ca, "encoding", "");
+	/* Map an empty encoding to NULL, which gives us the system's default. */
+	if(encoding != NULL && *encoding == '\0')
+		encoding = NULL;
+	mode = car_keyword_get_enum(ca, "mode", 1, "auto", "text", "hex", NULL);
 	switch(mode)
 	{
 		case 0:
-			return view_selection(min, src, view_file_text_or_hex);
+			return view_selection(min, src, view_file_text_or_hex, encoding);
 		case 1:
-			return view_selection(min, src, view_file_text);
+			return view_selection(min, src, view_file_text, encoding);
 		case 2:
-			return view_selection(min, src, view_file_hex);
+			return view_selection(min, src, view_file_hex, NULL);
+		default:
+			;
 	}
-	fprintf(stderr, "ViewText: Unknown viewing mode '%s'\n", car_keyword_get_value(ca, "mode", NULL));
+
 	return 0;
 }
 
@@ -196,14 +195,12 @@ void cfg_viewtext(MainInfo *min)
 	{
 		/* Initialize options to legacy default values. */
 		viewtext_options.modified    = FALSE;
-		viewtext_options.use_mmap    = TRUE;
 		viewtext_options.buf_size    = (1 << 18);
 		viewtext_options.check_size  = (1 << 9);
 		viewtext_options.exit_on_left = TRUE;
 
 		viewtext_cmc = cmc_config_new("ViewText", &viewtext_options);
 		cmc_field_add_boolean(viewtext_cmc, "modified", NULL, offsetof(OptViewText, modified));
-		cmc_field_add_boolean(viewtext_cmc, "use_mmap", _("Use mmap() to Speed Loading?"), offsetof(OptViewText, use_mmap));
 		cmc_field_add_size(viewtext_cmc, "buf_size", _("Buffer Size"), offsetof(OptViewText, buf_size), 1024, (1 << 20), 1024);
 		cmc_field_add_size(viewtext_cmc, "check_size", _("Hex-Check First"), offsetof(OptViewText, check_size), 32, (1 << 16), 32);
 		cmc_field_add_boolean(viewtext_cmc, "exit_left", _("Exit on Left Arrow Key?"), offsetof(OptViewText, exit_on_left));
