@@ -70,14 +70,14 @@ typedef struct {
 	Dialog		*dlg;			/* The dialog window. */
 	CmdInp		input[DLG_MAX_INPUT];
 	GtkWidget	*body;			/* This holds the entire GUI. */
-	GtkWidget	*table;			/* Input widgets and labels are added here. */
+	GtkWidget	*grid;			/* Input widgets and labels are added here. */
 	gint		row;			/* Current row in table. */
 	gchar		*wintitle;		/* Title to set on window (example: {Iw:"gentoo"}). */
 	gint		str_count;		/* Counts number of string fields (for focus). */
 	gboolean	str_focused;		/* Gets set as the string is first focused. */
 } CmdDlg;
 
-static const gchar	the_true[] = "TRUE", the_false[] = "FALSE";
+static const gchar	the_true[] = "true", the_false[] = "false";
 
 /* ----------------------------------------------------------------------------------------- */
 
@@ -96,7 +96,8 @@ static void parse_code_file(MainInfo *min, GString *cstr, const gchar *code)
 	const gboolean	unsel = (strchr(code + 1, 'u') != NULL),
 			path  = (strchr(code + 1, 'p') != NULL),
 			dest  = (strchr(code + 1, 'd') != NULL),
-			quote = (strchr(code + 1, 'Q') == NULL);	/* Upper case for default-on. */
+			quote = (strchr(code + 1, 'Q') == NULL),	/* Upper case for default-on. */
+			ext   = (strchr(code + 1, 'E') == NULL);
 	GSList		*slist, *iter;
 
 	if(dest)		/* User requested the destination pane? */
@@ -110,7 +111,19 @@ static void parse_code_file(MainInfo *min, GString *cstr, const gchar *code)
 
 		if(iter != slist)
 			g_string_append_c(cstr, ' ');
-		g_string_append(cstr, name);
+		if(ext)
+			g_string_append(cstr, name);
+		else
+		{
+			const gchar	*dot = strchr(name, '.');
+			if(dot != NULL && dot > name)
+			{
+				/* Only append up to, but not including, the dot. */
+				g_string_append_len(cstr, name, dot - name);
+			}
+			else
+				g_string_append(cstr, name);
+		}
 		if(unsel)
 			dp_unselect(dp, iter->data);
 		if(code[0] == 'f')		/* Only first? */
@@ -237,30 +250,34 @@ static void parse_code_home(MainInfo *min, GString *cstr, const gchar *code)
 */
 static void init_input_dialog(CmdDlg *dlg)
 {
-	dlg->body  = gtk_vbox_new(FALSE, 0);
-	dlg->table = gtk_table_new(DLG_MAX_INPUT, 2, FALSE);
-	gtk_table_set_col_spacing(GTK_TABLE(dlg->table), 0, 5);
-	gtk_box_pack_start(GTK_BOX(dlg->body), dlg->table, TRUE, TRUE, 0);
+	dlg->body = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
+	dlg->grid = gtk_grid_new();
+	gtk_grid_set_column_spacing(GTK_GRID(dlg->grid), 5);
+	gtk_box_pack_start(GTK_BOX(dlg->body), dlg->grid, TRUE, TRUE, 0);
 }
 
 /* 1999-02-27 -	A little helper routine to pack widgets into the table forming the command input
 **		dialog. <pos> can be either 0 (left column), 1 (right column) or 2 (both columns).
-**		The <padding> is vertical, and best left at 0 for non-separator widgets. Also shows
-**		the widget.
+**		The <padding> is vertical, and best left at 0 for non-separator widgets.
 */
 static void pack_input_widget(CmdDlg *dlg, GtkWidget *wid, gint pos, gint padding)
 {
 	if(dlg != NULL && wid != NULL)
 	{
-		gint	left = 0, right = 2;		/* Assume <pos> will be 2. */
+		gint	x = 0, width = 2;	/* Assume <pos> will be 2. */
 
 		if(pos == 0)
-			right--;
+			--width;
 		else if(pos == 1)
-			left++;
-
-		gtk_table_attach(GTK_TABLE(dlg->table), wid, left, right, dlg->row, dlg->row + 1,
-				 GTK_FILL|GTK_EXPAND, 0, 0, padding);
+			++x;
+		gtk_widget_set_hexpand(wid, TRUE);
+		gtk_widget_set_halign(wid, GTK_ALIGN_FILL);
+		if(padding != 0)
+		{
+			gtk_widget_set_margin_top(wid, padding);
+			gtk_widget_set_margin_bottom(wid, padding);
+		}
+		gtk_grid_attach(GTK_GRID(dlg->grid), wid, x, dlg->row, width, 1);
 	}
 }
 
@@ -397,7 +414,7 @@ static gint build_input_label(CmdDlg *dlg, const gchar *def, CmdInp *inp)
 		if(strcmp(def, "-") == 0)	/* Is the label a single hyphen? Then generate separator bar. */
 		{
 			inp->type = CITP_LABEL;
-			pack_input_widget(dlg, inp->inp.label.label = gtk_hseparator_new(), 2, 5);
+			pack_input_widget(dlg, inp->inp.label.label = gtk_separator_new(GTK_ORIENTATION_HORIZONTAL), 2, 5);
 		}
 		else
 		{
@@ -505,7 +522,7 @@ static void parse_code_input(MainInfo *min, GString *cstr, gchar *code, CmdDlg *
 			if(ltxt)
 			{
 				label = gtk_label_new(ltxt);
-				gtk_table_attach(GTK_TABLE(dlg->table), label, 0, 1, dlg->row, dlg->row + 1, GTK_FILL,0,0,0);
+				gtk_grid_attach(GTK_GRID(dlg->grid), label, 0, dlg->row, 1, 1);
 			}
 			g_string_sprintfa(cstr, INP_PATTERN, dlg->row++);
 		}
