@@ -32,7 +32,6 @@
 /* ----------------------------------------------------------------------------------------- */
 
 static const gchar	*row_type_name[] = { "Built-In", "External" };
-static GMemChunk	*builtin_chunk = NULL;
 
 /* This describes a single built-in command. The 'cmd' field is the actual command execution
 ** function, like cmd_mkdir or cmd_activateother. The 'cfg' field is the corresponding commands's
@@ -53,13 +52,11 @@ static void csq_add_builtin(MainInfo *min, const gchar *name, Command cmd, Comma
 
 	if(cfg->commands.builtin == NULL)
 		cfg->commands.builtin = g_hash_table_new(g_str_hash, g_str_equal);
-	if(builtin_chunk == NULL)
-		builtin_chunk = g_mem_chunk_new("CmdDesc", sizeof (CmdDesc), 32, G_ALLOC_ONLY);
-	if((cfg->commands.builtin != NULL) && (builtin_chunk != NULL))
+	if(cfg->commands.builtin != NULL)
 	{
 		CmdDesc	*cdesc;
 
-		if((cdesc = g_mem_chunk_alloc(builtin_chunk)) != NULL)
+		if((cdesc = g_slice_alloc(sizeof *cdesc)) != NULL)
 		{
 			cdesc->cmd = cmd;
 			cdesc->cfg = cmd_cfg;
@@ -332,10 +329,10 @@ void csq_cmdseq_row_delete_all(CmdSeq *cs)
 static gint execute_builtin(MainInfo *min, const gchar *cmd)
 {
 	gchar	msg[CSQ_NAME_SIZE + 64];
-	gchar	*spc, **argv;
+	gchar	**argv;
 	CmdDesc	*cdesc;
 
-	if((spc = strchr(cmd, ' ')) != NULL)		/* Does command string contain space, and thus arguments? */
+	if(strchr(cmd, ' ') != NULL)		/* Does command string contain space, and thus arguments? */
 	{
 		if((argv = cpr_parse(min, cmd)) != NULL)
 		{
@@ -562,7 +559,7 @@ gint csq_execute(MainInfo *min, const gchar *name)
 	if((name == NULL) || (*name == '\0'))
 		return 0;
 
-	if((min->cfg.commands.cmdseq != NULL) && ((cs = (CmdSeq *) g_hash_table_lookup(min->cfg.commands.cmdseq, name)) != NULL))
+	if((min->cfg.commands.cmdseq != NULL) && ((cs = g_hash_table_lookup(min->cfg.commands.cmdseq, name)) != NULL))
 		ok = cmdseq_execute(min, cs);
 	else
 		ok = execute_builtin(min, name);
@@ -617,9 +614,7 @@ void csq_continue(MainInfo *min)
 	chd_clear_running();
 
 	/* Determine if the sequence should repeat. */
-	if((cs != NULL) && (iter == NULL) && (cs->flags & CSFLG_REPEAT) &&
-	   (min->gui->cur_pane->focus_row == -1) && !dpf_get_fake_select() &&
-           (dp_has_selection(min->gui->cur_pane)))
+	if((cs != NULL) && (iter == NULL) && (cs->flags & CSFLG_REPEAT) && dp_has_selection(min->gui->cur_pane))
 		csq_execute(min, cs->name);
 
 	min->gui->pane[0].dbclk_row = min->gui->pane[1].dbclk_row = -1;
@@ -737,7 +732,7 @@ void csq_init_commands(MainInfo *min)
 	csq_add_builtin(min, "DpHide",		cmd_dphide,		NULL);
 	csq_add_builtin(min, "DpRecenter",	cmd_dprecenter,		NULL);
 	csq_add_builtin(min, "DpReorient",	cmd_dpreorient,		NULL);
-	csq_add_builtin(min, "DpFocus",		cmd_dpfocus,		cfg_dpfocus);
+	csq_add_builtin(min, "DpFocus",		cmd_dpfocus,		NULL);
 	csq_add_builtin(min, "DpFocusPath",	cmd_dpfocuspath,	NULL);
 	csq_add_builtin(min, "DpFocusISrch",	cmd_dpfocusisrch,	cfg_dpfocusisrch);
 	csq_add_builtin(min, "DpGotoRow",	cmd_dpgotorow,		NULL);
