@@ -23,8 +23,9 @@ extern void	csq_cmdseq_hash(GHashTable **hash, CmdSeq *cs);
 
 extern void	csq_cmdseq_destroy(CmdSeq *cs);
 extern gint	csq_cmdseq_row_append(CmdSeq *cs, CmdRow *row);
-extern gint	csq_cmdseq_row_move(CmdSeq *cs, CmdRow *row, gint delta);
+extern void	csq_cmdseq_rows_relink(CmdSeq *cs, GList *rows);
 extern gint	csq_cmdseq_row_delete(CmdSeq *cs, CmdRow *row);
+extern void	csq_cmdseq_row_delete_all(CmdSeq *cs);
 
 extern gint	csq_execute(MainInfo *min, const gchar *name);
 extern gint	csq_execute_format(MainInfo *min, const gchar *fmt, ...);
