@@ -31,7 +31,7 @@ struct CmdCfg {
 	GList		*fields;
 };
 
-typedef enum { CFT_INTEGER = 0, CFT_BOOLEAN, CFT_ENUM, CFT_SIZE, CFT_STRING, CFT_COLOR, CFT_PATH } CFType;
+typedef enum { CFT_INTEGER = 0, CFT_BOOLEAN, CFT_ENUM, CFT_SIZE, CFT_STRING, CFT_PATH } CFType;
 
 typedef struct {
 	gint	min, max;
@@ -45,7 +45,7 @@ typedef struct {
 
 typedef struct {
 	gsize	size;
-	gunichar	separator;		/* Used for CFT_PATH. */
+	gunichar	separator;	/* Used for CFT_PATH. */
 } CFStr;
 
 typedef struct {
@@ -140,9 +140,6 @@ void cmc_config_base_save(CmdCfg *cmc, FILE *out)
 				case CFT_PATH:
 					xml_put_text(out, fld->name, (gchar *) cmc->base + fld->offset);
 					break;
-				case CFT_COLOR:
-					xml_put_color(out, fld->name, (GdkColor *) ((gchar *) cmc->base + fld->offset));
-					break;
 			}
 		}
 		xml_put_node_close(out, cmc->name);
@@ -186,9 +183,6 @@ void cmc_config_base_load(CmdCfg *cmc, const XmlNode *node)
 				case CFT_PATH:
 					xml_get_text_copy(node, fld->name, (gchar *) cmc->base + fld->offset, fld->field.string.size);
 					break;
-				case CFT_COLOR:
-					xml_get_color(node, fld->name, (GdkColor *) ((gchar *) cmc->base + fld->offset));
-					break;
 			}
 		}
 	}
@@ -389,12 +383,6 @@ void cmc_field_add_path(CmdCfg *cmc, const gchar *name, const gchar *desc, gsize
 	}
 }
 
-void cmc_field_add_color(CmdCfg *cmc, const gchar *name, const gchar *desc, gsize offset)
-{
-	if(cmc != NULL)
-		field_add(cmc, field_new(CFT_COLOR, name, desc, offset));
-}
-
 /* 1999-04-05 -	Find a named field. */
 static CField * field_find(CmdCfg *cmc, const gchar *name)
 {
@@ -526,7 +514,7 @@ static gint evt_size_changed(GObject *obj, gpointer user)
 	if((cmc != NULL) && (fld != NULL) && (instance != NULL))
 	{
 		gchar		buf[32];
-		gsize		value = GTK_ADJUSTMENT(obj)->value;
+		gsize		value = gtk_adjustment_get_value(GTK_ADJUSTMENT(obj));
 		GtkWidget	*label = g_object_get_data(obj, "label");
 
 		value *= fld->field.size.step;
@@ -543,18 +531,6 @@ static gint evt_size_changed(GObject *obj, gpointer user)
 	return TRUE;
 }
 
-/* 2008-11-28 -	User changed the color. Write new back from widget. */
-static void evt_color_set(GtkWidget *wid, gpointer wuser, guint index, gpointer suser)
-{
-	CmdCfg		*cmc = suser;
-	CField		*fld;
-	gpointer	instance;
-
-	get_object_data(G_OBJECT(wid), &fld, &instance);
-	gtk_color_button_get_color(GTK_COLOR_BUTTON(wid), (GdkColor *) ((gchar *) instance + fld->offset));
-	set_modified(cmc, instance, TRUE);
-}
-
 static void evt_path_pick_clicked(GtkWidget *wid, gpointer user)
 {
 	CField		*fld;
@@ -568,60 +544,62 @@ static void evt_path_pick_clicked(GtkWidget *wid, gpointer user)
 }
 
 /* 2002-08-01 -	Build a neatly right-aligned label, and stuff it into <table>. */
-static void label_build(const gchar *text, GtkWidget *table, gint x, gint y)
+static void label_build(const gchar *text, GtkWidget *grid, gint x, gint y)
 {
-	GtkWidget	*algn, *label;
+	GtkWidget	*lab;
 
-	algn  = gtk_alignment_new(0.9f, 0.0f, 0.0f, 0.0f);
-	label = gtk_label_new(text);
-	gtk_container_add(GTK_CONTAINER(algn), label);
-	gtk_table_attach(GTK_TABLE(table), algn, x, x + 1, y, y +1,  GTK_FILL,0,0,0);
+	lab = gtk_label_new(text);
+	gtk_widget_set_halign(lab, GTK_ALIGN_END);
+	gtk_widget_set_margin_end(lab, 5);
+	gtk_grid_attach(GTK_GRID(grid), lab, x, y, 1, 1);
 }
 
-/* 1999-04-05 -	Build config widgetry for given <fld>. This function is a bit schizofrenic, since it
-**		does one of two things: if <table> is non-NULL, the widgetry is packed into it, which
-**		looks neat. If it is NULL, a new table is created (height==1), packed, and returned.
+/* 1999-04-05 -	Build config widgetry for given <fld>. This function is a bit schizophrenic, since it
+**		does one of two things: if <grid> is non-NULL, the widgetry is packed into it, which
+**		looks neat. If it is NULL, a new grid is created, packed, and returned.
 */
-static GtkWidget * field_build(CmdCfg *cmc, CField *fld, gpointer instance, GtkWidget *table, gint row)
+static GtkWidget * field_build(CmdCfg *cmc, CField *fld, gpointer instance, GtkWidget *grid, gint row)
 {
 	if((cmc != NULL) && (fld != NULL) && (instance != NULL))
 	{
 		GtkWidget	*wid, *hbox, *label;
 		GtkAdjustment	*adj;
-		GtkObject	*obj;
 
-		if(table == NULL)
+		if(grid == NULL)
 		{
-			table = gtk_table_new(1, 2, FALSE);
+			grid = gtk_grid_new();
 			row = 0;
 		}
 		switch(fld->type)
 		{
 			case CFT_INTEGER:
-				label_build(fld->desc, table, 0, row);
+				label_build(fld->desc, grid, 0, row);
 				wid = gtk_spin_button_new(NULL, 1, 0);
 				set_object_data(G_OBJECT(wid), fld, instance);
 				adj = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(wid));
 				gtk_adjustment_clamp_page(adj, fld->field.integer.min, fld->field.integer.max);
 				gtk_adjustment_set_value(adj, *(gint *) ((gchar *) instance + fld->offset));
-				gtk_table_attach(GTK_TABLE(table), wid, 1, 2, row, row + 1,  GTK_EXPAND|GTK_FILL,0,5,0);
+				gtk_widget_set_hexpand(wid, TRUE);
+				gtk_widget_set_halign(wid, GTK_ALIGN_FILL);
+				gtk_grid_attach(GTK_GRID(grid), wid, 1, row, 1, 1);
 				break;
 			case CFT_BOOLEAN:
 				wid = gtk_check_button_new_with_label(fld->desc);
 				set_object_data(G_OBJECT(wid), fld, instance);
 				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wid), *(gboolean *) ((gchar *) instance + fld->offset));
 				g_signal_connect(G_OBJECT(wid), "clicked", G_CALLBACK(evt_boolean_clicked), cmc);
-				gtk_table_attach(GTK_TABLE(table), wid, 0, 2, row, row + 1,  GTK_EXPAND|GTK_FILL,0,5,0);
+				gtk_widget_set_hexpand(wid, TRUE);
+				gtk_widget_set_halign(wid, GTK_ALIGN_FILL);
+				gtk_grid_attach(GTK_GRID(grid), wid, 0, row, 2, 1);
 				break;
 			case CFT_ENUM:
 				wid = gtk_frame_new(fld->desc);
 				{
 					GtkWidget	*vbox, *radio[sizeof fld->field.fenum.label / sizeof *fld->field.fenum.label];
-					GSList		*group;
 					gint		i, current = *(gint *) ((gchar *) instance + fld->offset);
 
-					vbox = gtk_vbox_new(FALSE, 0);
-					group = gui_radio_group_new(fld->field.fenum.num, (const gchar **) fld->field.fenum.label, radio);
+					vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
+					gui_radio_group_new(fld->field.fenum.num, (const gchar **) fld->field.fenum.label, radio);
 					for(i = 0; i < fld->field.fenum.num; i++)
 					{
 						set_object_data(G_OBJECT(radio[i]), fld, instance);
@@ -633,49 +611,48 @@ static GtkWidget * field_build(CmdCfg *cmc, CField *fld, gpointer instance, GtkW
 					}
 					gtk_container_add(GTK_CONTAINER(wid), vbox);
 				}
-				gtk_table_attach(GTK_TABLE(table), wid, 0, 2, row, row + 1,  GTK_EXPAND|GTK_FILL,0,5,0);
+				gtk_widget_set_hexpand(wid, TRUE);
+				gtk_widget_set_halign(wid, GTK_ALIGN_FILL);
+				gtk_grid_attach(GTK_GRID(grid), wid, 0, row, 2, 1);
 				break;
 			case CFT_SIZE:
-				label_build(fld->desc, table, 0, row);
-				hbox = gtk_hbox_new(FALSE, 0);
-				obj = gtk_adjustment_new((gfloat) *(gint *) ((gchar *) instance + fld->offset) / fld->field.size.step,
+				label_build(fld->desc, grid, 0, row);
+				hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
+				adj = gtk_adjustment_new((gfloat) *(gint *) ((gchar *) instance + fld->offset) / fld->field.size.step,
 							 fld->field.size.min / fld->field.size.step,
 							 fld->field.size.max / fld->field.size.step,
 							 1, 1, 0);
 
-				set_object_data(G_OBJECT(obj), fld, instance);
-				wid = gtk_hscale_new(GTK_ADJUSTMENT(obj));
+				set_object_data(G_OBJECT(adj), fld, instance);
+				wid = gtk_scale_new(GTK_ORIENTATION_HORIZONTAL, adj);
 				gtk_scale_set_draw_value(GTK_SCALE(wid), FALSE);
 				gtk_box_pack_start(GTK_BOX(hbox), wid, TRUE, TRUE, 0);
 				label = gtk_label_new("");
 				gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
-				g_object_set_data(G_OBJECT(obj), "label", label);
-				g_signal_connect(G_OBJECT(obj), "value_changed", G_CALLBACK(evt_size_changed), cmc);
-				gtk_adjustment_value_changed(GTK_ADJUSTMENT(obj));
-				gtk_table_attach(GTK_TABLE(table), hbox, 1, 2, row, row + 1,  GTK_EXPAND|GTK_FILL,0,5,0);
+				g_object_set_data(G_OBJECT(adj), "label", label);
+				g_signal_connect(G_OBJECT(adj), "value_changed", G_CALLBACK(evt_size_changed), cmc);
+				gtk_adjustment_value_changed(adj);
+				gtk_widget_set_hexpand(hbox, TRUE);
+				gtk_widget_set_halign(hbox, GTK_ALIGN_FILL);
+				gtk_grid_attach(GTK_GRID(grid), hbox, 1, row, 1, 1);
 				break;
 			case CFT_STRING:
-				label_build(fld->desc, table, 0, row);
+				label_build(fld->desc, grid, 0, row);
 				wid = gtk_entry_new();
 				gtk_entry_set_max_length(GTK_ENTRY(wid), fld->field.string.size - 1);
 				set_object_data(G_OBJECT(wid), fld, instance);
 				gtk_entry_set_text(GTK_ENTRY(wid), (gchar *) instance + fld->offset);
 				g_signal_connect(G_OBJECT(wid), "changed", G_CALLBACK(evt_string_changed), cmc);
-				gtk_table_attach(GTK_TABLE(table), wid, 1, 2, row, row + 1,  GTK_EXPAND|GTK_FILL,0,5,0);
-				break;
-			case CFT_COLOR:
-				label_build(fld->desc, table, 0, row);
-				wid = gtk_color_button_new_with_color((const GdkColor *) ((gchar *) instance + fld->offset));
-				set_object_data(G_OBJECT(wid), fld, instance);
-				g_signal_connect(G_OBJECT(wid), "color_set", G_CALLBACK(evt_color_set), cmc);
-				gtk_table_attach(GTK_TABLE(table), wid, 1, 2, row, row + 1, GTK_EXPAND|GTK_FILL,0,5,0);
+				gtk_widget_set_hexpand(wid, TRUE);
+				gtk_widget_set_halign(wid, GTK_ALIGN_FILL);
+				gtk_grid_attach(GTK_GRID(grid), wid, 1, row, 1, 1);
 				break;
 			case CFT_PATH:
 				{
 					GtkWidget	*pick;
 
-					label_build(fld->desc, table, 0, row);
-					hbox = gtk_hbox_new(FALSE, 0);
+					label_build(fld->desc, grid, 0, row);
+					hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 					wid = gtk_entry_new();
 					gtk_entry_set_max_length(GTK_ENTRY(wid), fld->field.string.size - 1);
 					set_object_data(G_OBJECT(wid), fld, instance);
@@ -687,11 +664,13 @@ static GtkWidget * field_build(CmdCfg *cmc, CField *fld, gpointer instance, GtkW
 					g_object_set_data(G_OBJECT(pick), "entry", wid);
 					g_signal_connect(G_OBJECT(pick), "clicked", G_CALLBACK(evt_path_pick_clicked), cmc);
 					gtk_box_pack_start(GTK_BOX(hbox), pick, FALSE, FALSE, 0);
-					gtk_table_attach(GTK_TABLE(table), hbox, 1, 2, row, row + 1, GTK_EXPAND|GTK_FILL,0,5,0);
+					gtk_widget_set_hexpand(hbox, TRUE);
+					gtk_widget_set_halign(hbox, GTK_ALIGN_FILL);
+					gtk_grid_attach(GTK_GRID(grid), hbox, 1, row, 1, 1);
 				}
 				break;
 		}
-		return table;
+		return grid;
 	}
 	return NULL;
 }
@@ -762,9 +741,6 @@ gpointer cmc_instance_new(CmdCfg *cmc)
 				case CFT_PATH:
 					fsize += fld->field.string.size;
 					break;
-				case CFT_COLOR:
-					fsize += sizeof (GdkColor);
-					break;
 			}
 			if(fsize > size)
 				size = fsize;
@@ -826,9 +802,6 @@ void cmc_instance_copy(CmdCfg *cmc, gpointer dst, gpointer src)
 				case CFT_PATH:
 					g_strlcpy((gchar *) dst + fld->offset, (gchar *) src + fld->offset, fld->field.string.size);
 					break;
-				case CFT_COLOR:
-					*(GdkColor *) ((gchar *) dst + fld->offset) = *(GdkColor *) ((gchar *) src + fld->offset);
-					break;
 			}
 		}
 	}
@@ -853,22 +826,22 @@ void cmc_instance_copy_from_base(CmdCfg *cmc, gpointer dst)
 /* 1999-04-05 -	Build a container widget containing the widgetry for configuring each field. */
 GtkWidget * cmc_instance_build(CmdCfg *cmc, gpointer instance)
 {
-	GtkWidget	*table = NULL;
+	GtkWidget	*grid = NULL;
 
 	if((cmc != NULL) && (instance != NULL))
 	{
 		const GList	*iter;
 		gint		row = 0;
 
-		table = gtk_table_new(g_list_length(cmc->fields) - 1, 2, FALSE);
+		grid = gtk_grid_new();
 		for(iter = cmc->fields; iter != NULL; iter = g_list_next(iter))
 		{
 			if(strcmp(((CField *) iter->data)->name, "modified") == 0)
 				continue;
-			field_build(cmc, iter->data, instance, table, row++);
+			field_build(cmc, iter->data, instance, grid, row++);
 		}
 	}
-	return table;
+	return grid;
 }
 
 /* 1999-04-05 -	Get the 'modified' state of given instance. */
