@@ -28,11 +28,10 @@ typedef struct {
 	Dialog		*dlg;
 	GtkWidget	*vbox;
 	GtkWidget	*entry;
-	guint		evt_changed;
 	GList		*builtin, *cmdseq;
-	GCompletion	*completion;
 	GList		*clist, *citer;
 	GtkTreeStore	*store;
+	GtkListStore	*lstore;	/* Only used for completion, to work around sub-tree issues. */
 	gboolean	expanded[2];
 } CDlg;
 
@@ -80,10 +79,8 @@ static void evt_cursor_changed(GtkWidget *wid, gpointer user)
 	gtk_tree_model_get(GTK_TREE_MODEL(cdlg->store), &iter, 0, &text, -1);
 	if(text != NULL)
 	{
-		g_signal_handler_block(G_OBJECT(cdlg->entry), cdlg->evt_changed);
 		gtk_entry_set_text(GTK_ENTRY(cdlg->entry), text);
 		gtk_widget_grab_focus(cdlg->entry);
-		g_signal_handler_unblock(G_OBJECT(cdlg->entry), cdlg->evt_changed);
 		g_string_assign(cdlg->cmd, text);
 		g_free(text);
 	}
@@ -107,7 +104,7 @@ static void evt_row_activated(GtkWidget *wid, GtkTreePath *path, GtkTreeViewColu
 }
 
 /* 2004-11-21 -	Append a subtree from a list of strings. */
-static void append_subtree_list(const GList *list, GtkTreeStore *store, GtkTreeIter *parent, gpointer user)
+static void append_subtree_list(const GList *list, GtkTreeStore *store, GtkTreeIter *parent)
 {
 	for(; list != NULL; list = g_list_next(list))
 	{
@@ -118,28 +115,61 @@ static void append_subtree_list(const GList *list, GtkTreeStore *store, GtkTreeI
 	}
 }
 
+static void append_liststore_list(const GList *list, GtkListStore *store)
+{
+	for(; list != NULL; list = g_list_next(list))
+	{
+		GtkTreeIter	iter;
+
+		gtk_list_store_append(store, &iter);
+		gtk_list_store_set(store, &iter, 0, list->data, -1);
+	}
+}
+
+/* 2012-05-23 -	Callback for GTK+'s built-in completion support. This replaces the old
+**		TAB-triggered completion, since that in turn was based on GCompletion
+**		(in glib), which has been deprecated.
+*/
+static gboolean cb_match_func(GtkEntryCompletion *ec, const gchar *key, GtkTreeIter *iter, gpointer user)
+{
+	CDlg		*cdlg = user;
+	const gchar	*text = gtk_entry_get_text(GTK_ENTRY(cdlg->entry));
+	gchar		*cname;
+	gboolean	ret;
+
+	gtk_tree_model_get(GTK_TREE_MODEL(gtk_entry_completion_get_model(ec)), iter, 0, &cname, -1);
+	ret = strstr(cname, text) != NULL;
+	g_free(cname);
+
+	return ret;
+}
+
 /* 2004-11-21 -	Build tree holding built-in and user commands. New GTK+ 2.0 version. */
 static GtkWidget * build_tree(MainInfo *min, CDlg *cdlg)
 {
-	GtkTreeIter	iter;
-	GtkWidget	*view;
-	GtkCellRenderer	*cr;
-	GtkTreeViewColumn *vc;
-	GtkTreePath	*path;
-/*	GtkEntryCompletion	*compl;*/
-	gchar		label[32];
+	GtkTreeIter		iter;
+	GtkWidget		*view;
+	GtkCellRenderer		*cr;
+	GtkTreeViewColumn	 *vc;
+	GtkTreePath		*path;
+	GtkEntryCompletion	*compl;
+	gchar			label[32];
 
 	/* Create tree store, populate with contents of built-in and user-defined lists. */
 	cdlg->store = gtk_tree_store_new(1, G_TYPE_STRING);
 	gtk_tree_store_append(cdlg->store, &iter, NULL);
 	g_snprintf(label, sizeof label, _("Built-Ins (%u)"), g_list_length(cdlg->builtin));
 	gtk_tree_store_set(cdlg->store, &iter, 0, label, -1);
-	append_subtree_list(cdlg->builtin, cdlg->store, &iter, cdlg);
+	append_subtree_list(cdlg->builtin, cdlg->store, &iter);
 
 	gtk_tree_store_append(cdlg->store, &iter, NULL);
 	g_snprintf(label, sizeof label, _("User Defined (%u)"), g_list_length(cdlg->cmdseq));
 	gtk_tree_store_set(cdlg->store, &iter, 0, label, -1);
-	append_subtree_list(cdlg->cmdseq, cdlg->store, &iter, cdlg);
+	append_subtree_list(cdlg->cmdseq, cdlg->store, &iter);
+
+	cdlg->lstore = gtk_list_store_new(1, G_TYPE_STRING);
+	append_liststore_list(cdlg->builtin, cdlg->lstore);
+	append_liststore_list(cdlg->cmdseq,  cdlg->lstore);
 
 	/* Create view, renderer, and column, and stuff it all together. */
 	view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(cdlg->store));
@@ -173,121 +203,18 @@ static GtkWidget * build_tree(MainInfo *min, CDlg *cdlg)
 	g_signal_connect(G_OBJECT(view), "row_activated", G_CALLBACK(evt_row_activated), cdlg);
 
 	/* Create GtkEntryCompletion and link to tree. */
-/*	compl = gtk_entry_completion_new();
+	compl = gtk_entry_completion_new();
 	gtk_entry_completion_set_match_func(compl, cb_match_func, cdlg, NULL);
 	gtk_entry_completion_set_text_column(compl, 0);
+	gtk_entry_completion_set_model(compl, GTK_TREE_MODEL(cdlg->lstore));
+	gtk_entry_completion_set_inline_selection(compl, TRUE);
 	gtk_entry_set_completion(GTK_ENTRY(cdlg->entry), compl);
-	g_object_unref(compl);
-	gtk_entry_completion_set_model(compl, GTK_TREE_MODEL(store));
-*/
+
 	return view;
 }
 
 /* ----------------------------------------------------------------------------------------- */
 
-/* 1999-05-09 -	Construct a GCompletion capable of doing completion on command name
-**		strings.
-*/
-static void init_completion(CDlg *cdlg)
-{
-	if((cdlg->completion = g_completion_new(NULL)) != NULL)
-	{
-		g_completion_add_items(cdlg->completion, cdlg->builtin);
-		g_completion_add_items(cdlg->completion, cdlg->cmdseq);
-		cdlg->clist = cdlg->citer = NULL;
-	}
-}
-
-/* 1999-05-09 -	Free the completion as the dialog closes. */
-static void free_completion(CDlg *cdlg)
-{
-	if(cdlg->completion != NULL)
-	{
-		g_completion_free(cdlg->completion);
-		cdlg->completion = NULL;
-		cdlg->clist = cdlg->citer = NULL;
-	}
-}
-
-/* 1999-05-09 -	This gets called as the user edits the entry. Zzap any buffered completion. */
-static void evt_entry_changed(GtkWidget *wid, gpointer user)
-{
-	CDlg	*cdlg = user;
-
-	if(cdlg->clist != NULL)
-		free_completion(cdlg);
-	g_string_assign(cdlg->cmd, gtk_entry_get_text(GTK_ENTRY(wid)));
-}
-
-/* 1999-09-05 -	Set the command text, without losing the completion state. */
-static void set_command(CDlg *cdlg, const gchar *text)
-{
-	if((cdlg != NULL) && (text != NULL))
-	{
-		g_signal_handler_block(G_OBJECT(cdlg->entry), cdlg->evt_changed);
-		gtk_entry_set_text(GTK_ENTRY(cdlg->entry), text);
-		gtk_editable_set_position(GTK_EDITABLE(cdlg->entry), strlen(text));
-		g_signal_handler_unblock(G_OBJECT(cdlg->entry), cdlg->evt_changed);
-		g_string_assign(cdlg->cmd, text);
-	}
-}
-
-/* 1999-05-09 -	Happy birthday to me! :) Um, this is an attempt to move to TAB-triggered
-**		command name completion, rather than automatic as-you-type one. Using glib.
-*/
-static gint evt_entry_key_press(GtkWidget *wid, GdkEventKey *evt, gpointer user)
-{
-	gboolean	back;
-	CDlg		*cdlg = user;
-
-	if((evt->keyval == GDK_Tab) || (evt->keyval == GDK_ISO_Left_Tab))
-	{
-		back = evt->keyval == GDK_ISO_Left_Tab;
-		if(cdlg->completion == NULL)
-			init_completion(cdlg);
-		if(cdlg->completion != NULL)
-		{
-			if(cdlg->citer == NULL)
-			{
-				const gchar	*text;
-				gchar		*prefix = NULL;
-
-				text = gtk_entry_get_text(GTK_ENTRY(wid));
-				if((cdlg->clist = g_completion_complete(cdlg->completion, text, &prefix)) != NULL)
-				{
-					set_command(cdlg, prefix);
-					g_free(prefix);
-				}
-				if(back)
-					cdlg->citer = g_list_last(cdlg->clist);
-				else
-					cdlg->citer = cdlg->clist;
-			}
-			else
-			{
-				set_command(cdlg, cdlg->citer->data);
-				if(back)
-				{
-					cdlg->citer = g_list_previous(cdlg->citer);
-					if(cdlg->citer == NULL)
-						cdlg->citer = g_list_last(cdlg->clist);
-				}
-				else
-				{
-					cdlg->citer = g_list_next(cdlg->citer);
-					if(cdlg->citer == NULL)
-						cdlg->citer = cdlg->clist;
-				}
-			}
-			g_signal_stop_emission_by_name(G_OBJECT(wid), "key_press_event");
-			return TRUE;
-		}
-	}
-	return FALSE;
-}
-
-/* ----------------------------------------------------------------------------------------- */
-
 /* 1998-09-27 -	Open up a dialog where the user can select (or type) a command name.
 **		The <cmdseq> argument should contain the current user-defined command sequences.
 **		If it is NULL, the possibly-not-so-current ones in min->cfg.commands are used.
@@ -307,21 +234,18 @@ const gchar * csq_dialog_sync_new_wait(MainInfo *min, GHashTable *cmdseq)
 	if(cdlg->cmd == NULL)
 		cdlg->cmd = g_string_new(NULL);
 
-	cdlg->completion = NULL;
 	cdlg->clist = cdlg->citer = NULL;
 
-	cdlg->vbox = gtk_vbox_new(FALSE, 0);
-	label = gtk_label_new(_("Select a command, or type start\nof name and press TAB."));
+	cdlg->vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
+	label = gtk_label_new(_("Select a command, or type part of its name."));
 	gtk_box_pack_start(GTK_BOX(cdlg->vbox), label, FALSE, FALSE, 0);
 	scwin  = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	cdlg->entry = gui_dialog_entry_new();
 	if((tree = build_tree(min, cdlg)) != NULL)
 	{
-		gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scwin), tree);
+		gtk_container_add(GTK_CONTAINER(scwin), tree);
 		gtk_box_pack_start(GTK_BOX(cdlg->vbox), scwin, TRUE, TRUE, 0);
-		g_signal_connect(G_OBJECT(cdlg->entry), "key_press_event", G_CALLBACK(evt_entry_key_press), cdlg);
-		cdlg->evt_changed = g_signal_connect(G_OBJECT(cdlg->entry), "changed", G_CALLBACK(evt_entry_changed), cdlg);
 		gtk_box_pack_start(GTK_BOX(cdlg->vbox), cdlg->entry, FALSE, FALSE, 0);
 
 		gtk_entry_set_text(GTK_ENTRY(cdlg->entry), cdlg->cmd->str);
@@ -331,11 +255,14 @@ const gchar * csq_dialog_sync_new_wait(MainInfo *min, GHashTable *cmdseq)
 		gtk_window_set_default_size(GTK_WINDOW(dlg_dialog_get_dialog(cdlg->dlg)), 320, 544);
 		gtk_widget_grab_focus(cdlg->entry);
 		if(dlg_dialog_sync_wait(cdlg->dlg) == DLG_POSITIVE)
+		{
+			g_string_assign(cdlg->cmd, gtk_entry_get_text(GTK_ENTRY(cdlg->entry)));
 			ret = cdlg->cmd->str;
+		}
 		dlg_dialog_sync_destroy(cdlg->dlg);
 		g_list_free(cdlg->builtin);
 		g_list_free(cdlg->cmdseq);
-		free_completion(cdlg);
+		g_object_unref(cdlg->lstore);
 	}
 	return ret;
 }
