@@ -67,6 +67,7 @@
 typedef struct {
 	MainInfo	*min;
 	GtkWidget	*dlg;
+	GtkWidget	*view;
 	GtkTreeStore	*store;
 	GtkTreeIter	treeiter[4];
 	guint		level;		/* Current depth in tree, used when building. */
@@ -163,6 +164,17 @@ static void hide_config(CfgGui *cgu)
 		rebuild_top(cgu->min);
 	if(the_cfggui.flags & CFLG_REBUILD_MIDDLE)
 		rebuild_middle(cgu->min);
+
+	/* The rescanning needs to happen early, since we now have potentially stale Type pointers in the panes. */
+	if(the_cfggui.flags & CFLG_RESCAN_LEFT)
+		dp_rescan(&cgu->min->gui->pane[0]);
+	else if(the_cfggui.flags & CFLG_REDISP_LEFT)
+		dp_redisplay_preserve(&cgu->min->gui->pane[0]);
+	if(the_cfggui.flags & CFLG_RESCAN_RIGHT)
+		dp_rescan(&cgu->min->gui->pane[1]);
+	else if(the_cfggui.flags & CFLG_REDISP_RIGHT)
+		dp_redisplay_preserve(&cgu->min->gui->pane[1]);
+
 	if(the_cfggui.flags & CFLG_REBUILD_BOTTOM)
 	{
 		rebuild_bottom(cgu->min);
@@ -173,16 +185,6 @@ static void hide_config(CfgGui *cgu)
 	if(the_cfggui.flags & CFLG_FLUSH_ICONS)
 		ico_flush(cgu->min);
 
-	if(the_cfggui.flags & CFLG_RESCAN_LEFT)
-		dp_rescan(&cgu->min->gui->pane[0]);
-	else if(the_cfggui.flags & CFLG_REDISP_LEFT)
-		dp_redisplay_preserve(&cgu->min->gui->pane[0]);
-
-	if(the_cfggui.flags & CFLG_RESCAN_RIGHT)
-		dp_rescan(&cgu->min->gui->pane[1]);
-	else if(the_cfggui.flags & CFLG_REDISP_RIGHT)
-		dp_redisplay_preserve(&cgu->min->gui->pane[1]);
-
 	if(the_cfggui.flags & CFLG_RESET_KEYBOARD)
 		ctrl_keys_install(cgu->min->cfg.ctrlinfo, cgu->min->gui->kbd_ctx);
 
@@ -216,13 +218,11 @@ static gint evt_ok_clicked(GtkWidget *wid, gpointer user)
 */
 void cfg_save_all(MainInfo *min)
 {
-	const gchar	*root = "GentooConfig", *rcname;
+	const gchar	*root = "GentooConfig", *rcname = get_config_filename(RCNAME);
 	FILE		*out;
 	guint		i;
 	const CfgModule	*page;
 
-	rcname = get_config_filename(RCNAME);
-
 	cfg_modified_clear(min);
 	if((out = xml_put_open(rcname)) != NULL)
 	{
@@ -251,7 +251,6 @@ static gint evt_save_clicked(GtkWidget *wid, gpointer user)
 	MainInfo	*min = cgu->min;
 	guint		i;
 
-	cfg_modified_clear(min);
 	for(i = 0; i < CFG_PAGES; i++)
 	{
 		if(cfg_page[i]->accept != NULL)
@@ -275,23 +274,17 @@ static void build_buttons(CfgGui *cgu)
 {
 	cgu->ok = gtk_button_new_with_label(_("OK"));
 	g_signal_connect(G_OBJECT(cgu->ok), "clicked", G_CALLBACK(evt_ok_clicked), cgu);
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(cgu->dlg)->action_area), cgu->ok, TRUE, TRUE, 0);
+	gtk_dialog_add_action_widget(GTK_DIALOG(cgu->dlg), cgu->ok, GTK_RESPONSE_OK);
 	cgu->save = gtk_button_new_with_label(_("Save"));
 	g_signal_connect(G_OBJECT(cgu->save), "clicked", G_CALLBACK(evt_save_clicked), cgu);
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(cgu->dlg)->action_area), cgu->save, TRUE, TRUE, 0);
+	gtk_dialog_add_action_widget(GTK_DIALOG(cgu->dlg), cgu->save, GTK_RESPONSE_OK);
 	cgu->cancel = gtk_button_new_with_label(_("Cancel"));
 	g_signal_connect(G_OBJECT(cgu->cancel), "clicked", G_CALLBACK(evt_cancel_clicked), cgu);
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(cgu->dlg)->action_area), cgu->cancel, TRUE, TRUE, 0);
+	gtk_dialog_add_action_widget(GTK_DIALOG(cgu->dlg), cgu->cancel, GTK_RESPONSE_CANCEL);
 
-#if GTK_CHECK_VERSION(2, 2, 0)
 	gtk_widget_set_can_default(cgu->ok, TRUE);
 	gtk_widget_set_can_default(cgu->save, TRUE);
 	gtk_widget_set_can_default(cgu->cancel, TRUE);
-#else
-	GTK_WIDGET_SET_FLAGS(cgu->ok, GTK_CAN_DEFAULT);
-	GTK_WIDGET_SET_FLAGS(cgu->save, GTK_CAN_DEFAULT);
-	GTK_WIDGET_SET_FLAGS(cgu->cancel, GTK_CAN_DEFAULT);
-#endif
 	gtk_widget_grab_default(cgu->ok);
 }
 
@@ -334,7 +327,7 @@ gint cfg_configure(MainInfo *min)
 	CfgGui		*cgu = &the_cfggui;
 	gchar		*name = NULL;
 	guint		i;
-	GtkWidget	*hbox, *scwin, *simple, *view;
+	GtkWidget	*hbox, *scwin, *simple;
 	GtkCellRenderer	*cr;
 	GtkTreeViewColumn *vc;
 
@@ -385,24 +378,24 @@ gint cfg_configure(MainInfo *min)
 			}
 		}
 	}
-	hbox  = gtk_hbox_new(FALSE, 0);
+	hbox  = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	scwin = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scwin), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 	/* Build tree view, showing only the first column. */
-	view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(cgu->store));
+	cgu->view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(cgu->store));
 	cr = gtk_cell_renderer_text_new();
 	vc = gtk_tree_view_column_new_with_attributes("(ConfigPages)", cr, "text", 0, NULL);
-	gtk_tree_view_append_column(GTK_TREE_VIEW(view), vc);
-	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(view), FALSE);
-	g_signal_connect(G_OBJECT(view), "cursor_changed", G_CALLBACK(evt_view_cursor_changed), cgu);
-	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scwin), view);
+	gtk_tree_view_append_column(GTK_TREE_VIEW(cgu->view), vc);
+	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(cgu->view), FALSE);
+	g_signal_connect(G_OBJECT(cgu->view), "cursor_changed", G_CALLBACK(evt_view_cursor_changed), cgu);
+	gtk_container_add(GTK_CONTAINER(scwin), cgu->view);
 	gtk_box_pack_start(GTK_BOX(hbox), scwin, FALSE, FALSE, 0);
 
 	scwin = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scwin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scwin), cgu->nbook);
+	gtk_container_add(GTK_CONTAINER(scwin), cgu->nbook);
 	gtk_box_pack_start(GTK_BOX(hbox), scwin, TRUE, TRUE, 0);
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(cgu->dlg)->vbox), hbox, TRUE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(cgu->dlg))), hbox, TRUE, TRUE, 0);
 	build_buttons(cgu);
 
 	return cfg_configure(min);		/* Recurse! */
@@ -410,6 +403,50 @@ gint cfg_configure(MainInfo *min)
 
 /* ----------------------------------------------------------------------------------------- */
 
+static gboolean goto_iterate(const CfgGui *cgu, GtkTreeIter *iter, const char *label)
+{
+	GtkTreeIter	citer;
+	const gchar	*lh;
+	gint		lp;
+
+	do
+	{
+		gtk_tree_model_get(GTK_TREE_MODEL(cgu->store), iter, 0, &lh, 1, &lp, -1);
+		if(strcmp(lh, label) == 0)
+		{
+			GtkTreePath	*path;
+
+			gtk_notebook_set_current_page(GTK_NOTEBOOK(cgu->nbook), lp);
+			path = gtk_tree_model_get_path(GTK_TREE_MODEL(cgu->store), iter);
+			gtk_tree_view_expand_to_path(GTK_TREE_VIEW(cgu->view), path);
+			gtk_tree_view_set_cursor(GTK_TREE_VIEW(cgu->view), path, NULL, FALSE);
+			gtk_tree_path_free(path);
+			return TRUE;
+		}
+		if(gtk_tree_model_iter_children(GTK_TREE_MODEL(cgu->store), &citer, iter))
+		{
+			if(goto_iterate(cgu, &citer, label))
+				return TRUE;
+		}
+	} while(gtk_tree_model_iter_next(GTK_TREE_MODEL(cgu->store), iter));
+
+	return FALSE;
+}
+
+/* 2014-12-26 -	Switch to a named configuration page. */
+void cfg_goto_page(const char *label)
+{
+	const CfgGui	*cgu = &the_cfggui;
+	GtkTreeIter	iter;
+
+	if(gtk_tree_model_get_iter_first(GTK_TREE_MODEL(cgu->store), &iter))
+	{
+		goto_iterate(cgu, &iter, label);
+	}
+}
+
+/* ----------------------------------------------------------------------------------------- */
+
 void cfg_tree_level_begin(const gchar *label)
 {
 	CfgGui	*cgu = &the_cfggui;
