@@ -22,27 +22,29 @@ enum {
 
 /* These are flags returned by cfg_load_config(). */
 enum {
-	CLDF_NONE_FOUND		= (1<<0)		/* There was no config available. */
+	CLDF_NONE_FOUND		= (1 << 0)		/* There was no config available. */
 };
 
-extern gint	cfg_configure(MainInfo *min);
+gint	cfg_configure(MainInfo *min);
+
+void	cfg_goto_page(const char *label);
 
 /* Config modules which have internal hierarchy use these from their init() function.
 ** Simpler modules don't, they just return their single page and set the label, as
 ** with the old tabs-based layout.
 */
-extern void	cfg_tree_level_begin(const gchar *label);
-extern void	cfg_tree_level_append(const gchar *label, GtkWidget *page);
-extern void	cfg_tree_level_replace(GtkWidget *old, GtkWidget *new);
-extern void	cfg_tree_level_end(void);
+void	cfg_tree_level_begin(const gchar *label);
+void	cfg_tree_level_append(const gchar *label, GtkWidget *page);
+void	cfg_tree_level_replace(GtkWidget *old, GtkWidget *new);
+void	cfg_tree_level_end(void);
 
-extern void	cfg_save_all(MainInfo *min);
+void	cfg_save_all(MainInfo *min);
 
-extern guint32	cfg_load_config(MainInfo *min);
+guint32	cfg_load_config(MainInfo *min);
 
-extern void	cfg_set_flags(guint32 flags);
+void	cfg_set_flags(guint32 flags);
 
-extern void	cfg_modified_set(MainInfo *min);
-extern void	cfg_modified_clear(MainInfo *min);
+void	cfg_modified_set(MainInfo *min);
+void	cfg_modified_clear(MainInfo *min);
 
 #endif		/* CONFIGURE_H */
