@@ -272,7 +272,7 @@ gfloat dph_vpos_get(const DirPane *dp)
 	GtkAdjustment	*adj;
 
 	if((adj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(dp->scwin))) != NULL)
-		return adj->value;
+		return gtk_adjustment_get_value(adj);
 	return -1.0f;
 }
 
