@@ -68,7 +68,6 @@ enum { COL_FILE = 0, COL_INFO, COL_FTYPE, COL_FILENAME_COLLATE_KEY, COL_LINK_TAR
 
 static void	clear_total_stats(DirPane *dp);
 static void	clear_selection_stats(SelInfo *si);
-static void	update_selection_stats(DirPane *dp);
 
 static void	dp_activate_queued(DirPane *dp);
 
@@ -93,11 +92,7 @@ void dp_initialize(DirPane *dp, size_t num)
 /* 2010-06-02 -	Encapsulate this tiny porting/compatibility issue to spare us from #ifdef:s all over. */
 gboolean dp_realized(const MainInfo *min)
 {
-#if GTK_CHECK_VERSION(2, 20, 0)
 	return gtk_widget_get_realized(min->gui->panes);
-#else
-	return GTK_WIDGET_REALIZED(min->gui->panes);
-#endif
 }
 
 /* ----------------------------------------------------------------------------------------- */
@@ -287,7 +282,7 @@ static gboolean evt_path_key_press(GtkWidget *wid, GdkEventKey *evt, gpointer us
 {
 	DirPane	*dp = user;
 
-	if(evt->keyval == GDK_Escape)
+	if(evt->keyval == GDK_KEY_Escape)
 		dp_path_unfocus(dp);
 	return FALSE;
 }
@@ -325,7 +320,7 @@ static void evt_hide_clicked(GtkWidget *wid, gpointer user)
 	DirPane	*dp = user;
 
 	dp_activate(dp);
-	dp->main->cfg.dp_format[dp->index].hide_allowed = GTK_TOGGLE_BUTTON(wid)->active;
+	dp->main->cfg.dp_format[dp->index].hide_allowed = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wid));
 	csq_execute(dp->main, "DirRescan");
 }
 
@@ -495,35 +490,6 @@ void dp_redisplay_preserve(DirPane *dp)
 
 /* ----------------------------------------------------------------------------------------- */
 
-/* 1999-05-10 -	Unfocus the currently focused row of <dp>, if any. */
-void dp_unfocus(DirPane *dp)
-{
-	if(dp != NULL)
-	{
-		/* We use a depth-two path, which the actual view will never have. This qualifies
-		** (it seems) as a path that is "invalid" to the model, causing the focus to be
-		** removed.
-		**/
-		static GtkTreePath	*nowhere = NULL;
-
-		if(nowhere == NULL)
-			nowhere = gtk_tree_path_new_from_indices(1, 1, -1);
-		if(nowhere != NULL)
-			gtk_tree_view_set_cursor(GTK_TREE_VIEW(dp->view), nowhere, NULL, FALSE);
-	}
-}
-
-/* 2003-11-30 -	A new dir has been entered, figure out if there should be focusing. */
-void dp_focus_enter_dir(DirPane *dp)
-{
-	if(dp->focus_row != -1)
-		return;
-	if(dp->old_focus_row == -1)
-		return;
-}
-
-/* ----------------------------------------------------------------------------------------- */
-
 /* 1998-05-25 -	Moved this old workhorse into the new dirpane module, and discovered that
 **		it wasn't commented. Lazy me. Also modified its prototype, to take the
 **		DirPane to display rather than figuring it out from MainInfo.
@@ -564,48 +530,22 @@ void dp_show_stats(DirPane *dp)
 	}
 }
 
-/* 2011-04-10 -	Linear blend from a to b, out will be equal to a when alpha == 0, and equal to b when alpha == 1.0. */
-static void blend_colors(GdkColor *out, const GdkColor *a, const GdkColor *b, float alpha)
-{
-	const float	oma = 1.f - alpha;
-
-	out->red = oma * a->red + alpha * b->red;
-	out->green = oma * a->green + alpha * b->green;
-	out->blue = oma * a->blue + alpha * b->blue;
-}
-
 /* 2010-08-03 -	Set the activate-status of the given pane's columns (that's what "col" refers to). Slighly hackish. */
 static void dp_set_col_active(DirPane *dp, gboolean active)
 {
-	guint	i;
-
 	if(dp != NULL)
 	{
 		GtkTreeViewColumn	*column;
-		const GtkStyle		*style;
-		GdkColor		render;
-		const GdkColor		*setcolor;
-
-		if((style = gtk_widget_get_style(dp->view)) != NULL)
-			blend_colors(&render, &style->bg[GTK_STATE_NORMAL], &style->bg[GTK_STATE_SELECTED], 0.5f);
-		else
-			render.red = render.green = render.blue = (guint16) ~0u;
-		setcolor = active ? &render : NULL;
+		guint			i;
 
 		for(i = 0; (column = gtk_tree_view_get_column(GTK_TREE_VIEW(dp->view), i)) != NULL; i++)
 		{
-			GtkWidget	*header = gtk_tree_view_column_get_widget(column);
+			GtkWidget	*header = gtk_tree_view_column_get_button(column);
 
 			if(header != NULL)
-			{
  				gtk_widget_set_sensitive(header, active);
-				if((header = gtk_widget_get_ancestor(header, GTK_TYPE_BUTTON)) != NULL)
-				{
-					gtk_widget_modify_bg(header, GTK_STATE_NORMAL, setcolor);	
-					gtk_widget_modify_bg(header, GTK_STATE_PRELIGHT, setcolor);	
-				}
-			}
 		}
+		gtk_widget_set_name(dp->view, active ? "pane-current" : "pane");
 	}
 }
 
@@ -613,10 +553,7 @@ static void dp_set_col_active(DirPane *dp, gboolean active)
 static void activate_render(DirPane *from, DirPane *to)
 {
 	if(from != NULL)
-	{
-		dp_unfocus(from);
 		dp_set_col_active(from, FALSE);
-	}
 	if(to != NULL)
 	{
 		dp_set_col_active(to, TRUE);
@@ -679,8 +616,6 @@ gboolean dp_has_selection(DirPane *dp)
 {
 	if(dp->dbclk_row != -1)
 		return TRUE;
-	if(dp->focus_row != -1)
-		return TRUE;
 	return gtk_tree_selection_count_selected_rows(gtk_tree_view_get_selection(GTK_TREE_VIEW(dp->view))) > 0 && !no_repeat;
 }
 
@@ -910,18 +845,25 @@ void dp_path_focus(DirPane *dp)
 
 void dp_path_unfocus(DirPane *dp)
 {
-	gtk_widget_grab_focus(dp->hide);
-#if 0
-	GdkEvent	fe;
+	gtk_widget_grab_focus(dp->view);
+}
 
-	/* HACK: This fakes a focus-out event on the path entry combo widget, which seems to work. */
-	fe.focus_change.type = GDK_FOCUS_CHANGE;
-	/* FIXME: If we required GTK+ 2.14 and greater, we could use gtk_widget_get_window(). */
-	fe.focus_change.window = GTK_WIDGET(dp->path)->window;
-	fe.focus_change.send_event = TRUE;
-	fe.focus_change.in = FALSE;
-	gtk_main_do_event(&fe);
-#endif
+/* 2013-07-09 -	Clear the FType column. This is useful when re-scanning panes after editing the
+ *		FType definitions in the configuration window. It prevents crashes due to stale
+ *		pointers being followed.
+*/
+static void untype_rows(DirPane *dp)
+{
+	GtkTreeModel	*model = dp_get_tree_model(dp);
+	GtkTreeIter	iter;
+
+	if(gtk_tree_model_get_iter_first(model, &iter))
+	{
+		do
+		{
+			gtk_list_store_set(GTK_LIST_STORE(model), &iter, COL_FTYPE, NULL, -1);
+		} while(gtk_tree_model_iter_next(model, &iter));
+	}
 }
 
 /* 2010-06-05 -	Internal "do it" routine to enter a new directory. Uses GIO error handling for all I/O. */
@@ -941,6 +883,7 @@ static gboolean do_enter_dir(DirPane *dp, const gchar *path, GError **err)
 		if(dp->dir.root != NULL)
 			g_object_unref(dp->dir.root);
 		dp->dir.root = here;
+		untype_rows(dp);	/* Clearing is not atomic, and can cause redraws. */
 		gtk_list_store_clear(dp->dir.store);
 		g_strlcpy(dp->dir.path, path, sizeof dp->dir.path);
 
@@ -1036,6 +979,8 @@ static gboolean do_enter_dir(DirPane *dp, const gchar *path, GError **err)
 					}
 					g_clear_error(&terr);
 				}
+					else
+						tfi = NULL;
 			}
 			else
 				tfi = NULL;
@@ -1046,7 +991,7 @@ static gboolean do_enter_dir(DirPane *dp, const gchar *path, GError **err)
 					COL_INFO, fi,
 					COL_FLAGS, flags,
 					COL_FILENAME_COLLATE_KEY, ck,
-					tfi != NULL ? COL_LINK_TARGET_INFO : -1, tfi,	/* Not too clever, I hope. */
+					(tfi != NULL) ? COL_LINK_TARGET_INFO : -1, tfi,	/* Not too clever, I hope. */
 					-1);
 			statistics_add_row(dp, fi);
 			g_free(ck);
@@ -1085,7 +1030,6 @@ gboolean dp_enter_dir(DirPane *dp, const gchar *path)
 	gboolean	ok;
 
 	clear_stats(dp);
-	dp_unfocus(dp);
 
 	completion_clear(dp);
 
@@ -1203,7 +1147,8 @@ void dp_history_set(DirPane *dp, const GList *locations)
 /* 2009-07-04 -	Refreshes the split, making sure it's correctly sized according to current settings. */
 void dp_split_refresh(MainInfo *min)
 {
-	gint	pw, ph, np, pos;
+	GdkWindow	*pwin;
+	gint		np, pos = -1;
 
 	if(min == NULL || min->gui == NULL || min->gui->panes == NULL)
 		return;
@@ -1213,8 +1158,8 @@ void dp_split_refresh(MainInfo *min)
 	/* Inspect the size of the GtkPaned that holds the panes; the window's size
 	** doesn't work for vertical due to buttons. This is slightly hackish.
 	*/
-	gdk_window_get_size(min->gui->panes->window, &pw, &ph);
-	np = min->cfg.dp_paning.orientation == DPORIENT_HORIZ ? pw : ph;
+	pwin = gtk_widget_get_window(min->gui->panes);
+	np = min->cfg.dp_paning.orientation == DPORIENT_HORIZ ? gdk_window_get_width(pwin) : gdk_window_get_height(pwin);
 
 	/* Compute the proper new position, depending on the active paning mode. */
 	switch(min->cfg.dp_paning.mode)
@@ -1882,13 +1827,10 @@ static gint cmp_type(GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpoint
 {
 	GFileInfo	*fia, *fib;
 	FType		*ta, *tb;
-	guint32		na, nb;
 	gint		r;
 
 	gtk_tree_model_get(model, a, COL_INFO, &fia, COL_FTYPE, &ta, -1);
-	na = g_file_info_get_attribute_uint32(fia, G_FILE_ATTRIBUTE_UNIX_NLINK);
 	gtk_tree_model_get(model, b, COL_INFO, &fib, COL_FTYPE, &tb, -1);
-	nb = g_file_info_get_attribute_uint32(fib, G_FILE_ATTRIBUTE_UNIX_NLINK);
 	if(ta != NULL && tb != NULL)
 		r = sort_result(strcmp(ta->name, tb->name), model, a, b, fia, fib);
 	else
@@ -1943,39 +1885,11 @@ static void evt_selection_changed(GtkTreeSelection *ts, gpointer user)
 	dp_show_stats(user);
 }
 
-/* 2011-04-10 -	React to column property change, and track width changes. */
-static void evt_tree_column_notify(GObject *object, GParamSpec *pspec, gpointer user)
+static void dp_build_list(DirPane *dp, const DPFormat *fmt)
 {
-	const gint	index = GPOINTER_TO_INT(g_object_get_data(object, "index"));
-
-	if(strcmp(pspec->name, "width") == 0)
-	{
-		DirPane		*dp = user;
-		MainInfo	*min = dp->main;
-		DPFormat	*fmt = &min->cfg.dp_format[dp->index];
-
-		if(index >= 0 && index < fmt->num_columns)
-		{
-			gpointer	tmp;
-			gint		new_width;
-
-			g_object_get(object, "width", &tmp, NULL);
-			new_width = GPOINTER_TO_INT(tmp);
-			if(new_width != fmt->format[index].width)
-			{
-				fmt->format[index].width = new_width;
-				cfg_modified_set(min);
-			}
-		}
-	}
-}
-
-static void dp_build_list2(DirPane *dp, const DPFormat *fmt)
-{
-	gint		i;
 	const struct {
-	DPContent		content;
-	GtkTreeIterCompareFunc	func;
+		DPContent		content;
+		GtkTreeIterCompareFunc	func;
 	} sorters[] = {
 			{ DPC_NAME, cmp_name },
 			{ DPC_SIZE, cmp_size }, { DPC_BLOCKS, cmp_size }, { DPC_BLOCKSIZE, cmp_size },
@@ -1992,6 +1906,7 @@ static void dp_build_list2(DirPane *dp, const DPFormat *fmt)
 			{ DPC_TYPE, cmp_type },
 			{ DPC_URI_NOFILE, cmp_uri },
 	};
+	gint	i;
 
 	dp->dir.store = gtk_list_store_new(COL_NUMBER_OF_COLUMNS, G_TYPE_OBJECT, G_TYPE_OBJECT, G_TYPE_POINTER, G_TYPE_STRING, G_TYPE_OBJECT, G_TYPE_UINT );
 	for(i = 0; i < sizeof sorters / sizeof *sorters; i++)
@@ -2000,6 +1915,7 @@ static void dp_build_list2(DirPane *dp, const DPFormat *fmt)
 	dp->view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(dp->dir.store));
 	gtk_tree_view_set_search_column(GTK_TREE_VIEW(dp->view), -1);
 	gtk_tree_view_set_enable_search(GTK_TREE_VIEW(dp->view), FALSE);
+	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(dp->view), fmt->rules);
 
 	/* Connect signals. */
 	g_signal_connect(G_OBJECT(dp->view), "button_press_event", G_CALLBACK(evt_dirpane_button_press), dp);
@@ -2041,6 +1957,7 @@ static void dp_build_list2(DirPane *dp, const DPFormat *fmt)
 			case DPC_DEVICE:
 			case DPC_DEVMIN:
 			case DPC_DEVMAJ:
+			case DPC_TYPE:
 			case DPC_URI_NOFILE:
 				cr = gtk_cell_renderer_text_new();
 				break;
@@ -2049,6 +1966,7 @@ static void dp_build_list2(DirPane *dp, const DPFormat *fmt)
 				break;
 			default:
 				g_warning("Unsupported dirpane content type %d detected", fmt->format[i].content);
+				continue;
 			}
 
 			tc = gtk_tree_view_column_new();
@@ -2076,14 +1994,14 @@ static void dp_build_list2(DirPane *dp, const DPFormat *fmt)
 			/* Set the content ID as the sort ID; they're unique and all. */
 			gtk_tree_view_column_set_sort_column_id(tc, fmt->format[i].content);
 			/* Set the size, which we try to control ourselves. */
+			gtk_tree_view_column_set_min_width(tc, fmt->format[i].width);
 			gtk_tree_view_column_set_sizing(tc, GTK_TREE_VIEW_COLUMN_FIXED);
-			gtk_tree_view_column_set_fixed_width(tc, fmt->format[i].width);
+			gtk_tree_view_column_set_resizable(tc, FALSE);
+			gtk_tree_view_column_set_expand(tc, FALSE);
 			gtk_tree_view_append_column(GTK_TREE_VIEW(dp->view), tc);
 			gtk_tree_view_column_pack_start(tc, cr, TRUE);
-			/* Make resizable, expose the column index, and connect handler. */
-			g_object_set(tc, "resizable", GINT_TO_POINTER(TRUE), NULL);
+			/* Expose the column index. */
 			g_object_set_data(G_OBJECT(tc), "index", GINT_TO_POINTER(i));
-			g_signal_connect(G_OBJECT(tc), "notify", G_CALLBACK(evt_tree_column_notify), dp);
 			/* This must be done after column is appended. */
 			gtk_tree_view_column_set_cell_data_func(tc, cr, cdf, user, NULL);
 		}
@@ -2120,30 +2038,27 @@ GtkWidget * dp_build(MainInfo *min, DPFormat *fmt, DirPane *dp)
 	if(dp == NULL)
 		return NULL;
 
-	dp->vbox = gtk_vbox_new(FALSE, 0);
+	dp->vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	/* "Internal" hbox, holds the scrolled window of the pane and the optional
 	** "huge" parent button. If the latter is disabled, it's redundant, but hey.
 	*/
-	ihbox = gtk_hbox_new(FALSE, 0);
+	ihbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 
 	dp->scwin = gtk_scrolled_window_new(NULL, NULL);
 	if(fmt->sbar_pos == SBP_LEFT)
 		gtk_scrolled_window_set_placement(GTK_SCROLLED_WINDOW(dp->scwin), GTK_CORNER_TOP_RIGHT);
 	else if(fmt->sbar_pos == SBP_RIGHT)
 		gtk_scrolled_window_set_placement(GTK_SCROLLED_WINDOW(dp->scwin), GTK_CORNER_TOP_LEFT);
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(dp->scwin), GTK_POLICY_AUTOMATIC, fmt->scrollbar_always ? GTK_POLICY_ALWAYS : GTK_POLICY_AUTOMATIC);
 
-	dp_build_list2(dp, fmt);
+	dp_build_list(dp, fmt);
 
 	dp->notebook = gtk_notebook_new();
 	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(dp->notebook), FALSE);
 	gtk_notebook_set_show_border(GTK_NOTEBOOK(dp->notebook), FALSE);
-	hbox = gtk_hbox_new(FALSE, 0);
-	dp->parent = gui_stock_button_new(GTK_STOCK_GO_UP);
-#if GTK_CHECK_VERSION(2, 2, 0)
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
+	dp->parent = gtk_button_new_from_icon_name("go-up", GTK_ICON_SIZE_MENU);
 	gtk_widget_set_can_focus(dp->parent, FALSE);
-#else
-	GTK_WIDGET_UNSET_FLAGS(dp->parent, GTK_CAN_FOCUS);
-#endif
 	g_signal_connect(G_OBJECT(dp->parent), "clicked", G_CALLBACK(evt_parent_clicked), dp);
 	gtk_box_pack_start(GTK_BOX(hbox), dp->parent, FALSE, FALSE, 0);
 	gtk_widget_set_tooltip_text(dp->parent, _("Move up to the parent directory"));
@@ -2195,11 +2110,7 @@ GtkWidget * dp_build(MainInfo *min, DPFormat *fmt, DirPane *dp)
 		gtk_button_set_relief(GTK_BUTTON(dp->hparent), GTK_RELIEF_NONE);
 		g_signal_connect(G_OBJECT(dp->hparent), "clicked", G_CALLBACK(evt_hugeparent_clicked), dp);
 		gtk_widget_set_tooltip_text(dp->hparent, _("Move up to the parent directory"));
-#if GTK_CHECK_VERSION(2, 2, 0)
 		gtk_widget_set_can_focus(dp->hparent, FALSE);
-#else
-		GTK_WIDGET_UNSET_FLAGS(dp->hparent, GTK_CAN_FOCUS);
-#endif
 		if(dp->index == 0)
 		{
 			gtk_box_pack_start(GTK_BOX(ihbox), dp->hparent, FALSE, FALSE, 0);
@@ -2234,15 +2145,13 @@ GtkWidget * dp_build(MainInfo *min, DPFormat *fmt, DirPane *dp)
 
 		if((fdesc = pango_font_description_from_string(fmt->font_name)) != NULL)
 		{
-			gtk_widget_modify_font(dp->view, fdesc);
+			gtk_widget_override_font(dp->view, fdesc);
 			pango_font_description_free(fdesc);
 		}
 	}
 
 	gtk_widget_show_all(dp->vbox);
 	dp->dbclk_row = -1;
-	dp->focus_row = -1;
-	dp->old_focus_row = -1;
 	clear_stats(dp);
 
 	return dp->vbox;
