@@ -167,7 +167,8 @@ GtkWidget * dpf_get_content_combo_box(GCallback func, gpointer user)
 	cr = gtk_cell_renderer_text_new();
 	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(wid), cr, TRUE);
 	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(wid), cr, "text", 0, NULL);
-	g_signal_connect(G_OBJECT(wid), "changed", func, user);
+	if(func != NULL)
+		g_signal_connect(G_OBJECT(wid), "changed", func, user);
 
 	return wid;
 }
@@ -267,22 +268,23 @@ void dpf_init_defaults(CfgInfo *cfg)
 	DPFormat	*fmt = &cfg->dp_format[0];
 	guint		i;
 
-	for(i = 0; i < 15; i++)
+	for(i = 0; i < DPC_NUM_TYPES; i++)
+	{
 		dpf_set_default_format(&fmt->format[i], (DPContent) i);
+	}
 	fmt->num_columns = i;
-
 	fmt->sort.content = DPC_NAME;
 	fmt->sort.invert  = FALSE;
 	fmt->sort.mode    = DPS_DIRS_FIRST;
-
 	strcpy(fmt->def_path, "file:///");
 	fmt->path_above = FALSE;
 	fmt->hide_allowed = TRUE;
-	fmt->scrollbar_always = FALSE;
-
-	cfg->dp_format[1] = cfg->dp_format[0];
-
-	strcpy(cfg->dp_format[1].def_path, "file:///");
+	fmt->scrollbar_always = TRUE;
+	fmt->huge_parent = FALSE;
+	fmt->set_font = FALSE;
+	fmt->rubber_banding = FALSE;
+	fmt->rules = FALSE;
+	cfg->dp_format[1] = cfg->dp_format[0];	/* Duplicate default formats for second pane. */
 
 	cfg->dp_paning.orientation = DPORIENT_HORIZ;
 	cfg->dp_paning.mode	   = DPSPLIT_RATIO;
@@ -401,7 +403,7 @@ gboolean dpf_get_content(MainInfo *min, const DirPane *dp, const DirRow2 *row, D
 /* 2010-02-22 -	Refactored this into a stand-alone function since it's not something I would like
 **		seeing copied elsewhere (like, say, in the Styles config preview code).
 */
-void dpf_cell_set_style_colors(GtkCellRenderer *r, const Style *style, gboolean do_fg)
+void dpf_cell_set_style_colors(GtkCellRenderer *r, const Style *style, gboolean do_fg, gboolean do_bg)
 {
 	const GdkColor	*fg = NULL, *bg = NULL;
 
@@ -409,7 +411,8 @@ void dpf_cell_set_style_colors(GtkCellRenderer *r, const Style *style, gboolean
 	{
 		if(do_fg)
 			fg = stl_style_property_get_color(style, SPN_COL_UNSEL_FG);
-		bg = stl_style_property_get_color(style, SPN_COL_UNSEL_BG);
+		if(do_bg)
+			bg = stl_style_property_get_color(style, SPN_COL_UNSEL_BG);
 	}
 	if(do_fg)
 	{
@@ -418,26 +421,51 @@ void dpf_cell_set_style_colors(GtkCellRenderer *r, const Style *style, gboolean
 		else
 			g_object_set(G_OBJECT(r), "foreground-set", FALSE, NULL);
 	}
-
-	if(bg != NULL)
-		g_object_set(G_OBJECT(r), "cell-background-gdk", bg, "cell-background-set", TRUE, NULL);
-	else
-		g_object_set(G_OBJECT(r), "cell-background-set", FALSE, NULL);
+	if(do_bg)
+	{
+		if(bg != NULL)
+			g_object_set(G_OBJECT(r), "cell-background-gdk", bg, "cell-background-set", TRUE, NULL);
+		else
+			g_object_set(G_OBJECT(r), "cell-background-set", FALSE, NULL);
+	}
 }
 
 /* 2009-07-15 -	Formats the colors, if any, for the given renderer. The foreground is optional,
 **		since not all renderers (such as pixbuf) support a foreground color. This needs to be done
 **		per-column, there's no way of doing it per-row in GTK+ 2.0. :/
 */
-static void format_colors_row2(GtkTreeModel *model, const DirRow2 *row, GtkCellRenderer *r, gboolean do_fg)
+static void format_colors_row(GtkTreeModel *model, const DirRow2 *row, GtkCellRenderer *r, gboolean do_fg, gboolean do_bg)
 {
-	FType	*ft;
-	Style	*st = NULL;
+	if(do_fg || do_bg)
+	{
+		FType	*ft;
+		Style	*st;
 
-	ft = dp_row_get_ftype(model, row);
-	if(ft != NULL)
-		st = ft->style;
-	dpf_cell_set_style_colors(r, st, do_fg);
+		ft = dp_row_get_ftype(model, row);
+		if(ft != NULL)
+			st = ft->style;
+		else
+			st = NULL;
+		dpf_cell_set_style_colors(r, st, do_fg, do_bg);
+	}
+}
+
+/* 2012-05-29 -	Figure out whether the DirPane being formatted using <extra> would like to override the background
+**		color for pane rows, or not. This is Rather Extreme Trickery, you're not expected to consider this
+**		much of a shining beacon when it comes to design. But it is kind of practical, and should be safe
+**		as far as I know.
+*/
+static gboolean get_do_bg(GtkTreeViewColumn *tc, const DpCExtra *extra)
+{
+	/* Rely on the fact that the DpCExtra is embedded in a DpCFmt,
+	** which in turn is part of a DPFormat which has the bit we're after.
+	** If only we knew the column's index ...
+	*/
+	const DpCFmt	*cfmt = (const DpCFmt *) ((const unsigned char *) extra - offsetof(DpCFmt, extra));
+	const gint	index = GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(tc), "index"));
+	const DPFormat	*dpf = (const DPFormat *) ((const unsigned char *) cfmt - offsetof(DPFormat, format[index]));
+
+	return !dpf->rules;
 }
 
 static void cdf_name(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTreeModel *model, GtkTreeIter *iter, gpointer user)
@@ -476,7 +504,7 @@ static void cdf_name(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTreeM
 	else
 		g_object_set(renderer, "text", dname, NULL);
 
-	format_colors_row2(model, iter, renderer, TRUE);
+	format_colors_row(model, iter, renderer, TRUE, get_do_bg(col, user));
 }
 
 static void cdf_size(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTreeModel *model, GtkTreeIter *iter, gpointer user)
@@ -493,7 +521,7 @@ static void cdf_size(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTreeM
 	size = dp_row_get_size(model, iter);
 	sze_put_offset(buf, sizeof buf, size, cfg->unit, cfg->digits, cfg->ticks ? cfg->tick : '\0');
 	g_object_set(renderer, "text", buf, NULL);
-	format_colors_row2(model, iter, renderer, TRUE);
+	format_colors_row(model, iter, renderer, TRUE, get_do_bg(col, (const DpCExtra *) cfg));
 }
 
 static void cdf_blocks(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTreeModel *model, GtkTreeIter *iter, gpointer user)
@@ -504,12 +532,12 @@ static void cdf_blocks(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTre
 	blocks = dp_row_get_blocks(model, iter);
 	g_snprintf(buf, sizeof buf, "%" G_GUINT64_FORMAT, blocks);
 	g_object_set(renderer, "text", buf, NULL);
-	format_colors_row2(model, iter, renderer, TRUE);
+	format_colors_row(model, iter, renderer, TRUE, get_do_bg(col, user));
 }
 
 static void cdf_icon(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTreeModel *model, GtkTreeIter *iter, gpointer user)
 {
-	FType		*ftype;
+	FType	*ftype;
 
 	if((ftype = dp_row_get_ftype(model, iter)) != NULL)
 	{
@@ -525,10 +553,10 @@ static void cdf_icon(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTreeM
 				g_object_set(renderer, "pixbuf", pbuf, NULL);
 		}
 	}
-	format_colors_row2(model, iter, renderer, FALSE);
+	format_colors_row(model, iter, renderer, FALSE, FALSE);
 }
 
-static void do_cdf_time(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTreeModel *model, GtkTreeIter *iter, const gchar *format, guint64 timestamp)
+static void do_cdf_time(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTreeModel *model, GtkTreeIter *iter, const gchar *format, guint64 timestamp, gboolean do_bg)
 {
 	const time_t	now = (time_t) (timestamp & 0xffffffffu);
 	struct tm	local;
@@ -546,27 +574,27 @@ static void do_cdf_time(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTr
 	}
 	else
 		g_object_set(renderer, "text", "", NULL);
-	format_colors_row2(model, iter, renderer, TRUE);
+	format_colors_row(model, iter, renderer, TRUE, do_bg);
 }
 
 static void cdf_atime(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTreeModel *model, GtkTreeIter *iter, gpointer user)
 {
-	do_cdf_time(col, renderer, model, iter, user, dp_row_get_time_accessed(model, iter));
+	do_cdf_time(col, renderer, model, iter, user, dp_row_get_time_accessed(model, iter), get_do_bg(col, user));
 }
 
 static void cdf_crtime(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTreeModel *model, GtkTreeIter *iter, gpointer user)
 {
-	do_cdf_time(col, renderer, model, iter, user, dp_row_get_time_created(model, iter));
+	do_cdf_time(col, renderer, model, iter, user, dp_row_get_time_created(model, iter), get_do_bg(col, user));
 }
 
 static void cdf_mtime(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTreeModel *model, GtkTreeIter *iter, gpointer user)
 {
-	do_cdf_time(col, renderer, model, iter, user, dp_row_get_time_modified(model, iter));
+	do_cdf_time(col, renderer, model, iter, user, dp_row_get_time_modified(model, iter), get_do_bg(col, user));
 }
 
 static void cdf_chtime(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTreeModel *model, GtkTreeIter *iter, gpointer user)
 {
-	do_cdf_time(col, renderer, model, iter, user, dp_row_get_time_changed(model, iter));
+	do_cdf_time(col, renderer, model, iter, user, dp_row_get_time_changed(model, iter), get_do_bg(col, user));
 }
 
 static void cdf_devmax(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTreeModel *model, GtkTreeIter *iter, gpointer user)
@@ -575,7 +603,7 @@ static void cdf_devmax(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTre
 
 	g_snprintf(buf, sizeof buf, user, dp_row_get_device(model, iter) >> 8);
 	g_object_set(renderer, "text", buf, NULL);
-	format_colors_row2(model, iter, renderer, TRUE);
+	format_colors_row(model, iter, renderer, TRUE, get_do_bg(col, user));
 }
 
 static void cdf_device(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTreeModel *model, GtkTreeIter *iter, gpointer user)
@@ -584,7 +612,7 @@ static void cdf_device(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTre
 
 	g_snprintf(buf, sizeof buf, user, dp_row_get_device(model, iter));
 	g_object_set(renderer, "text", buf, NULL);
-	format_colors_row2(model, iter, renderer, TRUE);
+	format_colors_row(model, iter, renderer, TRUE, get_do_bg(col, user));
 }
 
 static void cdf_devmin(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTreeModel *model, GtkTreeIter *iter, gpointer user)
@@ -593,7 +621,7 @@ static void cdf_devmin(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTre
 
 	g_snprintf(buf, sizeof buf, user, dp_row_get_device(model, iter) & 255);
 	g_object_set(renderer, "text", buf, NULL);
-	format_colors_row2(model, iter, renderer, TRUE);
+	format_colors_row(model, iter, renderer, TRUE, get_do_bg(col, user));
 }
 
 static void cdf_gidnum(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTreeModel *model, GtkTreeIter *iter, gpointer user)
@@ -602,7 +630,7 @@ static void cdf_gidnum(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTre
 
 	g_snprintf(buf, sizeof buf, user, dp_row_get_gid(model, iter));
 	g_object_set(renderer, "text", buf, NULL);
-	format_colors_row2(model, iter, renderer, TRUE);
+	format_colors_row(model, iter, renderer, TRUE, get_do_bg(col, user));
 }
 
 static void cdf_gidstr(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTreeModel *model, GtkTreeIter *iter, gpointer user)
@@ -620,7 +648,7 @@ static void cdf_gidstr(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTre
 		g_snprintf(buf, sizeof buf, user, gid);
 		g_object_set(renderer, "text", buf, NULL);
 	}
-	format_colors_row2(model, iter, renderer, TRUE);
+	format_colors_row(model, iter, renderer, TRUE, get_do_bg(col, user));
 }
 
 static void cdf_modenum(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTreeModel *model, GtkTreeIter *iter, gpointer user)
@@ -629,7 +657,7 @@ static void cdf_modenum(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTr
 
 	g_snprintf(buf, sizeof buf, user, dp_row_get_mode(model, iter));
 	g_object_set(renderer, "text", buf, NULL);
-	format_colors_row2(model, iter, renderer, TRUE);
+	format_colors_row(model, iter, renderer, TRUE, get_do_bg(col, user));
 }
 
 static void cdf_modestr(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTreeModel *model, GtkTreeIter *iter, gpointer user)
@@ -638,7 +666,7 @@ static void cdf_modestr(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTr
 
 	stu_mode_to_text(buf, sizeof buf, dp_row_get_mode(model, iter));
 	g_object_set(renderer, "text", buf, NULL);
-	format_colors_row2(model, iter, renderer, TRUE);
+	format_colors_row(model, iter, renderer, TRUE, get_do_bg(col, user));
 }
 
 static void cdf_type(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTreeModel *model, GtkTreeIter *iter, gpointer user)
@@ -648,7 +676,7 @@ static void cdf_type(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTreeM
 	if((ft = dp_row_get_ftype(model, iter)) != NULL)
 	{
 		g_object_set(renderer, "text", ft->name, NULL);
-		format_colors_row2(model, iter, renderer, TRUE);
+		format_colors_row(model, iter, renderer, TRUE, get_do_bg(col, user));
 	}
 }
 
@@ -658,7 +686,7 @@ static void cdf_uidnum(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTre
 
 	g_snprintf(buf, sizeof buf, "%u", dp_row_get_uid(model, iter));
 	g_object_set(renderer, "text", buf, NULL);
-	format_colors_row2(model, iter, renderer, TRUE);
+	format_colors_row(model, iter, renderer, TRUE, get_do_bg(col, user));
 }
 
 static void cdf_uidstr(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTreeModel *model, GtkTreeIter *iter, gpointer user)
@@ -677,7 +705,7 @@ static void cdf_uidstr(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTre
 		g_snprintf(buf, sizeof buf, user, uid);
 		g_object_set(renderer, "text", buf, NULL);
 	}
-	format_colors_row2(model, iter, renderer, TRUE);
+	format_colors_row(model, iter, renderer, TRUE, get_do_bg(col, user));
 }
 
 static void cdf_nlink(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTreeModel *model, GtkTreeIter *iter, gpointer user)
@@ -686,7 +714,7 @@ static void cdf_nlink(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTree
 
 	g_snprintf(buf, sizeof buf, "%u", dp_row_get_nlink(model, iter));
 	g_object_set(renderer, "text", buf, NULL);
-	format_colors_row2(model, iter, renderer, TRUE);
+	format_colors_row(model, iter, renderer, TRUE, get_do_bg(col, user));
 }
 
 static void cdf_uri(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTreeModel *model, GtkTreeIter *iter, gpointer user)
@@ -709,7 +737,7 @@ static void cdf_uri(GtkTreeViewColumn *col, GtkCellRenderer *renderer, GtkTreeMo
 		else
 			g_snprintf(buf, sizeof buf, "(failed)");
 		g_object_set(renderer, "text", buf, NULL);
-		format_colors_row2(model, iter, renderer, TRUE);
+		format_colors_row(model, iter, renderer, TRUE, get_do_bg(col, user));
 	}
 }
 
@@ -739,12 +767,12 @@ GtkTreeCellDataFunc dpf_get_cell_data_func(DirPane *dp, gint column, gpointer *u
 	case DPC_CHTIME:
 		*user = col_fmt->extra.ch_time.format;
 		return cdf_chtime;
-	case DPC_DEVMAJ:
-		*user = col_fmt->extra.devmaj.format;
-		return cdf_devmax;
 	case DPC_DEVICE:
 		*user = col_fmt->extra.device.format;
 		return cdf_device;
+	case DPC_DEVMAJ:
+		*user = col_fmt->extra.devmaj.format;
+		return cdf_devmax;
 	case DPC_DEVMIN:
 		*user = col_fmt->extra.devmin.format;
 		return cdf_devmin;
@@ -761,7 +789,7 @@ GtkTreeCellDataFunc dpf_get_cell_data_func(DirPane *dp, gint column, gpointer *u
 		*user = col_fmt->extra.mode.format;
 		return cdf_modenum;
 	case DPC_MODESTR:
-		*user = NULL;
+		*user = col_fmt->extra.mode.format;
 		return cdf_modestr;
 	case DPC_MTIME:
 		*user = col_fmt->extra.m_time.format;
@@ -791,4 +819,3 @@ GtkTreeCellDataFunc dpf_get_cell_data_func(DirPane *dp, gint column, gpointer *u
 	}
 	return NULL;
 }
-
