@@ -83,16 +83,15 @@ static void init_pane(DirPane *dp, gint index)
 static void evt_paned_notify_position(GObject *obj, GParamSpec *spec, gpointer user)
 {
 	MainInfo	*min = user;
-	gint		pos, pw, ph, np;
-	gdouble		old;
+	GdkWindow	*pwin;
+	gint		pos, np;
 
 	if(!dp_realized(min))
 		return;
 
 	pos = gtk_paned_get_position(GTK_PANED(obj));
-	gdk_window_get_size(min->gui->panes->window, &pw, &ph);
-	np = min->cfg.dp_paning.orientation == DPORIENT_HORIZ ? pw : ph;
-	old = min->cfg.dp_paning.value;
+	pwin = gtk_widget_get_window(min->gui->panes);
+	np = min->cfg.dp_paning.orientation == DPORIENT_HORIZ ? gdk_window_get_width(pwin) : gdk_window_get_height(pwin);
 	switch(min->cfg.dp_paning.mode)
 	{
 		case DPSPLIT_FREE:
@@ -120,7 +119,7 @@ static GtkWidget * build_dirpanes(MainInfo *min)
 
 	gui = min->gui;
 
-	gui->panes = min->cfg.dp_paning.orientation == DPORIENT_HORIZ ? gtk_hpaned_new() : gtk_vpaned_new();
+	gui->panes = gtk_paned_new(min->cfg.dp_paning.orientation == DPORIENT_HORIZ ? GTK_ORIENTATION_HORIZONTAL : GTK_ORIENTATION_VERTICAL);
 	init_pane(&gui->pane[0], 0);
 	pane = dp_build(min, &min->cfg.dp_format[0], &gui->pane[0]);
 	gtk_paned_add1(GTK_PANED(gui->panes), pane);
@@ -215,7 +214,7 @@ void rebuild_middle(MainInfo *min)
 	gtk_container_remove(GTK_CONTAINER(min->gui->panes), min->gui->pane[0].vbox);
 	gtk_container_remove(GTK_CONTAINER(min->gui->panes), min->gui->pane[1].vbox);
 	gtk_widget_destroy(min->gui->panes);
-	min->gui->panes = min->cfg.dp_paning.orientation == DPORIENT_HORIZ ? gtk_hpaned_new() : gtk_vpaned_new();
+	min->gui->panes = gtk_paned_new(min->cfg.dp_paning.orientation == DPORIENT_HORIZ ? GTK_ORIENTATION_HORIZONTAL : GTK_ORIENTATION_VERTICAL);
 	gtk_box_pack_start(GTK_BOX(min->gui->vbox), min->gui->panes, TRUE, TRUE, 0);
 	gtk_box_reorder_child(GTK_BOX(min->gui->vbox), min->gui->panes, 1);
 	if((left = dp_build(min, &min->cfg.dp_format[0], &min->gui->pane[0])) != NULL)
@@ -255,11 +254,12 @@ static GtkWidget * build_gui(MainInfo *min)
 	min->gui = g_malloc(sizeof *min->gui);
 	min->gui->window = NULL;
 	min->gui->window = win_window_open(min->cfg.wininfo, WIN_MAIN);
+	gtk_widget_set_name(min->gui->window, "gentoo");
 	g_object_set_data(G_OBJECT(min->gui->window), "user", min);
 	min->gui->sig_main_configure = g_signal_connect(G_OBJECT(min->gui->window), "configure_event", G_CALLBACK(evt_main_configure), min);
 	min->gui->sig_main_delete = g_signal_connect(G_OBJECT(min->gui->window), "delete_event", G_CALLBACK(evt_main_delete), min);
 	min->gui->pane[0].main = min->gui->pane[1].main = min;
-	min->gui->vbox = gtk_vbox_new(FALSE, 0);
+	min->gui->vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	min->gui->kbd_ctx = kbd_context_new(min);
 
 	top = build_top(min);
@@ -313,6 +313,7 @@ static void init_paths(CfgInfo *cfg)
 **		it anywhere as long as you given gentoo the path (in the config).
 ** 2001-08-12 -	The dot is now optional, but is given priority if found.
 ** 2011-07-24 -	Now we even prefer "gtkrc", without "gentoo", to match .config/gentoo location.
+** 2012-05-02 -	Ported to use GtkCssProvider API, for GTK+ 3.0.
 */
 static void load_gtk_rc(MainInfo *min)
 {
@@ -324,8 +325,18 @@ static void load_gtk_rc(MainInfo *min)
 	{
 		if((name = fut_locate(min->cfg.path.path[PTID_GTKRC]->str, names[i])) != NULL)
 		{
-			gtk_rc_parse(name);
-			break;
+			GtkCssProvider	*prov;
+
+			if((prov = gtk_css_provider_new()) != NULL)
+			{
+				if(gtk_css_provider_load_from_path(prov, name, NULL))
+				{
+					GdkScreen	*scr = gtk_widget_get_screen(min->gui->window);
+
+					gtk_style_context_add_provider_for_screen(scr, GTK_STYLE_PROVIDER(prov), GTK_STYLE_PROVIDER_PRIORITY_THEME);
+					break;
+				}
+			}
 		}
 	}
 }
@@ -425,39 +436,60 @@ static gint evt_idle_run(gpointer user)
 	return FALSE;		/* Causes the handler to be removed, once really is enough. */
 }
 
+static gint cmp_list_commands(gconstpointer a, gconstpointer b)
+{
+	return strcmp(a, b);
+}
+
+static void cb_list_commands(gpointer key, gpointer value, gpointer user)
+{
+	GList	**head = user;
+
+	*head = g_list_insert_sorted(*head, key, cmp_list_commands);
+}
+
+static void do_list_commands(MainInfo *min)
+{
+	GList	*cmdseq = NULL, *iter;
+
+	g_hash_table_foreach(min->cfg.commands.builtin, cb_list_commands, &cmdseq);
+	for(iter = cmdseq; iter != NULL; iter = g_list_next(iter))
+		printf("%s\n", (const gchar *) iter->data);
+}
+
 int main(int argc, char *argv[])
 {
 	static MainInfo	min = { NULL };
 
 	gboolean	show_version = FALSE;
-	gboolean	show_locale_info = FALSE;
 	gboolean	root_ok = FALSE;
-	gboolean	no_rc = FALSE, no_gtk_rc = FALSE, no_dir_history = FALSE;
+	gboolean	no_rc = FALSE, no_gtk_rc = FALSE, no_dir_history = FALSE, list_commands = FALSE;
 	gchar		*dir_left = NULL;
 	gchar		*dir_right = NULL;
+#if defined ENABLE_NLS
+	gboolean	show_locale_info = FALSE;
+	gint		i;
+#endif
 	GOptionEntry	option_entries[] = {
 	{ "version", 0, 0, G_OPTION_ARG_NONE, &show_version,  N_("Report the version to standard output, and exit"), NULL },
 #if defined ENABLE_NLS
 	{ "locale-info", 0, 0, G_OPTION_ARG_NONE, &show_locale_info, N_("Report internal locale details, and exit"), NULL },
 #endif
-	{ "root-ok", 0, 0, G_OPTION_ARG_NONE, &root_ok, N_("Allows gentoo to be run by the root user. Could be dangerous"), NULL },
-	{ "no-rc", 0, 0, G_OPTION_ARG_NONE, &no_rc, N_("Do not load the ~/.gentoorc configuration file; instead, use default values"), NULL },
-	{ "no-gtk-rc", 0, 0, G_OPTION_ARG_NONE, &no_gtk_rc, N_("Do not load the ~/.gentoogtkrc GTK+ configuration file; instead, use system defaults"), NULL },
-	{ "no-dir-history", 0, 0, G_OPTION_ARG_NONE, &no_dir_history, N_("Do not load the dirhistory file"), NULL },
+	{ "root-ok", 0, 0, G_OPTION_ARG_NONE, &root_ok, N_("Allows gentoo to be run by the root user. Could be dangerous!"), NULL },
+	{ "no-rc", 0, 0, G_OPTION_ARG_NONE, &no_rc, N_("Do not load the ~/.config/gentoo/gentoorc configuration file; instead, use default values"), NULL },
+	{ "no-gtk-rc", 0, 0, G_OPTION_ARG_NONE, &no_gtk_rc, N_("Do not load the ~/.config/gentoo/gtkrc GTK+ configuration file; instead, use system defaults"), NULL },
+	{ "no-dir-history", 0, 0, G_OPTION_ARG_NONE, &no_dir_history, N_("Do not load the ~/.config/gentoo/dirhistory file; instead, start with empty history"), NULL },
 	{ "run", 'r', 0, G_OPTION_ARG_STRING_ARRAY, &min.run_commands, N_("Run COMMAND, a gentoo command. Done before user interaction allowed, but after configuration "
 			      "file has been read in. Can be used many times to run several commands in sequence"), N_("COMMAND") },
 	{ "left", '1', 0, G_OPTION_ARG_STRING, &dir_left, N_("Use DIR as path for the left directory pane. Overrides default (and history)"), N_("DIR") },
 	{ "right", '2', 0, G_OPTION_ARG_STRING, &dir_right, N_("Use DIR as path for the right directory pane. Overrides default (and history)"), N_("DIR") },
+	{ "list-commands", 0, 0, G_OPTION_ARG_NONE, &list_commands, N_("Print a list of all built-in commands, and exit"), NULL },
 	{ NULL }
 	};
-	gint		i;
 	GtkWidget	*box;
 	GOptionContext	*context;
 	GError		*err = NULL;
 
-	/* Must be run very early, before we start using glib functionality too much. */
-	g_thread_init(NULL);
-
 #if defined ENABLE_NLS
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
@@ -504,7 +536,7 @@ int main(int argc, char *argv[])
 		printf("G_FILENAME_ENCODING=\"%s\"\n", enc);
 
 		g_get_filename_charsets(&fenc);
-		printf("Filename encoding: \"%s\"\n", (fenc != NULL && *fenc != NULL) ? fenc[0] : "(unknown)");
+		printf("Filename encoding: \"%s\"\n", (fenc != NULL && *fenc != NULL) ? *fenc : "(unknown)");
 
 		g_get_charset(&cset);
 		if(cset == NULL)
@@ -516,7 +548,7 @@ int main(int argc, char *argv[])
 
 	if(geteuid() == 0 && !root_ok)
 	{
-		fprintf(stderr, _("%s: To run as root, envoke with the '--root-ok' option\n"), argv[0]);
+		fprintf(stderr, _("%s: To allow running as root, use the '--root-ok' option\n"), argv[0]);
 		return EXIT_FAILURE;
 	}
 
@@ -526,6 +558,13 @@ int main(int argc, char *argv[])
 	min.cfg.flags = 0;
 	dpf_init_defaults(&min.cfg);
 	csq_init_commands(&min);
+
+	if(list_commands)
+	{
+		do_list_commands(&min);
+		return EXIT_SUCCESS;
+	}
+
 	min.cfg.menus = mnu_menuinfo_new_default(&min);
 	btn_buttoninfo_new_default(&min, &min.cfg.buttons);
 	min.cfg.buttonlayout = btl_buttonlayout_new();
@@ -547,13 +586,14 @@ int main(int argc, char *argv[])
 		cfg_load_config(&min);
 	if(!usr_init())
 		g_warning(_("Couldn't initialize userinfo module - username resolving won't work"));
-	if(!no_gtk_rc)
-		load_gtk_rc(&min);
 
 	if((box = build_gui(&min)) != NULL)
 	{
 		dp_initialize(min.gui->pane, sizeof min.gui->pane / sizeof *min.gui->pane);
 
+		if(!no_gtk_rc)
+			load_gtk_rc(&min);
+
 		fam_initialize(&min);
 
 		dlg_main_window_set(GTK_WINDOW(min.gui->window));
