@@ -180,12 +180,12 @@ typedef struct {
 } DC_Name;
 
 typedef struct {
-	SzUnit		unit;
-	guint		ticks : 1;
-	gchar		tick;
-	gint		digits;
-	gchar		dformat[DP_FORMAT_SIZE];	/* Hidden from user, not saved, built by config code. */
-	guint		dir_show_fs_size : 1;		/* Show filesystem size for directories? */
+	SzUnit	unit;
+	guint	ticks : 1;
+	gchar	tick;
+	gint	digits;
+	gchar	dformat[DP_FORMAT_SIZE];	/* Hidden from user, not saved, built by config code. */
+	guint	dir_show_fs_size : 1;		/* Show filesystem size for directories? */
 } DC_Size;
 
 typedef struct {
@@ -238,6 +238,7 @@ typedef struct {
 	gboolean	set_font;		/* Set to override GTK+'s default font. */
 	gchar		font_name[FONT_MAX];	/* Last-set font name, remains valid even when set_font == FALSE. */
 	gboolean	rubber_banding;		/* Set to enable GTK+'s "rubberbanding" selection. */
+	gboolean	rules;			/* Set to request GTK+'s "ruling", and disable file style-based background coloring. */
 } DPFormat;
 
 typedef enum { DPORIENT_HORIZ = 0, DPORIENT_VERT } DpOrient;
@@ -251,7 +252,7 @@ typedef struct {
 } DPPaning;
 
 typedef struct {
-	gboolean	select;			/* Rememeber selections? */
+	gboolean	select;			/* Remember selections? */
 	gboolean	save;			/* Save history lists on exit? */
 } DPHistory;
 
@@ -474,8 +475,6 @@ struct DirPane {
 	gint		last_row;		/* The last clicked row. */
 	gint		last_row2;		/* The second last clicked row. */
 	gint		dbclk_row;		/* The row that was double clicked, or -1. */
-	gint		focus_row;		/* The last row to be rendered focused, or -1. */
-	gint		old_focus_row;		/* Used to remember focus when pane is deactivated. */
 	guint		sig_sel_changed;	/* Signal for the "changed" event on the view's GtkTreeSelection. */
 	guint		sig_path_activate;	/* Signal for the "activate" event on path entry combo box. */
 	guint		sig_path_changed;	/* Signal for the "changed" event on the path entry combo box. */
