@@ -34,32 +34,6 @@ struct GuiHandlerGroup
 
 /* ----------------------------------------------------------------------------------------- */
 
-/* 1998-07-12 -	Moved this out of the dirpane config module and into this brand new GUI
-**		utilities module.
-*/
-GtkWidget * gui_arrow_button_new(GtkArrowType at)
-{
-	GtkWidget	*but, *arr;
-
-	but = gtk_button_new();
-	arr = gtk_arrow_new(at, GTK_SHADOW_OUT);
-	gtk_container_add(GTK_CONTAINER(but), arr);
-
-	return but;
-}
-
-/* 2010-06-05 -	Wrapper since I can't get gtk_button_new_from_stock() to behave the way I like. Weird. */
-GtkWidget * gui_stock_button_new(const gchar *stock_id)
-{
-	GtkWidget	*btn, *img;
-
-	btn = gtk_button_new();
-	img = gtk_image_new_from_stock(stock_id, GTK_ICON_SIZE_MENU);
-	gtk_button_set_image(GTK_BUTTON(btn), img);
-
-	return btn;
-}
-
 /* 1999-04-26 -	Create a button with a "details" icon on it. This is typically a magnifying
 **		class. This button is a graphical replacement for a button labeled "...".
 ** 2009-04-23 -	Happy ten year anniversary, gui_details_button_new()! W00t! Rewritten to use
@@ -68,7 +42,7 @@ GtkWidget * gui_stock_button_new(const gchar *stock_id)
 */
 GtkWidget * gui_details_button_new(void)
 {
-	return gui_stock_button_new(GTK_STOCK_FIND);
+	return gtk_button_new_from_icon_name("edit-find", GTK_ICON_SIZE_MENU);
 }
 
 /* ----------------------------------------------------------------------------------------- */
@@ -125,7 +99,7 @@ gpointer gui_progress_begin(const gchar *body, const gchar *button)
 {
 	static PBar	pbar;
 
-	pbar.vbox  = gtk_vbox_new(FALSE, 0);
+	pbar.vbox  = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	pbar.label = gtk_label_new(body);
 	gtk_box_pack_start(GTK_BOX(pbar.vbox), pbar.label, TRUE, TRUE, 0);
 	pbar.obj = gtk_label_new("");
@@ -321,6 +295,23 @@ void gui_set_main_title(MainInfo *min, const gchar *title)
 		win_window_set_title(min->gui->window, "gentoo");
 }
 
+/* 2013-06-15 -	Converts a modern RGBA color to an old-school one. */
+void gui_color_from_rgba(GdkColor *color, const GdkRGBA *rgba)
+{
+	color->red = 65535.0f * rgba->red;
+	color->green = 65535.0f * rgba->green;
+	color->blue = 65535.0f * rgba->blue;
+	color->pixel = 0;
+}
+
+void gui_rgba_from_color(GdkRGBA *rgba, const GdkColor *color)
+{
+	rgba->red   = color->red / 65535.;
+	rgba->green = color->green / 65535.;
+	rgba->blue  = color->blue / 65535.;
+	rgba->alpha = 1.0;
+}
+
 /* ----------------------------------------------------------------------------------------- */
 
 /* 2008-07-04 -	Flush pending GTK+ events.
