@@ -3,8 +3,6 @@
 **		functions which are handy when building various kinds of GUIs.
 */
 
-extern GtkWidget *	gui_arrow_button_new(GtkArrowType at);
-extern GtkWidget *	gui_stock_button_new(const gchar *stock_id);
 extern GtkWidget *	gui_details_button_new(void);
 extern GtkWidget *	gui_dialog_entry_new(void);
 extern GSList *		gui_radio_group_new(guint num, const gchar **label, GtkWidget *but[]);
@@ -25,5 +23,7 @@ extern void		gui_handler_group_block(const GuiHandlerGroup *g);
 extern void		gui_handler_group_unblock(const GuiHandlerGroup *g);
 
 extern void		gui_set_main_title(MainInfo *min, const gchar *title);
+extern void		gui_color_from_rgba(GdkColor *color, const GdkRGBA *rgba);
+extern void		gui_rgba_from_color(GdkRGBA *rgba, const GdkColor *color);
 
 extern void		gui_events_flush(void);
