@@ -141,7 +141,7 @@ static void evt_cell_renderer_editing_started(GtkCellRenderer *renderer, GtkCell
 		return;
 	if(ldi->editor != NULL)
 	{
-		gtk_entry_set_icon_from_stock(GTK_ENTRY(editable), GTK_ENTRY_ICON_SECONDARY, GTK_STOCK_FIND);
+		gtk_entry_set_icon_from_icon_name(GTK_ENTRY(editable), GTK_ENTRY_ICON_SECONDARY, "edit-find");
 		g_signal_connect(G_OBJECT(editable), "icon_press", G_CALLBACK(evt_cell_renderer_entry_icon_press), user);
 		g_snprintf(ldi->edit_path, sizeof ldi->edit_path, "%s", path);
 	}
@@ -208,7 +208,7 @@ void ldl_dialog_sync_new_full_wait(gchar *list, gsize max, gunichar separator, c
 	GtkTreeSelection	*sel;
 
 	ldi.store = gtk_list_store_new(1, G_TYPE_STRING);
-	vbox = gtk_vbox_new(FALSE, 0);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	ldi.view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(ldi.store));
 	cr = gtk_cell_renderer_text_new();
 	g_object_set(G_OBJECT(cr), "editable", TRUE, NULL);
@@ -224,11 +224,11 @@ void ldl_dialog_sync_new_full_wait(gchar *list, gsize max, gunichar separator, c
 	gtk_container_add(GTK_CONTAINER(scwin), ldi.view);
 	gtk_box_pack_start(GTK_BOX(vbox), scwin, TRUE, TRUE, 0);
 
-	hbox = gtk_hbox_new(FALSE, 0);
-	ldi.add = gtk_button_new_from_stock(GTK_STOCK_ADD);
+	hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
+	ldi.add = gtk_button_new_with_label(_("Add"));
 	g_signal_connect(G_OBJECT(ldi.add), "clicked", G_CALLBACK(evt_add_clicked), &ldi);
 	gtk_box_pack_start(GTK_BOX(hbox), ldi.add, TRUE, TRUE, 0);
-	ldi.delete = gtk_button_new_from_stock(GTK_STOCK_DELETE);
+	ldi.delete = gtk_button_new_with_label(_("Delete"));
 	g_signal_connect(G_OBJECT(ldi.delete), "clicked", G_CALLBACK(evt_delete_clicked), &ldi);
 	gtk_box_pack_start(GTK_BOX(hbox), ldi.delete, TRUE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
