@@ -316,7 +316,7 @@ static void evt_menu_parents_activate(GtkWidget *wid, gpointer user)
 
 	if(!GTK_IS_MENU_ITEM(wid))
 		return;
-	wid = GTK_BIN(wid)->child;
+	wid = gtk_bin_get_child(GTK_BIN(wid));
 	if(!GTK_IS_LABEL(wid))
 		return;
 	text = gtk_label_get_text(GTK_LABEL(wid));
