@@ -2,7 +2,7 @@
 ** Custom button widget, that supports having a secondary click-function associated with being
 ** clicked by the middle mouse button. Very much inspired by Directory Opus on the Amiga.
 **
-** Original GTK+ 1.2.x version written by Johan Hanson, re-written for GTK+ 2.0 by Emil Brink.
+** Original GTK+ 1.2.x version by Johan Hanson, re-written for GTK+ 2.0 and 3.0 by Emil Brink.
 */
 
 #include <stdio.h>
@@ -64,7 +64,7 @@ static void od_multibutton_set_page(GtkWidget *widget, guint index)
 	mb = OD_MULTIBUTTON(widget);
 
 	p  = mb->page[index].widget;
-	op = GTK_BIN(mb)->child;
+	op = gtk_bin_get_child(GTK_BIN(mb));
 
 	if(op != p)
 	{
@@ -75,8 +75,8 @@ static void od_multibutton_set_page(GtkWidget *widget, guint index)
 		}
 		if(p != NULL)
 		{
-			if(gtk_widget_get_state(widget) != gtk_widget_get_state(p))
-				gtk_widget_set_state(p, gtk_widget_get_state(widget));
+			if(gtk_widget_get_state_flags(widget) != gtk_widget_get_state_flags(p))
+				gtk_widget_set_state_flags(p, gtk_widget_get_state_flags(widget), TRUE);
 			gtk_widget_show(p);
 			if(gtk_widget_get_parent(p) != NULL)
 				gtk_widget_reparent(p, widget);
@@ -104,115 +104,78 @@ static void od_multibutton_page_size_calc(ODMultiButton *mb)
 
 		if((w = mb->page[i].widget) != NULL)
 		{
-			GtkRequisition	req;
+			GtkRequisition	req_min, req_max;
 
-			gtk_widget_size_request(w, &req);
-			mb->width  = MAX(mb->width, req.width);
-			mb->height = MAX(mb->height, req.height);
+			gtk_widget_get_preferred_size(w, &req_min, &req_max);
+			mb->width  = MAX(mb->width, req_min.width);
+			mb->height = MAX(mb->height, req_min.height);
 		}
 	}
 }
 
-static void od_multibutton_size_request(GtkWidget *widget, GtkRequisition *req)
+static void od_multibutton_get_preferred_width(GtkWidget *widget, gint *min_width, gint *max_width)
 {
 	ODMultiButton	*mb;
-	gint		focus_width, focus_pad;
+	gint		width;
 
 	g_return_if_fail(widget != NULL);
-	g_return_if_fail(req != NULL);
 	g_return_if_fail(OD_IS_MULTIBUTTON(widget));
 
 	mb = OD_MULTIBUTTON(widget);
-
-	req->width  = 2 * (gtk_container_get_border_width(GTK_CONTAINER(widget)) + 1 + gtk_widget_get_style(widget)->xthickness);
-	req->height = 2 * (gtk_container_get_border_width(GTK_CONTAINER(widget)) + 1 + gtk_widget_get_style(widget)->ythickness);
-
-	/* Included for completeness, ODMultiButton can't actually be focused. */
-	gtk_widget_style_get(GTK_WIDGET(widget), "focus-line-width", &focus_width, "focus-padding", &focus_pad, NULL);
-	req->width  += 2 * (focus_width + focus_pad);
-	req->height += 2 * (focus_width + focus_pad);
-
 	od_multibutton_page_size_calc(mb);
-	req->width  += mb->width;
-	req->height += mb->height;
+	width = mb->width;
+	
+	width += 2 * gtk_container_get_border_width(GTK_CONTAINER(widget));
+	/* FIXME: Not sure if we need to dig up CSS box model spacing here (margin/padding). */
+
+	if(min_width != NULL)
+		*min_width = width;
+	if(max_width != NULL)
+		*max_width = width;
 }
 
 /* Paint the "dog ear" that indicates second-mouse-button binding. This code is pretty much lifted
 ** from Johan Hanson's original ODEmilButton widget, although not a straight copy and paste.
 */
-static void od_multibutton_paint_dog_ear(GtkWidget *widget, GdkRectangle *area)
+static void od_multibutton_paint_dog_ear(GtkWidget *widget, cairo_t *cr, const GtkAllocation *alloc, gint bw)
 {
-	const GtkStateType	state = gtk_widget_get_state(widget);
+	const GtkStateFlags	sflags = gtk_widget_get_state_flags(widget);
 
-	if(gtk_widget_is_drawable(widget) && state != GTK_STATE_ACTIVE)
+	if(gtk_widget_is_drawable(widget) && sflags != GTK_STATE_FLAG_ACTIVE)
 	{
-		GtkStyle	*style = gtk_widget_get_style(widget);
-		const guint	EARSIZE = 5;
-		gint		x, y, w, h, bw;
-		GdkPoint	p[5];
+		const gint	EAR_SIZE = 5;
 
-		bw = gtk_container_get_border_width(GTK_CONTAINER(widget));
-		x = widget->allocation.x + bw;
-		y = widget->allocation.y + bw;
-		w = widget->allocation.width  - 2 * bw;
-		h = widget->allocation.height - 2 * bw;
-		p[0].x = x + w - (EARSIZE + 1);
-		p[0].y = y;
-		p[1].x = p[0].x;
-		p[1].y = y + EARSIZE;
-		p[2].x = x + w - 1;
-		p[2].y = p[1].y;
-		p[3].x = p[2].x;
-		p[3].y = y + (EARSIZE - 1);
-		p[4].x = x + w - EARSIZE;
-		p[4].y = y;
-
-		/* Now draw the ear, respecting state. */
-		if(state == GTK_STATE_INSENSITIVE)
-		{
-			guint	i;
-
-			gdk_gc_set_clip_rectangle(style->bg_gc[GTK_STATE_INSENSITIVE],    area);
-			gdk_gc_set_clip_rectangle(style->light_gc[GTK_STATE_INSENSITIVE], area);
-			gdk_gc_set_clip_rectangle(style->dark_gc[GTK_STATE_INSENSITIVE],  area);
-			for(i = 0; i < sizeof p / sizeof *p; i++)
-				p[i].y++;
-			gdk_draw_polygon(widget->window, style->bg_gc[GTK_STATE_INSENSITIVE],    TRUE,  p, sizeof p / sizeof *p);
-			gdk_draw_polygon(widget->window, style->light_gc[GTK_STATE_INSENSITIVE], FALSE, p, sizeof p / sizeof *p);
-			for(i = 0; i < sizeof p / sizeof *p; i++)
-			{
-				p[i].x--;
-				p[i].y--;
-			}
-			gdk_draw_polygon(widget->window, style->dark_gc[GTK_STATE_INSENSITIVE],  FALSE, p, sizeof p / sizeof *p);
-			gdk_gc_set_clip_rectangle(style->dark_gc[GTK_STATE_INSENSITIVE],  NULL);
-			gdk_gc_set_clip_rectangle(style->light_gc[GTK_STATE_INSENSITIVE], NULL);
-			gdk_gc_set_clip_rectangle(style->bg_gc[GTK_STATE_INSENSITIVE],    NULL);
-		}
-		else
-		{
-			gdk_gc_set_clip_rectangle(style->white_gc, area);
-			gdk_gc_set_clip_rectangle(style->black_gc, area);
-			gdk_draw_polygon(widget->window, style->white_gc, TRUE,  p, sizeof p / sizeof *p);
-			gdk_draw_polygon(widget->window, style->black_gc, FALSE, p, sizeof p / sizeof *p);
-			gdk_gc_set_clip_rectangle(style->black_gc, NULL);
-			gdk_gc_set_clip_rectangle(style->white_gc, NULL);
-		}
+		cairo_move_to(cr, alloc->width - 2 * bw - (EAR_SIZE + 1), 0);
+		cairo_rel_line_to(cr, 0, EAR_SIZE);
+		cairo_rel_line_to(cr, EAR_SIZE, 0);
+		cairo_close_path(cr);
+		cairo_set_line_width(cr, 0.5);
+		cairo_stroke(cr);
 	}
 }
 
-static void od_multibutton_paint_foreground(GtkWidget *widget, GdkRectangle *area)
+static void od_multibutton_paint_foreground(GtkWidget *widget, cairo_t *cr)
 {
 	g_return_if_fail(widget != NULL);
-	g_return_if_fail(area != NULL);
+	g_return_if_fail(cr != NULL);
 	g_return_if_fail(OD_IS_MULTIBUTTON(widget));
 
 	if(gtk_widget_is_drawable(widget))
 	{
-		ODMultiButton	*mb = OD_MULTIBUTTON(widget);
+		const ODMultiButton	*mb = OD_MULTIBUTTON(widget);
+		GtkAllocation		alloc;
+		const gint		bw = gtk_container_get_border_width(GTK_CONTAINER(widget));
 
+		gtk_widget_get_allocation(widget, &alloc);
 		if(mb->page[1].widget != NULL)
-			od_multibutton_paint_dog_ear(widget, area);
+			od_multibutton_paint_dog_ear(widget, cr, &alloc, bw);
+		if(mb->config && mb->config_down)
+		{
+			GtkStyleContext	*sctx = gtk_widget_get_style_context(widget);
+
+			/* Trivial border-rendering, instead of faked sunken-in. */
+			gtk_render_focus(sctx, cr, 0, 0, alloc.width - 2 * bw, alloc.height - 2 * bw);
+		}
 	}
 }
 
@@ -221,22 +184,16 @@ static void od_multibutton_paint_foreground(GtkWidget *widget, GdkRectangle *are
 ** of the drawing, taking care to fool it into doing what we want by poking a little. We also
 ** have our own foreground painting routine, for that oh-so-cute dog ear.
 */
-static gboolean od_multibutton_expose_event(GtkWidget *widget, GdkEventExpose *evt)
+static gboolean od_multibutton_draw(GtkWidget *widget, cairo_t *cr)
 {
 	g_return_val_if_fail(widget != NULL, FALSE);
-	g_return_val_if_fail(evt != NULL, FALSE);
+	g_return_val_if_fail(cr != NULL, FALSE);
 	g_return_val_if_fail(OD_IS_MULTIBUTTON(widget), FALSE);
 
-	if(evt->count > 0)
-		return FALSE;
-
-	/* In config mode, trick GTK+ into rendering depressed (sunken-in) border. A bit hackish. */
-	if(OD_MULTIBUTTON(widget)->config && OD_MULTIBUTTON(widget)->config_down)
-		GTK_BUTTON(widget)->depressed = TRUE;
-	/* Let superclass (GtkButton) handle the event, getting the children painted. */
-	GTK_WIDGET_CLASS(button_class)->expose_event(widget, evt);
-	/* Then paint our own modifying graphics on top. */
-	od_multibutton_paint_foreground(widget, &evt->area);
+	/* First let GtkButton draw, getting the basic button painted. */
+	GTK_WIDGET_CLASS(button_class)->draw(widget, cr);
+	/* Then paint our own modifying graphics on top, to get the dog ear and config border. */
+	od_multibutton_paint_foreground(widget, cr);
 
 	return FALSE;
 }
@@ -287,7 +244,7 @@ static gboolean od_multibutton_button_release_event(GtkWidget *widget, GdkEventB
 	{
 		if(evt->button > 1)
 			return FALSE;
-		OD_MULTIBUTTON(widget)->config_down ^= GTK_BUTTON(widget)->in_button;
+		OD_MULTIBUTTON(widget)->config_down ^= 1/*GTK_BUTTON(widget)->in_button*/;	/*FIXME?*/
 	}
 
 	/* GtkButton wants to believe button 1 was pressed, so let's make it so. */
@@ -308,8 +265,9 @@ static void od_multibutton_class_init(ODMultiButtonClass *mbc)
 	button_class = g_type_class_peek_parent(mbc);
 
 	/* Override methods. */
-	widget->size_request		= od_multibutton_size_request;
-	widget->expose_event		= od_multibutton_expose_event;
+	widget->draw			= od_multibutton_draw;
+	widget->get_preferred_width	= od_multibutton_get_preferred_width;
+
 	widget->button_press_event	= od_multibutton_button_press_event;
 	widget->button_release_event	= od_multibutton_button_release_event;
 }
@@ -319,13 +277,10 @@ static void od_multibutton_init(ODMultiButton *mb)
 {
 	guint	i;
 
-#if GTK_CHECK_VERSION(2, 2, 0)
 	gtk_widget_set_can_focus(GTK_WIDGET(mb), FALSE);
 	gtk_widget_set_can_default(GTK_WIDGET(mb), FALSE);
 	gtk_widget_set_receives_default(GTK_WIDGET(mb), FALSE);
-#else
-	GTK_WIDGET_UNSET_FLAGS(mb, GTK_CAN_FOCUS | GTK_CAN_DEFAULT | GTK_RECEIVES_DEFAULT);
-#endif
+
 	for(i = 0; i < sizeof mb->page / sizeof *mb->page; i++)
 	{
 		mb->page[i].widget = NULL;
@@ -361,9 +316,9 @@ void od_multibutton_set_config_selected(ODMultiButton *mb, gboolean selected)
 	g_return_if_fail(OD_IS_MULTIBUTTON(mb));
 	g_return_if_fail(OD_MULTIBUTTON(mb)->config);
 
-	if(selected != mb->config_down)
+	if(mb->config && selected != mb->config_down)
 	{
-		GTK_BUTTON(mb)->depressed = mb->config_down = selected;
+		mb->config_down = selected;
 		gtk_widget_queue_draw(GTK_WIDGET(mb));
 	}
 }
@@ -377,7 +332,7 @@ static void od_multibutton_remove_widget(ODMultiButton *mb, guint index)
 	g_return_if_fail(index > sizeof mb->page / sizeof *mb->page);
 
 	widget = mb->page[index].widget;
-	if(GTK_BIN(mb)->child == widget)
+	if(gtk_bin_get_child(GTK_BIN(mb)) == widget)
 		gtk_container_remove(GTK_CONTAINER(mb), widget);
 	else
 		g_object_unref(G_OBJECT(widget));
@@ -451,7 +406,7 @@ void od_multibutton_set_text(ODMultiButton *mb, guint index, const gchar *text,
 		if(GTK_IS_LABEL(w))
 		{
 			od_multibutton_reset_label(mb, index, GTK_LABEL(w), text, bg, fg);
-			if(GTK_WIDGET(mb)->parent)
+			if(gtk_widget_get_parent(GTK_WIDGET(mb)))
 				gtk_widget_queue_resize(GTK_WIDGET(mb));
 			if(gtk_widget_is_drawable(GTK_WIDGET(mb)))
 				gtk_widget_queue_draw(GTK_WIDGET(mb));
