@@ -52,7 +52,6 @@ static struct {
 
 	GtkWidget	*item;
 	GtkWidget	*item_pgs;
-	GtkWidget	*item_high;
 
 	GtkWidget	*eta;		/* Label for elapsed time, speed, and ETA. */
 } pgs_info = { 0U };
@@ -126,7 +125,7 @@ static gint evt_cancel_clicked(GtkWidget *wid, gpointer user)
 
 static gint evt_keypress(GtkWidget *wid, GdkEventKey *evt, gpointer user)
 {
-	if(evt->keyval == GDK_Escape)
+	if(evt->keyval == GDK_KEY_Escape)
 		g_cancellable_cancel(user);
 	return TRUE;
 }
@@ -169,10 +168,23 @@ void pgs_progress_begin(MainInfo *min, const gchar *op_name, guint32 flags)
 		pgs_info.body = gtk_label_new(op_name);
 		g_signal_connect(G_OBJECT(pgs_info.dlg), "delete_event", G_CALLBACK(evt_delete), pgs_info.cancel);
 		g_signal_connect(G_OBJECT(pgs_info.dlg), "key_press_event", G_CALLBACK(evt_keypress), pgs_info.cancel);
-		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(pgs_info.dlg)->vbox), pgs_info.body, FALSE, FALSE, 0);
+		gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(pgs_info.dlg))), pgs_info.body, FALSE, FALSE, 0);
+
+		if(pgs_info.show_item)
+		{
+			pgs_info.item = gtk_label_new("");
+			gtk_misc_set_alignment(GTK_MISC(pgs_info.item), 0.0f, 0.5f);
+			gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(pgs_info.dlg))), pgs_info.item, FALSE, FALSE, 5);
+			hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
+			pgs_info.item_pgs = gtk_progress_bar_new();
+			gtk_progress_bar_set_show_text(GTK_PROGRESS_BAR(pgs_info.item_pgs), TRUE);
+			gtk_box_pack_start(GTK_BOX(hbox), pgs_info.item_pgs, TRUE, TRUE, 0);
+			gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(pgs_info.item_pgs), 0.f);
+			gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(pgs_info.dlg))), hbox, FALSE, FALSE, 0);
+		}
 
-		hbox = gtk_hbox_new(FALSE, 0);
 		pgs_info.tot_pgs = gtk_progress_bar_new();
+		hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 		if(flags & PFLG_BUSY_MODE)
 		{
 			gtk_progress_bar_set_pulse_step(GTK_PROGRESS_BAR(pgs_info.tot_pgs), 1.f / 10.f);
@@ -186,40 +198,31 @@ void pgs_progress_begin(MainInfo *min, const gchar *op_name, guint32 flags)
 			g_snprintf(buf, sizeof buf, "%d", pgs_info.tot_num);
 			label = gtk_label_new(buf);
 			gtk_box_pack_end(GTK_BOX(hbox), label, FALSE, FALSE, 5);
+			gtk_progress_bar_set_show_text(GTK_PROGRESS_BAR(pgs_info.tot_pgs), TRUE);
 		}
 		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(pgs_info.tot_pgs), 0.f);
-		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(pgs_info.dlg)->vbox), hbox, FALSE, FALSE, 0);
+		gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(pgs_info.dlg))), hbox, FALSE, FALSE, 0);
 
 		if(pgs_info.show_byte)
 		{
 			sze_put_offset(size_buf, sizeof size_buf, pgs_info.byte_tot, SZE_AUTO, 3, ',');
 			g_snprintf(buf, sizeof buf, _("Total (%s)"), size_buf);
 			label = gtk_label_new(buf);
-			gtk_box_pack_start(GTK_BOX(GTK_DIALOG(pgs_info.dlg)->vbox), label, FALSE, FALSE, 0);
+			gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(pgs_info.dlg))), label, FALSE, FALSE, 0);
 			pgs_info.byte_pgs = gtk_progress_bar_new();
-			gtk_box_pack_start(GTK_BOX(GTK_DIALOG(pgs_info.dlg)->vbox), pgs_info.byte_pgs, TRUE, TRUE, 0);
-		}
-		if(pgs_info.show_item)
-		{
-			pgs_info.item = gtk_label_new("");
-			gtk_misc_set_alignment(GTK_MISC(pgs_info.item), 0.0f, 0.5f);
-			gtk_box_pack_start(GTK_BOX(GTK_DIALOG(pgs_info.dlg)->vbox), pgs_info.item, FALSE, FALSE, 5);
-			hbox = gtk_hbox_new(FALSE, 0);
-			pgs_info.item_pgs = gtk_progress_bar_new();
-			gtk_box_pack_start(GTK_BOX(hbox), pgs_info.item_pgs, TRUE, TRUE, 0);
-			gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(pgs_info.item_pgs), 0.f);
-			gtk_box_pack_start(GTK_BOX(GTK_DIALOG(pgs_info.dlg)->vbox), hbox, FALSE, FALSE, 0);
+			gtk_progress_bar_set_show_text(GTK_PROGRESS_BAR(pgs_info.byte_pgs), TRUE);
+			gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(pgs_info.dlg))), pgs_info.byte_pgs, TRUE, TRUE, 0);
 		}
 		if(!(flags & PFLG_BUSY_MODE))
 		{
 			pgs_info.eta = gtk_label_new("");
-			gtk_box_pack_start(GTK_BOX(GTK_DIALOG(pgs_info.dlg)->vbox), pgs_info.eta, FALSE, FALSE, 0);
+			gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(pgs_info.dlg))), pgs_info.eta, FALSE, FALSE, 0);
 		}
 		else
 			pgs_info.eta = NULL;
 		cancel = gtk_button_new_with_label(_("Cancel"));
 		g_signal_connect(G_OBJECT(cancel), "clicked", G_CALLBACK(evt_cancel_clicked), pgs_info.cancel);
-		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(pgs_info.dlg)->action_area), cancel, TRUE, TRUE, 0);
+		gtk_dialog_add_action_widget(GTK_DIALOG(pgs_info.dlg), cancel, GTK_RESPONSE_CANCEL);
 
 		g_get_current_time(&pgs_info.time_begin);
 	}
@@ -252,7 +255,12 @@ void pgs_progress_item_begin(MainInfo *min, const gchar *name, off_t size)
 	if(pgs_info.flags & PFLG_BUSY_MODE)
 		gtk_progress_bar_pulse(GTK_PROGRESS_BAR(pgs_info.tot_pgs));
 	else
+	{
+		gchar buf[32];
+		g_snprintf(buf, sizeof buf, "%u", pgs_info.tot_pos);
+		gtk_progress_bar_set_text(GTK_PROGRESS_BAR(pgs_info.tot_pgs), buf);
 		pgs_info.tot_pos++;
+	}
 	g_snprintf(pgs_info.name, sizeof pgs_info.name, "%s", name);
 	pgs_progress_item_resize(min, size);
 }
@@ -273,6 +281,33 @@ void pgs_progress_item_resize(MainInfo *min, off_t new_size)
 	gui_events_flush();
 }
 
+/* 2014-03-02 -	Format a bunch of seconds into a more friendly "time" format. */
+static void format_eta(gchar *buf, gsize buf_max, guint seconds)
+{
+	const guint	base[] = { 60, 60, 24, 0 };
+	guint		field[sizeof base / sizeof *base], i, conv = seconds;
+
+	for(i = 0; i < sizeof field / sizeof *field; ++i)
+	{
+		if(base[i])
+		{
+			field[i] = conv % base[i];
+			conv /= base[i];
+		}
+		else
+			field[i] = conv;
+	}
+
+	if(seconds < 60 * 60)	/* Enough with MM:SS? */
+		g_snprintf(buf, buf_max, "%02u:%02u", field[1], field[0]);
+	else if(seconds < 24 * 60 * 60)	/* Enough with HH:MM:SS? */
+		g_snprintf(buf, buf_max, "%02u:%02u:%02u", field[2], field[1], field[0]);
+	else if(field[3] == 1)		/* FIXME: Perhaps there's better I18N magic to use here for day(s). */
+		g_snprintf(buf, buf_max, _("%u day, %02u:%02u:%02u"), field[3], field[2], field[1], field[0]);
+	else
+		g_snprintf(buf, buf_max, _("%u days, %02u:%02u:%02u"), field[3], field[2], field[1], field[0]);
+}
+
 /* 1998-09-30 -	Indicate progress operating on the most recently registered item. Our
 **		current position in the item is <pos> bytes.
 */
@@ -285,10 +320,8 @@ PgsRes pgs_progress_item_update(MainInfo *min, off_t pos)
 		g_get_current_time(&time_now);
 		if(pgs_info.delayed)
 		{
-			gint	micro;
-
-			micro = 1E6 * (time_now.tv_sec  - pgs_info.time_begin.tv_sec) +
-				      (time_now.tv_usec - pgs_info.time_begin.tv_usec);
+			const guint	micro = 1E6 * (time_now.tv_sec  - pgs_info.time_begin.tv_sec) +
+						      (time_now.tv_usec - pgs_info.time_begin.tv_usec);
 
 			if(micro >= pgs_info.delay_time)
 				pgs_info.delayed = FALSE;
@@ -297,31 +330,42 @@ PgsRes pgs_progress_item_update(MainInfo *min, off_t pos)
 		{
 			if(pgs_info.eta != NULL)
 			{
-				gfloat	secs = msu_diff_timeval(&pgs_info.time_begin, &time_now);
+				const gfloat	secs = msu_diff_timeval(&pgs_info.time_begin, &time_now);
 				if((guint) secs != pgs_info.last_secs)
 				{
-					gchar	buf[64], spdbuf[16];
+					gchar	buf[64], spdbuf[16], etabuf[32];
 					gfloat	spd;
 					guint	eta;
 
 					pgs_info.last_secs = secs;
 					spd = (pgs_info.byte_pos + pos) / secs;
 					eta = (pgs_info.byte_tot - (pgs_info.byte_pos + pos)) / spd;
+					format_eta(etabuf, sizeof etabuf, eta);
 					sze_put_offset(spdbuf, sizeof spdbuf, spd, SZE_AUTO, 3, ',');
-					g_snprintf(buf, sizeof buf, _("Elapsed %02d:%02d  Speed %s/s  ETA %02u:%02u"),
+					g_snprintf(buf, sizeof buf, _("Elapsed %02d:%02d  Speed %s/s  ETA %s"),
 						pgs_info.last_secs / 60, pgs_info.last_secs % 60, spdbuf,
-						eta / 60, eta % 60);
+						etabuf);
 					gtk_label_set_text(GTK_LABEL(pgs_info.eta), buf);
 				}
 			}
 			gtk_widget_realize(pgs_info.dlg);
-			gdk_window_set_group(pgs_info.dlg->window, pgs_info.min->gui->window->window);
+			gdk_window_set_group(gtk_widget_get_window(pgs_info.dlg), gtk_widget_get_window(pgs_info.min->gui->window));
 			gtk_widget_show_all(pgs_info.dlg);
 		}
 		if(pgs_info.show_byte && pgs_info.byte_tot)
+		{
+			gchar tmp[32];
 			gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(pgs_info.byte_pgs), (gfloat) (pgs_info.byte_pos + pos) / pgs_info.byte_tot);
+			sze_put_offset(tmp, sizeof tmp, pgs_info.byte_pos + pos, SZE_AUTO, 3, ',');
+			gtk_progress_bar_set_text(GTK_PROGRESS_BAR(pgs_info.byte_pgs), tmp);
+		}
 		if(pgs_info.show_item && pgs_info.item_size)
+		{
+			gchar tmp[32];
 			gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(pgs_info.item_pgs), (gfloat) pos / pgs_info.item_size);
+			sze_put_offset(tmp, sizeof tmp, pos, SZE_AUTO, 3, ',');
+			gtk_progress_bar_set_text(GTK_PROGRESS_BAR(pgs_info.item_pgs), tmp);
+		}
 		gui_events_flush();
 		if(g_cancellable_is_cancelled(pgs_info.cancel))
 			return PGS_CANCEL;
