@@ -391,7 +391,7 @@ static gboolean find_callback(GNode *node, gpointer user)
 /* 1999-05-20 -	Find the named style in given styleinfo's style tree. NULL can be used as
 **		a shorthand for the name of the root style.
 */
-Style * stl_styleinfo_style_find(StyleInfo *si, const gchar *name)
+Style * stl_styleinfo_style_find(const StyleInfo *si, const gchar *name)
 {
 	struct style_find_data	sfd;
 
@@ -461,7 +461,7 @@ gboolean stl_styleinfo_style_siblings(StyleInfo *si, Style *stla, Style *stlb)
 /* 1999-05-26 -	Return the root style of <si>. This is actually the only reliable way of getting
 **		at it, since you don't know its name.
 */
-Style * stl_styleinfo_style_root(StyleInfo *si)
+Style * stl_styleinfo_style_root(const StyleInfo *si)
 {
 	if(si != NULL)
 	{
@@ -882,14 +882,18 @@ Style * stl_style_new_unique_name(StyleInfo *si)
 	if(si != NULL)
 	{
 		gchar	buf[STL_STYLE_NAME_SIZE];
-		guint	i = 0U;
 
-		do
+		g_strlcpy(buf, ("New Style"), sizeof buf);
+		if(stl_styleinfo_style_find(si, buf) != NULL)
 		{
-			g_snprintf(buf, sizeof buf, i == 0 ? _("New Style") : _("New Style %u"), i + 1);
-			i++;
-		} while((stl_styleinfo_style_find(si, buf) != NULL) && (i < 10000U));	/* This might fail. Big deal. */
+			guint	i = 1;
 
+			do
+			{
+				g_snprintf(buf, sizeof buf, _("New Style %u"), i + 1);
+				i++;
+			} while((stl_styleinfo_style_find(si, buf) != NULL) && (i < 10000U));	/* This might fail. Big deal. */
+		}
 		return stl_style_new(buf);
 	}
 	return NULL;
@@ -1057,8 +1061,6 @@ static void value_destroy(StlPVal *val)
 {
 	if(val != NULL)
 	{
-		if(val->type > 4)
-			abort();
 		switch(val->type)
 		{
 			case SPT_COLOR:
@@ -1194,7 +1196,7 @@ const gchar * stl_style_property_get_icon(const Style *stl, const gchar *propert
 }
 
 /* 1999-05-19 -	Get a (read-only, dammit) pointer to an action property definition. */
-const gchar * stl_style_property_get_action(Style *stl, const gchar *property)
+const gchar * stl_style_property_get_action(const Style *stl, const gchar *property)
 {
 	StlProp	*pr;
 
@@ -1211,7 +1213,7 @@ const gchar * stl_style_property_get_action(Style *stl, const gchar *property)
 /* 1999-05-24 -	Get the override status (which tells if the property is local or inherited) for
 **		a named property.
 */
-gboolean stl_style_property_get_override(Style *stl, const gchar *property)
+gboolean stl_style_property_get_override(const Style *stl, const gchar *property)
 {
 	StlProp	*pr;
 
@@ -1235,7 +1237,7 @@ static void get_action(gpointer key, gpointer value, gpointer user)
 **		until the next person sneezes, or someone calls property_set() or _remove().
 **		The list can be freed with a simple call to g_list_free().
 */
-GList * stl_style_property_get_actions(Style *stl)
+GList * stl_style_property_get_actions(const Style *stl)
 {
 	GList	*list = NULL;
 
