@@ -50,14 +50,14 @@ void		stl_styleinfo_dump(StyleInfo *si);
 
 void		stl_styleinfo_style_add(StyleInfo *si, Style *parent, Style *stl);
 void		stl_styleinfo_style_remove(StyleInfo *si, Style *stl);
-Style *		stl_styleinfo_style_find(StyleInfo *si, const gchar *name);
+Style *		stl_styleinfo_style_find(const StyleInfo *si, const gchar *name);
 Style *		stl_styleinfo_style_get_parent(StyleInfo *si, Style *stl);
 void		stl_styleinfo_style_set_parent(StyleInfo *si, Style *stl, Style *new_parent);
 Style *		stl_styleinfo_style_get_parent(StyleInfo *si, Style *stl);
 
 gboolean	stl_styleinfo_style_siblings(StyleInfo *si, Style *stla, Style *stlb);
 
-Style *		stl_styleinfo_style_root(StyleInfo *si);
+Style *		stl_styleinfo_style_root(const StyleInfo *si);
 gboolean	stl_styleinfo_style_has_children(StyleInfo *si, Style *stl);
 GList *		stl_styleinfo_style_get_children(StyleInfo *si, Style *stl, gboolean include_root);
 
@@ -89,9 +89,9 @@ void		stl_style_property_set_icon(Style *stl, const gchar *property, const gchar
 void		stl_style_property_set_action(Style *stl, const gchar *property, const gchar *value);
 const GdkColor*	stl_style_property_get_color(const Style *stl, const gchar *property);
 const gchar *	stl_style_property_get_icon(const Style *stl, const gchar *property);
-const gchar *	stl_style_property_get_action(Style *stl, const gchar *property);
-gboolean	stl_style_property_get_override(Style *stl, const gchar *property);
-GList *		stl_style_property_get_actions(Style *stl);
+const gchar *	stl_style_property_get_action(const Style *stl, const gchar *property);
+gboolean	stl_style_property_get_override(const Style *stl, const gchar *property);
+GList *		stl_style_property_get_actions(const Style *stl);
 gboolean	stl_style_property_has_action(const Style *stl, const gchar *action);
 
 gboolean	stl_style_property_overrides(Style *stl);
