@@ -116,6 +116,18 @@ static void really_destroy(GtkWidget *wid)
 	}
 }
 
+/* 2012-12-27 -	Helper function to move the cursor to the start of the buffer. This is handy after
+**		loading, since all the append()s tend to leave the cursor at the end, while the
+**		visible part is the first page. This causes weirdness when moving.
+*/
+static void cursor_to_start(TxvInfo *txi)
+{
+	GtkTextIter	iter;
+
+	gtk_text_buffer_get_start_iter(GTK_TEXT_BUFFER(txi->buffer), &iter);
+	gtk_text_buffer_place_cursor(GTK_TEXT_BUFFER(txi->buffer), &iter);
+}
+
 /* 1998-05-19 -	This simplistic thing causes the textviewing window to close when needed. */
 static gint evt_delete(GtkWidget *wid, GdkEvent *evt, gpointer user)
 {
@@ -213,8 +225,8 @@ static void sink_text(const gchar *text, gsize length, gpointer user)
 */
 gint txv_text_load(GtkWidget *wid, GFile *file, gsize buf_size, const gchar *encoding, GError **err)
 {
-	GFileInputStream	*is;
 	TxvInfo			*txi;
+	GFileInputStream	*is;
 
 	if((wid == NULL) || ((txi = g_object_get_data(G_OBJECT(wid), "user")) == NULL) || (file == NULL))
 		return 0;
@@ -237,6 +249,7 @@ gint txv_text_load(GtkWidget *wid, GFile *file, gsize buf_size, const gchar *enc
 		convstream_destroy(cs);
 		g_free(buf);
 		g_object_unref(is);
+		cursor_to_start(txi);
 	}
 	return *err == NULL;
 }
@@ -321,6 +334,7 @@ gint txv_text_load_hex(GtkWidget *win, GFile *file, GError **err)
 			offset += chunk;
 		}
 		g_object_unref(is);
+		cursor_to_start(txi);
 	}
 
 	if((tag = gtk_text_tag_new("fixed")) != NULL)
@@ -336,7 +350,6 @@ gint txv_text_load_hex(GtkWidget *win, GFile *file, GError **err)
 		gtk_text_buffer_apply_tag(GTK_TEXT_BUFFER(txi->buffer), tag, &start, &end);
 		g_object_unref(G_OBJECT(tag));
 	}
-
 	return *err == NULL;
 }
 
@@ -364,14 +377,14 @@ static gint evt_keypress(GtkWidget *wid, GdkEventKey *evt, gpointer user)
 {
 	TxvInfo	*txi = g_object_get_data(G_OBJECT(wid), "user");
 
-	if(evt->keyval == GDK_Escape || evt->keyval == GDK_Left)
+	if(evt->keyval == GDK_KEY_Escape || evt->keyval == GDK_KEY_Left)
 		really_destroy(wid);
-	else if(evt->keyval == GDK_End)
+	else if(evt->keyval == GDK_KEY_End)
 	{
 		GtkAdjustment	*adj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(txi->scwin));
-		gtk_adjustment_set_value(adj, adj->upper);
+		gtk_adjustment_set_value(adj, gtk_adjustment_get_upper(adj));
 	}
-	else if(evt->keyval == GDK_F3)
+	else if(evt->keyval == GDK_KEY_F3)
 		do_search_repeat(txi);
 	else
 		return FALSE;
@@ -389,10 +402,13 @@ static gint evt_pos_changed(GtkAdjustment *adj, gpointer user)
 	gfloat		p;
 	GtkTextIter	iter;
 	gint		bx, by;
+	const float	value = gtk_adjustment_get_value(adj);
+	const float	upper = gtk_adjustment_get_upper(adj);
+	const float	page_size = gtk_adjustment_get_page_size(adj);
 
-	if(adj->upper - adj->page_size > 0.0f)
+	if(upper - page_size > 0.0f)
 	{
-		p = 100 * (adj->value / (adj->upper - adj->page_size));
+		p = 100 * (value / (upper - page_size));
 		if(p > 100.0)
 			p = 100.0;
 	}
@@ -451,7 +467,7 @@ static void evt_goto_clicked(GtkWidget *wid, gpointer user)
 	TxvInfo		*txi = user;
 	GtkAdjustment	*adj;
 
-	goto_dialog.vbox  = gtk_vbox_new(FALSE, 0);
+	goto_dialog.vbox  = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	goto_dialog.label = gtk_label_new(_("Enter Line Number or Percentage:"));
 	gtk_box_pack_start(GTK_BOX(goto_dialog.vbox), goto_dialog.label, FALSE, FALSE, 0);
 	goto_dialog.entry = gui_dialog_entry_new();
@@ -592,7 +608,7 @@ static void evt_search_clicked(GtkWidget *wid, gpointer user)
 {
 	TxvInfo	*txi = user;
 
-	srch_dialog.vbox  = gtk_vbox_new(FALSE, 0);
+	srch_dialog.vbox  = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	srch_dialog.label = gtk_label_new(_("Enter Text to Search For (RE)"));
 	gtk_box_pack_start(GTK_BOX(srch_dialog.vbox), srch_dialog.label, FALSE, FALSE, 0);
 	srch_dialog.entry = gtk_entry_new();
@@ -600,7 +616,7 @@ static void evt_search_clicked(GtkWidget *wid, gpointer user)
 	gtk_entry_set_text(GTK_ENTRY(srch_dialog.entry), last_search.re);
 	gtk_editable_select_region(GTK_EDITABLE(srch_dialog.entry), 0, -1);
 	gtk_box_pack_start(GTK_BOX(srch_dialog.vbox), srch_dialog.entry, FALSE, FALSE, 0);
-	srch_dialog.hbox  = gtk_hbox_new(FALSE, 0);
+	srch_dialog.hbox  = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	srch_dialog.fnocase = gtk_check_button_new_with_label(_("Ignore Case?"));
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(srch_dialog.fnocase), last_search.no_case);
 	gtk_box_pack_start(GTK_BOX(srch_dialog.hbox), srch_dialog.fnocase, TRUE, TRUE, 0);
@@ -687,7 +703,7 @@ GtkWidget * txv_open(MainInfo *min, const gchar *label)
 	txv_connect_delete(txi->win, NULL, NULL);
 	txv_connect_keypress(txi->win, NULL, NULL);
 
-	vbox = gtk_vbox_new(FALSE, 0);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 
 	txi->buffer = G_OBJECT(gtk_text_buffer_new(NULL));
 	txi->text = gtk_text_view_new_with_buffer(GTK_TEXT_BUFFER(txi->buffer));
@@ -702,7 +718,7 @@ GtkWidget * txv_open(MainInfo *min, const gchar *label)
 
 	gtk_box_pack_start(GTK_BOX(vbox), txi->scwin, TRUE, TRUE, 0);
 
-	txi->bhbox = gtk_hbox_new(FALSE, 0);
+	txi->bhbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 	if(label != NULL)
 		txi->blabel = gtk_label_new(label);
 	else
@@ -717,37 +733,37 @@ GtkWidget * txv_open(MainInfo *min, const gchar *label)
 
 	accel = gtk_accel_group_new();
 
-	btn = gui_stock_button_new(GTK_STOCK_GO_UP);
+	btn = gtk_button_new_from_icon_name("go-up", GTK_ICON_SIZE_MENU);
 	g_object_set_data(G_OBJECT(btn), "user", GINT_TO_POINTER(MVE_UP));
 	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(evt_move_clicked), txi);
 	gtk_box_pack_start(GTK_BOX(txi->bhbox), btn, FALSE, FALSE, 0);
-	btn = gui_stock_button_new(GTK_STOCK_GO_DOWN);
+	btn = gtk_button_new_from_icon_name("go-down", GTK_ICON_SIZE_MENU);
 	g_object_set_data(G_OBJECT(btn), "user", GINT_TO_POINTER(MVE_DOWN));
 	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(evt_move_clicked), txi);
 	gtk_box_pack_start(GTK_BOX(txi->bhbox), btn, FALSE, FALSE, 0);
-	btn = gui_stock_button_new(GTK_STOCK_GOTO_TOP);
+	btn = gtk_button_new_from_icon_name("go-top", GTK_ICON_SIZE_MENU);
 	g_object_set_data(G_OBJECT(btn), "user", GINT_TO_POINTER(MVE_TOP));
 	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(evt_move_clicked), txi);
-	gtk_widget_add_accelerator(btn, "clicked", accel, GDK_Home, 0, 0);
+	gtk_widget_add_accelerator(btn, "clicked", accel, GDK_KEY_Home, 0, 0);
 	gtk_box_pack_start(GTK_BOX(txi->bhbox), btn, FALSE, FALSE, 0);
-	btn = gui_stock_button_new(GTK_STOCK_GOTO_BOTTOM);
+	btn = gtk_button_new_from_icon_name("go-bottom", GTK_ICON_SIZE_MENU);
 	g_object_set_data(G_OBJECT(btn), "user", GINT_TO_POINTER(MVE_BOTTOM));
 	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(evt_move_clicked), txi);
-	gtk_widget_add_accelerator(btn, "clicked", accel, GDK_End, 0, 0);
+	gtk_widget_add_accelerator(btn, "clicked", accel, GDK_KEY_End, 0, 0);
 	gtk_box_pack_start(GTK_BOX(txi->bhbox), btn, FALSE, FALSE, 0);
 
-	bhbox = gtk_hbox_new(FALSE, 0);
-	btn = gui_stock_button_new(GTK_STOCK_JUMP_TO);
+	bhbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
+	btn = gtk_button_new_from_icon_name("go-jump", GTK_ICON_SIZE_MENU);
 	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(evt_goto_clicked), txi);
 	gtk_box_pack_start(GTK_BOX(bhbox), btn, FALSE, FALSE, 0);
 
-	btn = gui_stock_button_new(GTK_STOCK_FIND);
-	gtk_widget_add_accelerator(btn, "clicked", accel, GDK_F, GDK_CONTROL_MASK, 0);
+	btn = gui_details_button_new();
+	gtk_widget_add_accelerator(btn, "clicked", accel, GDK_KEY_F, GDK_CONTROL_MASK, 0);
 	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(evt_search_clicked), txi);
 	gtk_box_pack_start(GTK_BOX(bhbox), btn, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(txi->bhbox), bhbox, FALSE, FALSE, 5);
 
-	btn = gui_stock_button_new(GTK_STOCK_CLOSE);
+	btn = gtk_button_new_from_icon_name("window-close", GTK_ICON_SIZE_MENU);
 	g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(evt_close_clicked), txi);
 	gtk_box_pack_start(GTK_BOX(txi->bhbox), btn, FALSE, TRUE, 0);
 	gtk_widget_set_sensitive(txi->bhbox, FALSE);
