@@ -257,7 +257,7 @@ static gboolean check_type(DirPane *dp, const DirRow2 *row, FType *type, const g
 	if(type->flags & FTFL_REQSUFFIX)
 	{
 		tries++;
-		hits += stu_has_suffix(name, type->suffix);
+		hits += g_str_has_suffix(name, type->suffix);
 	}
 	if(type->flags & FTFL_NAMEMATCH)
 	{
