@@ -21,9 +21,9 @@ extern GList *	typ_type_set_style(GList *list, FType *type, StyleInfo *si, const
 extern GList *	typ_type_set_priority(GList *list, FType *type, guint16 prio);
 extern GList *	typ_type_move(GList *list, FType *type, gint delta);
 
-extern void	typ_identify_begin(MainInfo *min);
-extern void	typ_identify(MainInfo *min, DirRow *line);
-extern void	typ_identify_end(MainInfo *min, const gchar *path);
+extern void	typ_identify_begin(DirPane *dp);
+extern void	typ_identify(DirPane *dp, const DirRow2 *row);
+extern gboolean	typ_identify_end(DirPane *dp);
 
 extern void	typ_init(CfgInfo *cfg);
 
