@@ -153,6 +153,22 @@ static void scan_gid(void)
 
 /* ----------------------------------------------------------------------------------------- */
 
+/* 2010-02-11 -	Returns the current user's home directory. Implemented according to the
+**		guidelines in the glib documentation. Does never return NULL or an empty'
+**		string, defaults to "/" if all else fails. Prefers $HOME over passwd.
+*/
+const gchar * usr_get_home(void)
+{
+	const char *home = g_getenv ("HOME");
+	if(!home)
+		home = g_get_home_dir();
+	if(!home || *home == '\0')
+		return "/";
+	return home;
+}
+
+/* ----------------------------------------------------------------------------------------- */
+
 /* 1998-05-29 -	Rewritten & renamed. This routine looks up the name of the user with the given
 **		<uid>. Returns pointer to string, or NULL if the user is unknown.
 */
@@ -204,7 +220,7 @@ const gchar * usr_lookup_uhome(const gchar *uname)
 		if((uname = usr_lookup_uname(geteuid())) != NULL)
 			return g_hash_table_lookup(the_usr.dict_uhome, uname);
 	}
-	return g_hash_table_lookup(the_usr.dict_uhome, uname);
+	return usr_get_home();
 }
 
 /* 1998-05-29 -	Rewritten & renamed. This routine looks up the name of the group with the given
