@@ -91,11 +91,9 @@ WinInfo * win_wininfo_new_default(MainInfo *min)
 	win_window_size_update_set(wi, WIN_TEXTVIEW, TRUE);
 
 	win_window_new(wi, WIN_INFO, WIN_TYPE_SIMPLE_DIALOG, FALSE, "Info", "info");
-	win_window_pos_grab_set(wi, WIN_INFO, TRUE);
 	win_window_size_set(wi, WIN_INFO, 320, 480);
 	win_window_size_use_set(wi, WIN_INFO, TRUE);
 	win_window_size_update_set(wi, WIN_INFO, TRUE);
-	win_window_size_grab_set(wi, WIN_INFO, TRUE);
 
 	return wi;
 }
@@ -178,7 +176,7 @@ static void evt_pos_grab_clicked(GtkWidget *wid, gpointer user)
 	WinDef *wd = user;
 
 	min = g_object_get_data(G_OBJECT(wid), "min");
-	gdk_window_get_position(min->gui->window->window, &wd->x, &wd->y);
+	gdk_window_get_position(gtk_widget_get_window(min->gui->window), &wd->x, &wd->y);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(g_object_get_data(G_OBJECT(wid), "spin1")), wd->x);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(g_object_get_data(G_OBJECT(wid), "spin2")), wd->y);
 	*(gboolean *) g_object_get_data(G_OBJECT(wid), "modified") = TRUE;
@@ -191,7 +189,8 @@ static void evt_size_grab_clicked(GtkWidget *wid, gpointer user)
 	WinDef *wd = user;
 
 	min = g_object_get_data(G_OBJECT(wid), "min");
-	gdk_window_get_size(min->gui->window->window, &wd->width, &wd->height);
+	wd->width  = gdk_window_get_width(gtk_widget_get_window(min->gui->window));
+	wd->height = gdk_window_get_height(gtk_widget_get_window(min->gui->window));
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(g_object_get_data(G_OBJECT(wid), "spin1")), wd->width);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(g_object_get_data(G_OBJECT(wid), "spin2")), wd->height);
 	*(gboolean *) g_object_get_data(G_OBJECT(wid), "modified") = TRUE;
@@ -202,12 +201,12 @@ static GtkWidget * subframe_build(const WinInfo *wi, WinDef *win, gint type, gbo
 	const gchar *ltext[] =
 	{ N_("Position"), N_("Size") }, *etext[] =
 	{ N_("X"), N_("Y"), N_("Width"), N_("Height") };
-	GtkWidget *label, *frame, *cbtn, *vbox, *table, *spin1, *spin2;
-	GtkObject *adj;
-	gint val1 = 0, val2 = 0;
+	GtkWidget	*label, *frame, *cbtn, *vbox, *grid, *spin1, *spin2;
+	GtkAdjustment	*adj;
+	gint		val1 = 0, val2 = 0;
 
 	frame = gtk_frame_new(_(ltext[type]));
-	vbox = gtk_vbox_new(FALSE, 0);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 	cbtn = gtk_check_button_new_with_label(_("Set on Open?"));
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(cbtn), (type == SUBFRAME_POSITION) ? win_window_pos_use_get(wi, win->id) : win_window_size_use_get(wi, win->id));
 	g_object_set_data(G_OBJECT(cbtn), "modified", modified);
@@ -220,7 +219,7 @@ static GtkWidget * subframe_build(const WinInfo *wi, WinDef *win, gint type, gbo
 	g_signal_connect(G_OBJECT(cbtn), "clicked", G_CALLBACK(evt_boolean_clicked), (type == SUBFRAME_POSITION) ? &win->pos_update : &win->size_update);
 	gtk_box_pack_start(GTK_BOX(vbox), cbtn, FALSE, FALSE, 0);
 
-	table = gtk_table_new(2, 3, FALSE);
+	grid = gtk_grid_new();
 
 	if(type == SUBFRAME_POSITION)
 		win_window_pos_get(wi, win->id, &val1, &val2);
@@ -228,19 +227,21 @@ static GtkWidget * subframe_build(const WinInfo *wi, WinDef *win, gint type, gbo
 		win_window_size_get(wi, win->id, &val1, &val2);
 
 	label = gtk_label_new(_(etext[2 * type]));
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, 0, 0, 0, 0);
+	gtk_grid_attach(GTK_GRID(grid), label, 0, 0, 1, 1);
 	adj = gtk_adjustment_new(val1, -1.0, 65535.0, 1.0, 25.0, 0.0);
 	spin1 = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.0, 0.0);
 	g_object_set_data(G_OBJECT(adj), "modified", modified);
 	g_signal_connect(G_OBJECT(adj), "value_changed", G_CALLBACK(evt_integer_changed), type == SUBFRAME_POSITION ? &win->x : &win->width);
-	gtk_table_attach(GTK_TABLE(table), spin1, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+	gtk_widget_set_hexpand(spin1, TRUE);
+	gtk_widget_set_halign(spin1, GTK_ALIGN_FILL);
+	gtk_grid_attach(GTK_GRID(grid), spin1, 1, 0, 1, 1);
 	label = gtk_label_new(_(etext[2 * type + 1]));
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2, 0, 0, 0, 0);
+	gtk_grid_attach(GTK_GRID(grid), label, 0, 1, 1, 1);
 	adj = gtk_adjustment_new(val2, -1.0, 65535.0, 1.0, 25.0, 0.0);
 	spin2 = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.0, 0.0);
 	g_object_set_data(G_OBJECT(adj), "modified", modified);
 	g_signal_connect(G_OBJECT(adj), "value_changed", G_CALLBACK(evt_integer_changed), type == SUBFRAME_POSITION ? &win->y : &win->height);
-	gtk_table_attach(GTK_TABLE(table), spin2, 1, 2, 1, 2, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+	gtk_grid_attach(GTK_GRID(grid), spin2, 1, 1, 1, 1);
 	if((type == SUBFRAME_POSITION && win->pos_grab) || (type == SUBFRAME_SIZE && win->size_grab))
 	{
 		GtkWidget *grab;
@@ -252,9 +253,9 @@ static GtkWidget * subframe_build(const WinInfo *wi, WinDef *win, gint type, gbo
 		g_object_set_data(G_OBJECT(grab), "spin2", spin2);
 		g_object_set_data(G_OBJECT(grab), "modified", modified);
 		g_signal_connect(G_OBJECT(grab), "clicked", type == SUBFRAME_POSITION ? G_CALLBACK(evt_pos_grab_clicked) : G_CALLBACK(evt_size_grab_clicked), win);
-		gtk_table_attach(GTK_TABLE(table), grab, 2, 3, 0, 2, 0, 0, 0, 0);
+		gtk_grid_attach(GTK_GRID(grid), grab, 2, 0, 1, 2);
 	}
-	gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), grid, FALSE, FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(frame), vbox);
 
 	return frame;
@@ -267,7 +268,7 @@ GtkWidget * win_wininfo_build(const WinInfo *wi, gboolean *modified)
 	gchar tmp[64];
 	WinDef *win;
 
-	vbox = gtk_vbox_new(FALSE, 0);
+	vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
 
 	for(iter = wi->windows; iter != NULL; iter = g_list_next(iter))
 	{
@@ -275,7 +276,7 @@ GtkWidget * win_wininfo_build(const WinInfo *wi, gboolean *modified)
 
 		g_snprintf(tmp, sizeof tmp, "%c%s", toupper(*win->label), win->label + 1);
 		frame = gtk_frame_new(tmp);
-		hbox = gtk_hbox_new(FALSE, 0);
+		hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
 		sframe = subframe_build(wi, win, SUBFRAME_POSITION, modified);
 		gtk_box_pack_start(GTK_BOX(hbox), sframe, TRUE, TRUE, 5);
 		sframe = subframe_build(wi, win, SUBFRAME_SIZE, modified);
@@ -662,13 +663,8 @@ void win_window_show(GtkWidget *window)
 	WinInfo	*wi;
 	WinDef	*wd;
 
-#if GTK_CHECK_VERSION(2, 20, 0)
 	if(gtk_widget_get_visible(window))
 		return;
-#else
-	if(GTK_WIDGET_VISIBLE(window))
-		return;
-#endif
 
 	wi = g_object_get_data(G_OBJECT(window), "wininfo");
 	wd = g_object_get_data(G_OBJECT(window), "windef");
@@ -736,12 +732,13 @@ gboolean win_window_update(GtkWidget *win)
 
 		if(wd->pos_update)
 		{
-			gdk_window_get_root_origin(win->window, &ta, &tb);
+			gdk_window_get_root_origin(gtk_widget_get_window(win), &ta, &tb);
 			ret |= update_pair(&wd->x, &wd->y, ta, tb);
 		}
 		if(wd->size_update)
 		{
-			gdk_window_get_size(win->window, &ta, &tb);
+			ta = gdk_window_get_width(gtk_widget_get_window(win));
+			tb = gdk_window_get_height(gtk_widget_get_window(win));
 			ret |= update_pair(&wd->width, &wd->height, ta, tb);
 		}
 		return ret;
